"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const express_1 = tslib_1.__importDefault(require("express"));
const morgan = require("morgan");
const Logger_1 = tslib_1.__importDefault(require("../common/Logger"));
const AccountSASAuthenticator_1 = tslib_1.__importDefault(require("./authentication/AccountSASAuthenticator"));
const AuthenticationMiddlewareFactory_1 = tslib_1.__importDefault(require("./authentication/AuthenticationMiddlewareFactory"));
const BlobSASAuthenticator_1 = tslib_1.__importDefault(require("./authentication/BlobSASAuthenticator"));
const BlobSharedKeyAuthenticator_1 = tslib_1.__importDefault(require("./authentication/BlobSharedKeyAuthenticator"));
const PublicAccessAuthenticator_1 = tslib_1.__importDefault(require("./authentication/PublicAccessAuthenticator"));
const blobStorageContext_middleware_1 = tslib_1.__importDefault(require("./context/blobStorageContext.middleware"));
const ExpressMiddlewareFactory_1 = tslib_1.__importDefault(require("./generated/ExpressMiddlewareFactory"));
const AppendBlobHandler_1 = tslib_1.__importDefault(require("./handlers/AppendBlobHandler"));
const BlobHandler_1 = tslib_1.__importDefault(require("./handlers/BlobHandler"));
const BlockBlobHandler_1 = tslib_1.__importDefault(require("./handlers/BlockBlobHandler"));
const ContainerHandler_1 = tslib_1.__importDefault(require("./handlers/ContainerHandler"));
const PageBlobHandler_1 = tslib_1.__importDefault(require("./handlers/PageBlobHandler"));
const PageBlobRangesManager_1 = tslib_1.__importDefault(require("./handlers/PageBlobRangesManager"));
const ServiceHandler_1 = tslib_1.__importDefault(require("./handlers/ServiceHandler"));
const constants_1 = require("./utils/constants");
/**
 * Default RequestListenerFactory based on express framework.
 *
 * When creating other server implementations, such as based on Koa. Should also create a NEW
 * corresponding BlobKoaRequestListenerFactory class by extending IRequestListenerFactory.
 *
 * @export
 * @class BlobRequestListenerFactory
 * @implements {IRequestListenerFactory}
 */
class BlobRequestListenerFactory {
    constructor(dataStore, accountDataStore, enableAccessLog) {
        this.dataStore = dataStore;
        this.accountDataStore = accountDataStore;
        this.enableAccessLog = enableAccessLog;
    }
    createRequestListener() {
        const app = express_1.default().disable("x-powered-by");
        // MiddlewareFactory is a factory to create auto-generated middleware
        const middlewareFactory = new ExpressMiddlewareFactory_1.default(Logger_1.default, constants_1.DEFAULT_CONTEXT_PATH);
        // Create handlers into handler middleware factory
        const pageBlobRangesManager = new PageBlobRangesManager_1.default();
        const handlers = {
            appendBlobHandler: new AppendBlobHandler_1.default(this.dataStore, Logger_1.default),
            blobHandler: new BlobHandler_1.default(this.dataStore, Logger_1.default, pageBlobRangesManager),
            blockBlobHandler: new BlockBlobHandler_1.default(this.dataStore, Logger_1.default),
            containerHandler: new ContainerHandler_1.default(this.dataStore, Logger_1.default),
            pageBlobHandler: new PageBlobHandler_1.default(this.dataStore, Logger_1.default, pageBlobRangesManager),
            serviceHandler: new ServiceHandler_1.default(this.dataStore, Logger_1.default)
        };
        /*
         * Generated middleware should follow strict orders
         * Manually created middleware can be injected into any points
         */
        // Access log per request
        if (this.enableAccessLog) {
            app.use(morgan("common"));
        }
        // Manually created middleware to deserialize feature related context which swagger doesn't know
        app.use(blobStorageContext_middleware_1.default);
        // Dispatch incoming HTTP request to specific operation
        app.use(middlewareFactory.createDispatchMiddleware());
        // AuthN middleware, like shared key auth or SAS auth
        const authenticationMiddlewareFactory = new AuthenticationMiddlewareFactory_1.default(Logger_1.default);
        app.use(authenticationMiddlewareFactory.createAuthenticationMiddleware([
            new PublicAccessAuthenticator_1.default(this.dataStore, Logger_1.default),
            new BlobSharedKeyAuthenticator_1.default(this.accountDataStore, Logger_1.default),
            new AccountSASAuthenticator_1.default(this.accountDataStore, this.dataStore, Logger_1.default),
            new BlobSASAuthenticator_1.default(this.accountDataStore, this.dataStore, Logger_1.default)
        ]));
        // Generated, will do basic validation defined in swagger
        app.use(middlewareFactory.createDeserializerMiddleware());
        // Generated, inject handlers to create a handler middleware
        app.use(middlewareFactory.createHandlerMiddleware(handlers));
        // Generated, will serialize response models into HTTP response
        app.use(middlewareFactory.createSerializerMiddleware());
        // Generated, will return MiddlewareError and Errors thrown in previous middleware/handlers to HTTP response
        app.use(middlewareFactory.createErrorMiddleware());
        // Generated, will end and return HTTP response immediately
        app.use(middlewareFactory.createEndMiddleware());
        return app;
    }
}
exports.default = BlobRequestListenerFactory;
//# sourceMappingURL=BlobRequestListenerFactory.js.map