"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const BlobStorageContext_1 = tslib_1.__importDefault(require("../context/BlobStorageContext"));
const StorageErrorFactory_1 = tslib_1.__importDefault(require("../errors/StorageErrorFactory"));
const models_1 = require("../generated/artifacts/models");
const operation_1 = tslib_1.__importDefault(require("../generated/artifacts/operation"));
const BlobSASPermissions_1 = require("./BlobSASPermissions");
const BlobSASResourceType_1 = require("./BlobSASResourceType");
const IBlobSASSignatureValues_1 = require("./IBlobSASSignatureValues");
const OperationBlobSASPermission_1 = require("./OperationBlobSASPermission");
class BlobSASAuthenticator {
    constructor(accountDataStore, blobDataStore, logger) {
        this.accountDataStore = accountDataStore;
        this.blobDataStore = blobDataStore;
        this.logger = logger;
    }
    async validate(req, context) {
        this.logger.info(`BlobSASAuthenticator:validate() Start validation against blob service Shared Access Signature pattern.`, context.contextID);
        this.logger.debug("BlobSASAuthenticator:validate() Getting account properties...", context.contextID);
        const blobContext = new BlobStorageContext_1.default(context);
        const account = blobContext.account;
        if (account === undefined) {
            throw RangeError(`BlobSASAuthenticator:validate() account is undefined in context.`);
        }
        const containerName = blobContext.container;
        if (containerName === undefined) {
            this.logger.error(`BlobSASAuthenticator:validate() container name is undefined in context.`, context.contextID);
            return undefined;
        }
        const blobName = blobContext.blob;
        this.logger.debug(
        // tslint:disable-next-line:max-line-length
        `BlobSASAuthenticator:validate() Retrieved account name from context: ${account}, container: ${containerName}, blob: ${blobName}`, context.contextID);
        // TODO: Make following async
        const accountProperties = this.accountDataStore.getAccount(account);
        if (accountProperties === undefined) {
            throw StorageErrorFactory_1.default.getInvalidOperation(context.contextID, "Invalid storage account.");
        }
        this.logger.debug("BlobSASAuthenticator:validate() Got account properties successfully.", context.contextID);
        // Extract blob service SAS authentication required parameters
        const signature = this.decodeIfExist(req.getQuery("sig"));
        this.logger.debug(`BlobSASAuthenticator:validate() Retrieved signature from URL parameter sig: ${signature}`, context.contextID);
        if (signature === undefined) {
            this.logger.debug(`BlobSASAuthenticator:validate() No signature found in request. Skip blob service SAS validation.`, context.contextID);
            return undefined;
        }
        const resource = this.decodeIfExist(req.getQuery("sr"));
        if (resource !== BlobSASResourceType_1.BlobSASResourceType.Container &&
            resource !== BlobSASResourceType_1.BlobSASResourceType.Blob) {
            this.logger.debug(
            // tslint:disable-next-line:max-line-length
            `BlobSASAuthenticator:validate() Signed resource type ${resource} is invalid. Skip blob service SAS validation.`, context.contextID);
            return undefined;
        }
        this.logger.debug(`BlobSASAuthenticator:validate() Signed resource type is ${resource}.`, context.contextID);
        const values = this.getBlobSASSignatureValuesFromRequest(req, containerName, blobName);
        if (values === undefined) {
            this.logger.info(
            // tslint:disable-next-line:max-line-length
            `BlobSASAuthenticator:validate() Failed to get valid blob service SAS values from request. Skip blob service SAS validation.`, context.contextID);
            return undefined;
        }
        this.logger.debug(`BlobSASAuthenticator:validate() Successfully got valid blob service SAS values from request. ${JSON.stringify(values)}`, context.contextID);
        this.logger.info(`BlobSASAuthenticator:validate() Validate signature based account key1.`, context.contextID);
        const [sig1, stringToSign1] = IBlobSASSignatureValues_1.generateBlobSASSignature(values, resource, account, accountProperties.key1);
        this.logger.debug(`BlobSASAuthenticator:validate() String to sign is: ${JSON.stringify(stringToSign1)}`, context.contextID);
        this.logger.debug(`BlobSASAuthenticator:validate() Calculated signature is: ${sig1}`, context.contextID);
        const sig1Pass = sig1 === signature;
        this.logger.info(`BlobSASAuthenticator:validate() Signature based on key1 validation ${sig1Pass ? "passed" : "failed"}.`, context.contextID);
        if (accountProperties.key2 !== undefined) {
            this.logger.info(`BlobSASAuthenticator:validate() Account key2 is not empty, validate signature based account key2.`, context.contextID);
            const [sig2, stringToSign2] = IBlobSASSignatureValues_1.generateBlobSASSignature(values, resource, account, accountProperties.key2);
            this.logger.debug(`BlobSASAuthenticator:validate() String to sign is: ${JSON.stringify(stringToSign2)}`, context.contextID);
            this.logger.debug(`BlobSASAuthenticator:validate() Calculated signature is: ${sig2}`, context.contextID);
            const sig2Pass = sig2 !== signature;
            this.logger.info(`BlobSASAuthenticator:validate() Signature based on key2 validation ${sig2Pass ? "passed" : "failed"}.`, context.contextID);
            if (!sig2Pass && !sig1Pass) {
                this.logger.info(`BlobSASAuthenticator:validate() Validate signature based account key1 and key2 failed.`, context.contextID);
                return false;
            }
        }
        else {
            if (!sig1Pass) {
                return false;
            }
        }
        // When signature validation passes, we enforce blob service SAS validation
        // Any validation errors will stop this request immediately
        // TODO: Validate permissions from ACL identifier by extract permissions, start time and expiry time from ACL
        if (values.identifier !== undefined) {
            const accessPolicy = await this.getContainerAccessPolicyByIdentifier(account, containerName, values.identifier);
            if (accessPolicy === undefined) {
                this.logger.warn(`BlobSASAuthenticator:validate() Cannot get access policy defined for container ${containerName} with id ${values.identifier}.`, context.contextID);
                throw StorageErrorFactory_1.default.getAuthorizationFailure(context.contextID);
            }
            values.startTime = accessPolicy.start;
            values.expiryTime = accessPolicy.expiry;
            values.permissions = accessPolicy.permission;
        }
        this.logger.info(`BlobSASAuthenticator:validate() Validate start and expiry time.`, context.contextID);
        if (!this.validateTime(values.expiryTime, values.startTime)) {
            this.logger.info(`BlobSASAuthenticator:validate() Validate start and expiry failed.`, context.contextID);
            throw StorageErrorFactory_1.default.getAuthorizationFailure(context.contextID);
        }
        this.logger.info(`BlobSASAuthenticator:validate() Validate IP range.`, context.contextID);
        if (!this.validateIPRange()) {
            this.logger.info(`BlobSASAuthenticator:validate() Validate IP range failed.`, context.contextID);
            throw StorageErrorFactory_1.default.getAuthorizationSourceIPMismatch(context.contextID);
        }
        this.logger.info(`BlobSASAuthenticator:validate() Validate request protocol.`, context.contextID);
        if (!this.validateProtocol(values.protocol, req.getProtocol())) {
            this.logger.info(`BlobSASAuthenticator:validate() Validate protocol failed.`, context.contextID);
            throw StorageErrorFactory_1.default.getAuthorizationProtocolMismatch(context.contextID);
        }
        const operation = context.operation;
        if (operation === undefined) {
            throw new Error(
            // tslint:disable-next-line:max-line-length
            `BlobSASAuthenticator:validate() Operation shouldn't be undefined. Please make sure DispatchMiddleware is hooked before authentication related middleware.`);
        }
        const blobSASPermission = resource === BlobSASResourceType_1.BlobSASResourceType.Blob
            ? OperationBlobSASPermission_1.OPERATION_BLOB_SAS_BLOB_PERMISSIONS.get(operation)
            : OperationBlobSASPermission_1.OPERATION_BLOB_SAS_CONTAINER_PERMISSIONS.get(operation);
        this.logger.debug(`BlobSASAuthenticator:validate() Got permission requirements for operation ${operation_1.default[operation]} - ${JSON.stringify(blobSASPermission)}`, context.contextID);
        if (blobSASPermission === undefined) {
            throw new Error(
            // tslint:disable-next-line:max-line-length
            `BlobSASAuthenticator:validate() ${resource === BlobSASResourceType_1.BlobSASResourceType.Blob
                ? "OPERATION_BLOB_SAS_BLOB_PERMISSIONS"
                : "OPERATION_BLOB_SAS_CONTAINER_PERMISSIONS"} doesn't have configuration for operation ${operation_1.default[operation]}'s blob service SAS permission.`);
        }
        if (!blobSASPermission.validatePermissions(values.permissions)) {
            throw StorageErrorFactory_1.default.getAuthorizationPermissionMismatch(context.contextID);
        }
        // Check 3 special permission requirements
        // If block blob exists, then permission must be Write only
        // If page blob exists, then permission must be Write only
        // If append blob exists, then permission must be Write only
        // If copy destination blob exists, then permission must be Write only
        if (operation === operation_1.default.BlockBlob_Upload ||
            operation === operation_1.default.PageBlob_Create ||
            operation === operation_1.default.AppendBlob_Create ||
            operation === operation_1.default.Blob_StartCopyFromURL) {
            this.logger.info(`BlobSASAuthenticator:validate() For ${operation_1.default[operation]}, if blob exists, the permission must be Write.`, context.contextID);
            if ((await this.blobExist(account, containerName, blobName)) &&
                blobSASPermission.permission !== BlobSASPermissions_1.BlobSASPermission.Write) {
                this.logger.info(`BlobSASAuthenticator:validate() Account SAS validation failed for special requirement.`, context.contextID);
                throw StorageErrorFactory_1.default.getAuthorizationPermissionMismatch(context.contextID);
            }
        }
        this.logger.info(`BlobSASAuthenticator:validate() Blob service SAS validation successfully.`, context.contextID);
        // TODO: Handle enforced response headers defined in blob service SAS
        return true;
    }
    getBlobSASSignatureValuesFromRequest(req, containerName, blobName) {
        const version = this.decodeIfExist(req.getQuery("sv"));
        const protocol = this.decodeIfExist(req.getQuery("spr"));
        const startTime = this.decodeIfExist(req.getQuery("st"));
        const expiryTime = this.decodeIfExist(req.getQuery("se"));
        const permissions = this.decodeIfExist(req.getQuery("sp"));
        const ipRange = this.decodeIfExist(req.getQuery("sip"));
        const identifier = this.decodeIfExist(req.getQuery("si"));
        const cacheControl = this.decodeIfExist(req.getQuery("rscc"));
        const contentDisposition = this.decodeIfExist(req.getQuery("rscd"));
        const contentEncoding = this.decodeIfExist(req.getQuery("rsce"));
        const contentLanguage = this.decodeIfExist(req.getQuery("rscl"));
        const contentType = this.decodeIfExist(req.getQuery("rsct"));
        if (!identifier && (!permissions || !expiryTime)) {
            this.logger.warn(
            // tslint:disable-next-line:max-line-length
            `BlobSASAuthenticator:generateBlobSASSignature(): Must provide 'permissions' and 'expiryTime' for Blob SAS generation when 'identifier' is not provided.`);
            return undefined;
        }
        if (version === undefined) {
            this.logger.warn(
            // tslint:disable-next-line:max-line-length
            `BlobSASAuthenticator:generateBlobSASSignature(): Must provide 'version'.`);
            return undefined;
        }
        const blobSASValues = {
            version,
            protocol,
            startTime,
            expiryTime,
            permissions,
            ipRange,
            containerName,
            blobName,
            identifier,
            cacheControl,
            contentDisposition,
            contentEncoding,
            contentLanguage,
            contentType
        };
        return blobSASValues;
    }
    validateTime(expiry, start) {
        if (expiry === undefined && start === undefined) {
            return true;
        }
        const now = new Date();
        if (expiry !== undefined) {
            const expiryTime = new Date(expiry);
            if (now > expiryTime) {
                return false;
            }
        }
        if (start !== undefined) {
            const startTime = new Date(start);
            if (now < startTime) {
                return false;
            }
        }
        return true;
    }
    validateIPRange() {
        // TODO: Emulator doesn't validate IP Address
        return true;
    }
    validateProtocol(sasProtocol = "https,http", requestProtocol) {
        if (sasProtocol.includes(",")) {
            return true;
        }
        else {
            return sasProtocol.toLowerCase() === requestProtocol;
        }
    }
    decodeIfExist(value) {
        return value === undefined ? value : decodeURIComponent(value);
    }
    async getContainerAccessPolicyByIdentifier(account, container, id) {
        const containerModel = await this.blobDataStore.getContainer(account, container);
        if (containerModel === undefined) {
            return undefined;
        }
        if (containerModel.containerAcl === undefined) {
            return undefined;
        }
        for (const acl of containerModel.containerAcl) {
            if (acl.id === id) {
                return acl.accessPolicy;
            }
        }
        return undefined;
    }
    async blobExist(account, container, blob) {
        const blobModel = await this.blobDataStore.getBlob(account, container, blob);
        if (blobModel === undefined) {
            return false;
        }
        if (blobModel.properties.blobType === models_1.BlobType.BlockBlob &&
            blobModel.isCommitted === false) {
            return false;
        }
        return true;
    }
}
exports.default = BlobSASAuthenticator;
//# sourceMappingURL=BlobSASAuthenticator.js.map