"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const url_1 = require("url");
const v4_1 = tslib_1.__importDefault(require("uuid/v4"));
const BlobStorageContext_1 = tslib_1.__importDefault(require("../context/BlobStorageContext"));
const blobStorageContext_middleware_1 = require("../context/blobStorageContext.middleware");
const NotImplementedError_1 = tslib_1.__importDefault(require("../errors/NotImplementedError"));
const StorageErrorFactory_1 = tslib_1.__importDefault(require("../errors/StorageErrorFactory"));
const Models = tslib_1.__importStar(require("../generated/artifacts/models"));
const constants_1 = require("../utils/constants");
const utils_1 = require("../utils/utils");
const BaseHandler_1 = tslib_1.__importDefault(require("./BaseHandler"));
/**
 * BlobHandler handles Azure Storage Blob related requests.
 *
 * @export
 * @class BlobHandler
 * @extends {BaseHandler}
 * @implements {IBlobHandler}
 */
class BlobHandler extends BaseHandler_1.default {
    constructor(dataStore, logger, rangesManager) {
        super(dataStore, logger);
        this.rangesManager = rangesManager;
    }
    /**
     * Update Blob lease Attributes according to the current time.
     * The Attribute not set back
     *
     * @private
     * @param {BlobModel} blob
     * @param {Date} currentTime
     * @returns {BlobModel}
     * @memberof BlobHandler
     */
    static updateLeaseAttributes(blob, currentTime) {
        // check Leased -> Expired
        if (blob.properties.leaseState === Models.LeaseStateType.Leased &&
            blob.properties.leaseDuration === Models.LeaseDurationType.Fixed) {
            if (blob.leaseExpireTime !== undefined &&
                currentTime > blob.leaseExpireTime) {
                blob.properties.leaseState = Models.LeaseStateType.Expired;
                blob.properties.leaseStatus = Models.LeaseStatusType.Unlocked;
                blob.properties.leaseDuration = undefined;
                blob.leaseExpireTime = undefined;
                blob.leaseBreakExpireTime = undefined;
            }
        }
        // check Breaking -> Broken
        if (blob.properties.leaseState === Models.LeaseStateType.Breaking) {
            if (blob.leaseBreakExpireTime !== undefined &&
                currentTime > blob.leaseBreakExpireTime) {
                blob.properties.leaseState = Models.LeaseStateType.Broken;
                blob.properties.leaseStatus = Models.LeaseStatusType.Unlocked;
                blob.properties.leaseDuration = undefined;
                blob.leaseExpireTime = undefined;
                blob.leaseBreakExpireTime = undefined;
            }
        }
        return blob;
    }
    /**
     * Check Blob lease status on write blob.
     *
     * Need run the funtion on: PutBlob, SetBlobMetadata, SetBlobProperties,
     * DeleteBlob, PutBlock, PutBlockList, PutPage, AppendBlock, CopyBlob(dest)
     *
     * @private
     * @param {Context} context
     * @param {BlobModel} blob
     * @param {LeaseAccessConditions} leaseAccessConditions
     * @returns {void}
     * @memberof BlobHandler
     */
    static checkBlobLeaseOnWriteBlob(context, blob, leaseAccessConditions) {
        // check Leased -> Expired
        if (blob.properties.leaseStatus === Models.LeaseStatusType.Locked) {
            if (leaseAccessConditions === undefined ||
                leaseAccessConditions.leaseId === undefined ||
                leaseAccessConditions.leaseId === "") {
                const blobCtx = new BlobStorageContext_1.default(context);
                throw StorageErrorFactory_1.default.getBlobLeaseIdMissing(blobCtx.contextID);
            }
            else if (blob.leaseId !== undefined &&
                leaseAccessConditions.leaseId.toLowerCase() !==
                    blob.leaseId.toLowerCase()) {
                const blobCtx = new BlobStorageContext_1.default(context);
                throw StorageErrorFactory_1.default.getBlobLeaseIdMismatchWithBlobOperation(blobCtx.contextID);
            }
        }
        else if (leaseAccessConditions !== undefined &&
            leaseAccessConditions.leaseId !== undefined &&
            leaseAccessConditions.leaseId !== "") {
            const blobCtx = new BlobStorageContext_1.default(context);
            throw StorageErrorFactory_1.default.getBlobLeaseLost(blobCtx.contextID);
        }
    }
    /**
     * Update lease Expire Blob lease status to Available on write blob.
     *
     * Need run the funtion on: PutBlob, SetBlobMetadata, SetBlobProperties,
     * DeleteBlob, PutBlock, PutBlockList, PutPage, AppendBlock, CopyBlob(dest)
     *
     * @private
     * @param {BlobModel} blob
     * @returns {BlobModel}
     * @memberof BlobHandler
     */
    static UpdateBlobLeaseStateOnWriteBlob(blob) {
        if (blob.properties.leaseState === Models.LeaseStateType.Expired ||
            blob.properties.leaseState === Models.LeaseStateType.Broken) {
            blob.properties.leaseState = Models.LeaseStateType.Available;
            blob.properties.leaseStatus = Models.LeaseStatusType.Unlocked;
            blob.properties.leaseDuration = undefined;
            blob.leaseduration = undefined;
            blob.leaseId = undefined;
            blob.leaseExpireTime = undefined;
            blob.leaseBreakExpireTime = undefined;
        }
        return blob;
    }
    /**
     * Check Blob lease status on Read blob.
     *
     * @private
     * @param {Context} context
     * @param {BlobModel} blob
     * @param {LeaseAccessConditions} leaseAccessConditions
     * @returns {void}
     * @memberof BlobHandler
     */
    static checkLeaseOnReadBlob(context, blob, leaseAccessConditions) {
        // check only when input Leased Id is not empty
        if (leaseAccessConditions !== undefined &&
            leaseAccessConditions.leaseId !== undefined &&
            leaseAccessConditions.leaseId !== "") {
            // return error when lease is unlocked
            if (blob.properties.leaseStatus === Models.LeaseStatusType.Unlocked) {
                const blobCtx = new BlobStorageContext_1.default(context);
                throw StorageErrorFactory_1.default.getBlobLeaseLost(blobCtx.contextID);
            }
            else if (blob.leaseId !== undefined &&
                leaseAccessConditions.leaseId.toLowerCase() !==
                    blob.leaseId.toLowerCase()) {
                // return error when lease is locked but lease ID not match
                const blobCtx = new BlobStorageContext_1.default(context);
                throw StorageErrorFactory_1.default.getBlobLeaseIdMismatchWithBlobOperation(blobCtx.contextID);
            }
        }
    }
    /**
     * Download blob.
     *
     * @param {Models.BlobDownloadOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobDownloadResponse>}
     * @memberof BlobHandler
     */
    async download(options, context) {
        const blob = await this.getSimpleBlobFromStorage(context, options.snapshot);
        if (blob.snapshot === "") {
            BlobHandler.checkLeaseOnReadBlob(context, blob, options.leaseAccessConditions);
        }
        if (blob.properties.blobType === Models.BlobType.BlockBlob) {
            return this.downloadBlockBlob(options, context, blob);
        }
        else if (blob.properties.blobType === Models.BlobType.PageBlob) {
            return this.downloadPageBlob(options, context, blob);
        }
        else if (blob.properties.blobType === Models.BlobType.AppendBlob) {
            // TODO: Handle append blob
            throw new NotImplementedError_1.default(context.contextID);
        }
        else {
            throw StorageErrorFactory_1.default.getInvalidOperation(context.contextID);
        }
    }
    /**
     * Get blob properties.
     *
     * @param {Models.BlobGetPropertiesOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobGetPropertiesResponse>}
     * @memberof BlobHandler
     */
    async getProperties(options, context) {
        const blob = await this.getSimpleBlobFromStorage(context, options.snapshot);
        // TODO: Lease for a snapshot blob?
        if (blob.snapshot === "") {
            BlobHandler.checkLeaseOnReadBlob(context, blob, options.leaseAccessConditions);
        }
        // TODO: Create get metadata specific request in swagger
        const againstMetadata = context.request.getQuery("comp") === "metadata";
        const response = againstMetadata
            ? {
                statusCode: 200,
                metadata: blob.metadata,
                eTag: blob.properties.etag,
                requestId: context.contextID,
                version: constants_1.API_VERSION,
                date: context.startTime
            }
            : Object.assign({ statusCode: 200, metadata: blob.metadata, isIncrementalCopy: blob.properties.incrementalCopy, eTag: blob.properties.etag, requestId: context.contextID, version: constants_1.API_VERSION, date: context.startTime, acceptRanges: "bytes", blobCommittedBlockCount: undefined, isServerEncrypted: true }, blob.properties);
        return response;
    }
    /**
     * Delete blob or snapshots.
     *
     * @param {Models.BlobDeleteMethodOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobDeleteResponse>}
     * @memberof BlobHandler
     */
    async delete(options, context) {
        const blob = await this.getSimpleBlobFromStorage(context, options.snapshot);
        const againstBaseBlob = blob.snapshot === "";
        // Check Lease status
        if (againstBaseBlob) {
            BlobHandler.checkBlobLeaseOnWriteBlob(context, blob, options.leaseAccessConditions);
        }
        // Check bad requests
        if (!againstBaseBlob && options.deleteSnapshots !== undefined) {
            throw StorageErrorFactory_1.default.getInvalidOperation(context.contextID, "Invalid operation against a blob snapshot.");
        }
        // Check whether blob has snapshots
        const blobAndSnapshots = [];
        let marker;
        let blobs = [];
        do {
            [blobs, marker] = await this.dataStore.listBlobs(blob.accountName, blob.containerName, blob.name, undefined, 5000, marker, true);
            blobAndSnapshots.push(...blobs);
        } while (marker !== undefined);
        const hasSnapshots = blobAndSnapshots.length > 1;
        // Scenario: Delete base blob only
        if (againstBaseBlob && options.deleteSnapshots === undefined) {
            if (hasSnapshots) {
                throw StorageErrorFactory_1.default.getSnapshotsPresent(context.contextID);
            }
            else {
                await this.dataStore.deleteBlob(blob.accountName, blob.containerName, blob.name);
            }
        }
        // Scenario: Delete snapshot only
        if (!againstBaseBlob) {
            await this.dataStore.deleteBlob(blob.accountName, blob.containerName, blob.name, blob.snapshot);
        }
        // Scenario: Delete base blob and snapshots
        if (againstBaseBlob &&
            options.deleteSnapshots === Models.DeleteSnapshotsOptionType.Include) {
            const promises = [];
            for (const item of blobAndSnapshots) {
                promises.push(this.dataStore.deleteBlob(item.accountName, item.containerName, item.name, item.snapshot));
            }
            await Promise.all(promises);
        }
        // Scenario: Delete snapshots only
        if (againstBaseBlob &&
            options.deleteSnapshots === Models.DeleteSnapshotsOptionType.Only) {
            const promises = [];
            for (const item of blobAndSnapshots.filter(value => {
                return value.snapshot !== "";
            })) {
                promises.push(this.dataStore.deleteBlob(item.accountName, item.containerName, item.name, item.snapshot));
            }
            await Promise.all(promises);
        }
        const response = {
            statusCode: 202,
            requestId: context.contextID,
            date: context.startTime,
            version: constants_1.API_VERSION
        };
        return response;
    }
    /**
     * Undelete blob
     *
     * @param {Models.BlobUndeleteOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobUndeleteResponse>}
     * @memberof BlobHandler
     */
    async undelete(options, context) {
        throw new NotImplementedError_1.default(context.contextID);
    }
    /**
     * Set HTTP Headers
     * see also https://docs.microsoft.com/en-us/rest/api/storageservices/set-blob-properties
     *
     * @param {Models.BlobSetHTTPHeadersOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobSetHTTPHeadersResponse>}
     * @memberof BlobHandler
     */
    async setHTTPHeaders(options, context) {
        let blob = await this.getSimpleBlobFromStorage(context);
        // Check Lease status
        BlobHandler.checkBlobLeaseOnWriteBlob(context, blob, options.leaseAccessConditions);
        // Set lease to available if it's expired
        blob = BlobHandler.UpdateBlobLeaseStateOnWriteBlob(blob);
        await this.dataStore.updateBlob(blob);
        const blobHeaders = options.blobHTTPHeaders;
        const blobProps = blob.properties;
        // as per https://docs.microsoft.com/en-us/rest/api/storageservices/set-blob-properties#remarks
        // If any one or more of the following properties is set in the request,
        // then all of these properties are set together.
        // If a value is not provided for a given property when at least one
        // of the properties listed below is set, then that property will
        // be cleared for the blob.
        if (blobHeaders !== undefined) {
            blobProps.cacheControl = blobHeaders.blobCacheControl;
            blobProps.contentType = blobHeaders.blobContentType;
            blobProps.contentMD5 = blobHeaders.blobContentMD5;
            blobProps.contentEncoding = blobHeaders.blobContentEncoding;
            blobProps.contentLanguage = blobHeaders.blobContentLanguage;
            blobProps.contentDisposition = blobHeaders.blobContentDisposition;
            blobProps.lastModified = context.startTime
                ? context.startTime
                : new Date();
        }
        blob.properties = blobProps;
        await this.dataStore.updateBlob(blob);
        // ToDo: return correct headers and test for these.
        const response = {
            statusCode: 200,
            eTag: blob.properties.etag,
            lastModified: blob.properties.lastModified,
            blobSequenceNumber: blob.properties.blobSequenceNumber,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            date: context.startTime
        };
        return response;
    }
    /**
     * Set Metadata
     *
     * @param {Models.BlobSetMetadataOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobSetMetadataResponse>}
     * @memberof BlobHandler
     */
    async setMetadata(options, context) {
        let blob = await this.getSimpleBlobFromStorage(context);
        // Check Lease status
        BlobHandler.checkBlobLeaseOnWriteBlob(context, blob, options.leaseAccessConditions);
        blob.metadata = options.metadata;
        // Set lease to available if it's expired
        blob = BlobHandler.UpdateBlobLeaseStateOnWriteBlob(blob);
        await this.dataStore.updateBlob(blob);
        // ToDo: return correct headers and test for these.
        const response = {
            statusCode: 200,
            eTag: blob.properties.etag,
            lastModified: blob.properties.lastModified,
            isServerEncrypted: true,
            requestId: context.contextID,
            date: context.startTime,
            version: constants_1.API_VERSION
        };
        return response;
    }
    /**
     * Acquire Blob Lease
     *
     * @param {Models.BlobAcquireLeaseOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobAcquireLeaseResponse>}
     * @memberof BlobHandler
     */
    async acquireLease(options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        let blob;
        try {
            blob = await this.getSimpleBlobFromStorage(context);
        }
        catch (error) {
            throw error;
        }
        // check the lease action aligned with current lease state.
        if (blob.snapshot !== "") {
            throw StorageErrorFactory_1.default.getBlobSnapshotsPresent(blobCtx.contextID);
        }
        if (blob.properties.leaseState === Models.LeaseStateType.Breaking) {
            throw StorageErrorFactory_1.default.getLeaseAlreadyPresent(context.contextID);
        }
        if (blob.properties.leaseState === Models.LeaseStateType.Leased &&
            options.proposedLeaseId !== blob.leaseId) {
            throw StorageErrorFactory_1.default.getLeaseAlreadyPresent(context.contextID);
        }
        // update the lease information
        if (options.duration === -1 || options.duration === undefined) {
            blob.properties.leaseDuration = Models.LeaseDurationType.Infinite;
        }
        else {
            // verify options.duration between 15 and 60
            if (options.duration > 60 || options.duration < 15) {
                throw StorageErrorFactory_1.default.getInvalidLeaseDuration(context.contextID);
            }
            blob.properties.leaseDuration = Models.LeaseDurationType.Fixed;
            blob.leaseExpireTime = blobCtx.startTime;
            blob.leaseExpireTime.setSeconds(blob.leaseExpireTime.getSeconds() + options.duration);
            blob.leaseduration = options.duration;
        }
        blob.properties.leaseState = Models.LeaseStateType.Leased;
        blob.properties.leaseStatus = Models.LeaseStatusType.Locked;
        blob.leaseId =
            options.proposedLeaseId !== "" && options.proposedLeaseId !== undefined
                ? options.proposedLeaseId
                : v4_1.default();
        blob.leaseBreakExpireTime = undefined;
        await this.dataStore.updateBlob(blob);
        const response = {
            date: blobCtx.startTime,
            eTag: blob.properties.etag,
            lastModified: blob.properties.lastModified,
            leaseId: blob.leaseId,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            statusCode: 201
        };
        return response;
    }
    /**
     * release blob lease
     *
     * @param {string} leaseId
     * @param {Models.BlobReleaseLeaseOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobReleaseLeaseResponse>}
     * @memberof BlobHandler
     */
    async releaseLease(leaseId, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        let blob;
        try {
            blob = await this.getSimpleBlobFromStorage(context);
        }
        catch (error) {
            throw error;
        }
        // check the lease action aligned with current lease state.
        if (blob.snapshot !== "") {
            throw StorageErrorFactory_1.default.getBlobSnapshotsPresent(blobCtx.contextID);
        }
        if (blob.properties.leaseState === Models.LeaseStateType.Available) {
            throw StorageErrorFactory_1.default.getBlobLeaseIdMismatchWithLeaseOperation(blobCtx.contextID);
        }
        // Check lease ID
        if (blob.leaseId !== leaseId) {
            throw StorageErrorFactory_1.default.getBlobLeaseIdMismatchWithLeaseOperation(blobCtx.contextID);
        }
        // update the lease information
        blob.properties.leaseState = Models.LeaseStateType.Available;
        blob.properties.leaseStatus = Models.LeaseStatusType.Unlocked;
        blob.properties.leaseDuration = undefined;
        blob.leaseduration = undefined;
        blob.leaseId = undefined;
        blob.leaseExpireTime = undefined;
        blob.leaseBreakExpireTime = undefined;
        await this.dataStore.updateBlob(blob);
        const response = {
            date: blobCtx.startTime,
            eTag: blob.properties.etag,
            lastModified: blob.properties.lastModified,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            statusCode: 200
        };
        return response;
    }
    /**
     * Renew blob lease
     *
     * @param {string} leaseId
     * @param {Models.BlobRenewLeaseOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobRenewLeaseResponse>}
     * @memberof BlobHandler
     */
    async renewLease(leaseId, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        let blob;
        try {
            blob = await this.getSimpleBlobFromStorage(context);
        }
        catch (error) {
            throw error;
        }
        // check the lease action aligned with current lease state.
        if (blob.snapshot !== "") {
            throw StorageErrorFactory_1.default.getBlobSnapshotsPresent(blobCtx.contextID);
        }
        if (blob.properties.leaseState === Models.LeaseStateType.Available) {
            throw StorageErrorFactory_1.default.getBlobLeaseIdMismatchWithLeaseOperation(blobCtx.contextID);
        }
        if (blob.properties.leaseState === Models.LeaseStateType.Breaking ||
            blob.properties.leaseState === Models.LeaseStateType.Broken) {
            throw StorageErrorFactory_1.default.getLeaseIsBrokenAndCannotBeRenewed(blobCtx.contextID);
        }
        // Check lease ID
        if (blob.leaseId !== leaseId) {
            throw StorageErrorFactory_1.default.getBlobLeaseIdMismatchWithLeaseOperation(blobCtx.contextID);
        }
        // update the lease information
        blob.properties.leaseState = Models.LeaseStateType.Leased;
        blob.properties.leaseStatus = Models.LeaseStatusType.Locked;
        // when container.leaseduration has value (not -1), means fixed duration
        if (blob.leaseduration !== undefined && blob.leaseduration !== -1) {
            blob.leaseExpireTime = blobCtx.startTime;
            blob.leaseExpireTime.setSeconds(blob.leaseExpireTime.getSeconds() + blob.leaseduration);
            blob.properties.leaseDuration = Models.LeaseDurationType.Fixed;
        }
        else {
            blob.properties.leaseDuration = Models.LeaseDurationType.Infinite;
        }
        await this.dataStore.updateContainer(blob);
        const response = {
            date: blobCtx.startTime,
            eTag: blob.properties.etag,
            lastModified: blob.properties.lastModified,
            leaseId: blob.leaseId,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            statusCode: 200
        };
        return response;
    }
    /**
     * Change lease.
     *
     * @param {string} leaseId
     * @param {string} proposedLeaseId
     * @param {Models.BlobChangeLeaseOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobChangeLeaseResponse>}
     * @memberof BlobHandler
     */
    async changeLease(leaseId, proposedLeaseId, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        let blob;
        blob = await this.getSimpleBlobFromStorage(context);
        // check the lease action aligned with current lease state.
        if (blob.snapshot !== "") {
            throw StorageErrorFactory_1.default.getBlobSnapshotsPresent(blobCtx.contextID);
        }
        if (blob.properties.leaseState === Models.LeaseStateType.Available ||
            blob.properties.leaseState === Models.LeaseStateType.Expired ||
            blob.properties.leaseState === Models.LeaseStateType.Broken) {
            throw StorageErrorFactory_1.default.getBlobLeaseNotPresentWithLeaseOperation(blobCtx.contextID);
        }
        if (blob.properties.leaseState === Models.LeaseStateType.Breaking) {
            throw StorageErrorFactory_1.default.getLeaseIsBreakingAndCannotBeChanged(blobCtx.contextID);
        }
        // Check lease ID
        if (blob.leaseId !== leaseId && blob.leaseId !== proposedLeaseId) {
            throw StorageErrorFactory_1.default.getBlobLeaseIdMismatchWithLeaseOperation(blobCtx.contextID);
        }
        // update the lease information, only need update lease ID
        blob.leaseId = proposedLeaseId;
        await this.dataStore.updateBlob(blob);
        const response = {
            date: blobCtx.startTime,
            eTag: blob.properties.etag,
            lastModified: blob.properties.lastModified,
            leaseId: blob.leaseId,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            statusCode: 200
        };
        return response;
    }
    /**
     * Break lease.
     *
     * @param {Models.BlobBreakLeaseOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobBreakLeaseResponse>}
     * @memberof BlobHandler
     */
    async breakLease(options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        let blob;
        let leaseTimeinSecond;
        leaseTimeinSecond = 0;
        blob = await this.getSimpleBlobFromStorage(context);
        // check the lease action aligned with current lease state.
        if (blob.snapshot !== "") {
            throw StorageErrorFactory_1.default.getBlobSnapshotsPresent(blobCtx.contextID);
        }
        if (blob.properties.leaseState === Models.LeaseStateType.Available) {
            throw StorageErrorFactory_1.default.getBlobLeaseNotPresentWithLeaseOperation(blobCtx.contextID);
        }
        // update the lease information
        // verify options.breakPeriod between 0 and 60
        if (options.breakPeriod !== undefined &&
            (options.breakPeriod > 60 || options.breakPeriod < 0)) {
            throw StorageErrorFactory_1.default.getInvalidLeaseBreakPeriod(blobCtx.contextID);
        }
        if (blob.properties.leaseState === Models.LeaseStateType.Expired ||
            blob.properties.leaseState === Models.LeaseStateType.Broken ||
            options.breakPeriod === 0 ||
            options.breakPeriod === undefined) {
            blob.properties.leaseState = Models.LeaseStateType.Broken;
            blob.properties.leaseStatus = Models.LeaseStatusType.Unlocked;
            blob.properties.leaseDuration = undefined;
            blob.leaseduration = undefined;
            blob.leaseExpireTime = undefined;
            blob.leaseBreakExpireTime = undefined;
            leaseTimeinSecond = 0;
        }
        else {
            blob.properties.leaseState = Models.LeaseStateType.Breaking;
            blob.properties.leaseStatus = Models.LeaseStatusType.Locked;
            blob.leaseduration = undefined;
            if (blob.properties.leaseDuration === Models.LeaseDurationType.Infinite) {
                blob.properties.leaseDuration = undefined;
                blob.leaseExpireTime = undefined;
                blob.leaseBreakExpireTime = new Date(blobCtx.startTime);
                blob.leaseBreakExpireTime.setSeconds(blob.leaseBreakExpireTime.getSeconds() + options.breakPeriod);
                leaseTimeinSecond = options.breakPeriod;
            }
            else {
                let newleaseBreakExpireTime = new Date(blobCtx.startTime);
                newleaseBreakExpireTime.setSeconds(newleaseBreakExpireTime.getSeconds() + options.breakPeriod);
                if (blob.leaseExpireTime !== undefined &&
                    newleaseBreakExpireTime > blob.leaseExpireTime) {
                    newleaseBreakExpireTime = blob.leaseExpireTime;
                }
                if (blob.leaseBreakExpireTime === undefined ||
                    blob.leaseBreakExpireTime > newleaseBreakExpireTime) {
                    blob.leaseBreakExpireTime = newleaseBreakExpireTime;
                }
                leaseTimeinSecond = Math.round(Math.abs(blob.leaseBreakExpireTime.getTime() - blobCtx.startTime.getTime()) / 1000);
                blob.leaseExpireTime = undefined;
                blob.properties.leaseDuration = undefined;
            }
        }
        await this.dataStore.updateBlob(blob);
        const response = {
            date: blobCtx.startTime,
            eTag: blob.properties.etag,
            lastModified: blob.properties.lastModified,
            leaseTime: leaseTimeinSecond,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            statusCode: 202
        };
        return response;
    }
    /**
     * Create snapshot.
     *
     * @see https://docs.microsoft.com/en-us/rest/api/storageservices/snapshot-blob
     *
     * @param {Models.BlobCreateSnapshotOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobCreateSnapshotResponse>}
     * @memberof BlobHandler
     */
    async createSnapshot(options, context) {
        const blob = await this.getSimpleBlobFromStorage(context);
        // Deep clone blob model to snapshot blob model.
        // TODO: Create a method for deep object copy
        const snapshotBlob = {
            name: blob.name,
            deleted: false,
            snapshot: context.startTime.toISOString(),
            properties: Object.assign({}, blob.properties),
            metadata: Object.assign({}, blob.metadata),
            accountName: blob.accountName,
            containerName: blob.containerName,
            pageRangesInOrder: blob.pageRangesInOrder === undefined
                ? undefined
                : blob.pageRangesInOrder.slice(),
            isCommitted: blob.isCommitted,
            leaseduration: blob.leaseduration,
            leaseId: blob.leaseId,
            leaseExpireTime: blob.leaseExpireTime,
            leaseBreakExpireTime: blob.leaseBreakExpireTime,
            committedBlocksInOrder: blob.committedBlocksInOrder === undefined
                ? undefined
                : blob.committedBlocksInOrder.slice(),
            persistency: blob.persistency === undefined ? undefined : Object.assign({}, blob.persistency)
        };
        await this.dataStore.updateBlob(snapshotBlob);
        const response = {
            statusCode: 201,
            eTag: snapshotBlob.properties.etag,
            lastModified: snapshotBlob.properties.lastModified,
            requestId: context.contextID,
            date: context.startTime,
            version: constants_1.API_VERSION,
            snapshot: snapshotBlob.snapshot
        };
        return response;
    }
    /**
     * start copy from Url
     *
     * @param {string} copySource
     * @param {Models.BlobStartCopyFromURLOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobStartCopyFromURLResponse>}
     * @memberof BlobHandler
     */
    async startCopyFromURL(copySource, options, context) {
        const blobContext = new BlobStorageContext_1.default(context);
        // TODO: Check dest Lease status, and set to available if it's expired, see sample in BlobHandler.setMetadata()
        const url = new url_1.URL(copySource);
        const [sourceAccount, sourceContainer, sourceBlob] = blobStorageContext_middleware_1.extractStoragePartsFromPath(url.pathname);
        const snapshot = url.searchParams.get("snapshot") || "";
        if (sourceAccount !== blobContext.account ||
            sourceAccount === undefined ||
            sourceContainer === undefined ||
            sourceBlob === undefined) {
            throw new NotImplementedError_1.default(context.contextID);
        }
        // TODO: Only supports copy from devstoreaccount1, not a complete copy implementation
        // Extract source account name, container name, blob name and snapshot
        // If within devstoreaccount1
        const sourceContainerModel = await this.dataStore.getContainer(sourceAccount, sourceContainer);
        if (sourceContainerModel === undefined) {
            // TODO: Check error message
            throw StorageErrorFactory_1.default.getInvalidOperation(context.contextID, "Source container doesn't exist.");
        }
        // Get source storage blob model
        const sourceBlobModel = await this.dataStore.getBlob(sourceAccount, sourceContainer, sourceBlob, snapshot);
        // If source is uncommitted or deleted
        if (sourceBlobModel === undefined ||
            sourceBlobModel.deleted ||
            !sourceBlobModel.isCommitted) {
            // TODO: Check error message
            throw StorageErrorFactory_1.default.getInvalidOperation(context.contextID, "Source blob doesn't exist.");
        }
        const destContainerModel = await this.dataStore.getContainer(blobContext.account, blobContext.container);
        if (destContainerModel === undefined) {
            throw StorageErrorFactory_1.default.getContainerNotFound(blobContext.contextID);
        }
        // Deep clone a copied blob
        const copiedBlob = {
            name: blobContext.blob,
            deleted: false,
            snapshot: "",
            properties: Object.assign({}, sourceBlobModel.properties, { creationTime: context.startTime, lastModified: context.startTime, etag: utils_1.newEtag(), leaseStatus: Models.LeaseStatusType.Unlocked, leaseState: Models.LeaseStateType.Available, leaseDuration: undefined, copyId: v4_1.default(), copyStatus: Models.CopyStatusType.Success, copySource, copyProgress: sourceBlobModel.properties.contentLength
                    ? `${sourceBlobModel.properties.contentLength}/${sourceBlobModel.properties.contentLength}`
                    : undefined, copyCompletionTime: context.startTime, copyStatusDescription: undefined, incrementalCopy: false, destinationSnapshot: undefined, deletedTime: undefined, remainingRetentionDays: undefined, archiveStatus: undefined, accessTierChangeTime: undefined }),
            metadata: options.metadata === undefined
                ? Object.assign({}, sourceBlobModel.metadata) : options.metadata,
            accountName: blobContext.account,
            containerName: blobContext.container,
            pageRangesInOrder: sourceBlobModel.pageRangesInOrder,
            isCommitted: sourceBlobModel.isCommitted,
            leaseduration: undefined,
            leaseId: undefined,
            leaseExpireTime: undefined,
            leaseBreakExpireTime: undefined,
            committedBlocksInOrder: sourceBlobModel.committedBlocksInOrder,
            persistency: sourceBlobModel.persistency
        };
        await this.dataStore.updateBlob(copiedBlob);
        const response = {
            statusCode: 202,
            eTag: copiedBlob.properties.etag,
            lastModified: copiedBlob.properties.lastModified,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            date: context.startTime,
            copyId: copiedBlob.properties.copyId,
            copyStatus: copiedBlob.properties.copyStatus
        };
        return response;
    }
    /**
     * abort copy from Url
     *
     * @param {string} copyId
     * @param {Models.BlobAbortCopyFromURLOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobAbortCopyFromURLResponse>}
     * @memberof BlobHandler
     */
    async abortCopyFromURL(copyId, options, context) {
        const blob = await this.getSimpleBlobFromStorage(context);
        if (blob.properties.copyId !== copyId) {
            throw StorageErrorFactory_1.default.getCopyIdMismatch(context.contextID);
        }
        if (blob.properties.copyStatus === Models.CopyStatusType.Success) {
            throw StorageErrorFactory_1.default.getNoPendingCopyOperation(context.contextID);
        }
        const response = {
            statusCode: 204,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            date: context.startTime
        };
        return response;
    }
    /**
     * set blob tier
     *
     * @param {Models.AccessTier} tier
     * @param {Models.BlobSetTierOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.BlobSetTierResponse>}
     * @memberof BlobHandler
     */
    async setTier(tier, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        let blob;
        blob = await this.getSimpleBlobFromStorage(context);
        // check the lease action aligned with current lease state.
        // the API has not lease ID input, but run it on a lease blocked blob will fail with LeaseIdMissing,
        // this is aliged with server behavior
        BlobHandler.checkBlobLeaseOnWriteBlob(context, blob, undefined);
        // Check Blob is not snapshot
        if (blob.snapshot !== "") {
            throw StorageErrorFactory_1.default.getBlobSnapshotsPresent(blobCtx.contextID);
        }
        const response = {
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            statusCode: 200
        };
        // Check BlobTier matches blob type
        if ((tier === Models.AccessTier.Archive ||
            tier === Models.AccessTier.Cool ||
            tier === Models.AccessTier.Hot) &&
            blob.properties.blobType === Models.BlobType.BlockBlob) {
            // Block blob
            // tslint:disable-next-line:max-line-length
            // TODO: check blob is not block blob with snapshot, throw StorageErrorFactory.getBlobSnapshotsPresent_hassnapshot()
            // Archive -> Coo/Hot will return 202
            if (blob.properties.accessTier === Models.AccessTier.Archive &&
                (tier === Models.AccessTier.Cool || tier === Models.AccessTier.Hot)) {
                response.statusCode = 202;
            }
            blob.properties.accessTier = tier;
            blob.properties.accessTierChangeTime = context.startTime;
        }
        // else if (
        //   tier
        //     .toString()
        //     .toUpperCase()
        //     .startsWith("P") &&
        //   blob.properties.blobType === Models.BlobType.PageBlob
        // ) {
        //   // page blob
        //   // Check Page blob tier not set to lower
        //   if (blob.properties.accessTier !== undefined) {
        //     const oldTierInt = parseInt(
        //       blob.properties.accessTier.toString().substring(1),
        //       10
        //     );
        //     const newTierInt = parseInt(tier.toString().substring(1), 10);
        //     if (oldTierInt > newTierInt) {
        //       throw StorageErrorFactory.getBlobCannotChangeToLowerTier(
        //         blobCtx.contextID!
        //       );
        //     }
        //   }
        //   if (!PageBlobAccessTierThreshold.has(tier)) {
        //     throw Error(
        //       `BlobHandler:setTier() PageBlobAccessTierThreshold doesn't define tier threshold for tier ${tier}`
        //     );
        //   }
        //   // Check Blob size match tier
        //   if (
        //     blob.properties.contentLength! > PageBlobAccessTierThreshold.get(tier)!
        //   ) {
        //     throw StorageErrorFactory.getBlobBlobTierInadequateForContentLength(
        //       blobCtx.contextID!
        //     );
        //   }
        //   blob.properties.accessTier = tier;
        //   blob.properties.accessTierChangeTime = context.startTime;
        // }
        // tslint:disable-next-line:one-line
        else {
            // Blob tier and blob type not match
            throw StorageErrorFactory_1.default.getBlobInvalidBlobType(blobCtx.contextID);
        }
        await this.dataStore.updateBlob(blob);
        return response;
    }
    /**
     * get account info
     *
     * @param {Context} context
     * @returns {Promise<Models.BlobGetAccountInfoResponse>}
     * @memberof BlobHandler
     */
    async getAccountInfo(context) {
        return utils_1.getContainerGetAccountInfoResponse(context);
    }
    /**
     * get account info with headers
     *
     * @param {Context} context
     * @returns {Promise<Models.BlobGetAccountInfoResponse>}
     * @memberof BlobHandler
     */
    async getAccountInfoWithHead(context) {
        return utils_1.getContainerGetAccountInfoResponse(context);
    }
    /**
     * download block blob
     *
     * @private
     * @param {Models.BlobDownloadOptionalParams} options
     * @param {Context} context
     * @param {BlobModel} blob
     * @returns {Promise<Models.BlobDownloadResponse>}
     * @memberof BlobHandler
     */
    async downloadBlockBlob(options, context, blob) {
        if (blob.snapshot === "") {
            BlobHandler.checkLeaseOnReadBlob(context, blob, options.leaseAccessConditions);
        }
        if (blob.isCommitted === false) {
            throw StorageErrorFactory_1.default.getBlobNotFound(context.contextID);
        }
        // Deserializer doesn't handle range header currently, manually parse range headers here
        const rangesParts = utils_1.deserializeRangeHeader(context.request.getHeader("range"), context.request.getHeader("x-ms-range"));
        const rangeStart = rangesParts[0];
        let rangeEnd = rangesParts[1];
        // Will automatically shift request with longer data end than blob size to blob size
        if (rangeEnd + 1 >= blob.properties.contentLength) {
            rangeEnd = blob.properties.contentLength - 1;
        }
        const contentLength = rangeEnd - rangeStart + 1;
        const partialRead = contentLength !== blob.properties.contentLength;
        this.logger.info(
        // tslint:disable-next-line:max-line-length
        `BlobHandler:downloadBlockBlob() NormalizedDownloadRange=bytes=${rangeStart}-${rangeEnd} RequiredContentLength=${contentLength}`, context.contextID);
        let bodyGetter;
        const blocks = blob.committedBlocksInOrder;
        if (blocks === undefined || blocks.length === 0) {
            bodyGetter = async () => {
                return this.dataStore.readPayload(blob.persistency, rangeStart, rangeEnd + 1 - rangeStart);
            };
        }
        else {
            bodyGetter = async () => {
                return this.dataStore.readPayloads(blocks.map(block => block.persistency), rangeStart, rangeEnd + 1 - rangeStart);
            };
        }
        let body = await bodyGetter();
        let contentMD5;
        if (!partialRead) {
            contentMD5 = blob.properties.contentMD5;
        }
        if (contentLength <= 4 * 1024 * 1024 &&
            contentMD5 === undefined &&
            body !== undefined) {
            contentMD5 = await utils_1.getMD5FromStream(body);
            body = await bodyGetter();
        }
        const response = Object.assign({ statusCode: 200, body, metadata: blob.metadata, eTag: blob.properties.etag, requestId: context.contextID, date: context.startTime, version: constants_1.API_VERSION }, blob.properties, { blobContentMD5: blob.properties.contentMD5, contentLength,
            contentMD5, isServerEncrypted: true });
        return response;
    }
    /**
     * download page blob
     *
     * @private
     * @param {Models.BlobDownloadOptionalParams} options
     * @param {Context} context
     * @param {BlobModel} blob
     * @returns {Promise<Models.BlobDownloadResponse>}
     * @memberof BlobHandler
     */
    async downloadPageBlob(options, context, blob) {
        if (blob.snapshot === "") {
            BlobHandler.checkLeaseOnReadBlob(context, blob, options.leaseAccessConditions);
        }
        // Deserializer doesn't handle range header currently, manually parse range headers here
        const rangesParts = utils_1.deserializePageBlobRangeHeader(context.request.getHeader("range"), context.request.getHeader("x-ms-range"));
        const rangeStart = rangesParts[0];
        let rangeEnd = rangesParts[1];
        // Will automatically shift request with longer data end than blob size to blob size
        if (rangeEnd + 1 >= blob.properties.contentLength) {
            rangeEnd = blob.properties.contentLength - 1;
        }
        const contentLength = rangeEnd - rangeStart + 1;
        const partialRead = contentLength !== blob.properties.contentLength;
        this.logger.info(
        // tslint:disable-next-line:max-line-length
        `BlobHandler:downloadPageBlob() NormalizedDownloadRange=bytes=${rangeStart}-${rangeEnd} RequiredContentLength=${contentLength}`, context.contextID);
        // if (contentLength <= 0) {
        //   return {
        //     statusCode: 200,
        //     body: undefined,
        //     metadata: blob.metadata,
        //     eTag: blob.properties.etag,
        //     requestId: context.contextID,
        //     date: context.startTime!,
        //     version: API_VERSION,
        //     ...blob.properties,
        //     contentLength,
        //     contentMD5: undefined
        //   };
        // }
        blob.pageRangesInOrder = blob.pageRangesInOrder || [];
        const ranges = contentLength <= 0
            ? []
            : this.rangesManager.fillZeroRanges(blob.pageRangesInOrder, {
                start: rangeStart,
                end: rangeEnd
            });
        const bodyGetter = async () => {
            return this.dataStore.readPayloads(ranges.map(value => value.persistency), 0, contentLength);
        };
        let body = await bodyGetter();
        let contentMD5;
        if (!partialRead) {
            contentMD5 = blob.properties.contentMD5;
        }
        if (contentLength <= 4 * 1024 * 1024 &&
            contentMD5 === undefined &&
            body !== undefined) {
            contentMD5 = await utils_1.getMD5FromStream(body);
            body = await bodyGetter();
        }
        const response = Object.assign({ statusCode: rangesParts[1] === Infinity ? 200 : 206, body, metadata: blob.metadata, eTag: blob.properties.etag, requestId: context.contextID, date: context.startTime, version: constants_1.API_VERSION }, blob.properties, { contentLength,
            contentMD5, blobContentMD5: blob.properties.contentMD5, isServerEncrypted: true });
        return response;
    }
    /**
     * Get blob object from persistency layer according to request context.
     *
     * @private
     * @param {Context} context
     * @param {string} [snapshot=""]
     * @returns {Promise<BlobModel>}
     * @memberof BlobHandler
     */
    async getSimpleBlobFromStorage(context, snapshot = "") {
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        const containerName = blobCtx.container;
        const blobName = blobCtx.blob;
        const container = await this.dataStore.getContainer(accountName, containerName);
        if (!container) {
            throw StorageErrorFactory_1.default.getContainerNotFound(blobCtx.contextID);
        }
        let blob = await this.dataStore.getBlob(accountName, containerName, blobName, snapshot);
        if (!blob) {
            throw StorageErrorFactory_1.default.getBlobNotFound(blobCtx.contextID);
        }
        blob = BlobHandler.updateLeaseAttributes(blob, blobCtx.startTime);
        return blob;
    }
}
exports.default = BlobHandler;
//# sourceMappingURL=BlobHandler.js.map