"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const BlobStorageContext_1 = tslib_1.__importDefault(require("../context/BlobStorageContext"));
const NotImplementedError_1 = tslib_1.__importDefault(require("../errors/NotImplementedError"));
const StorageErrorFactory_1 = tslib_1.__importDefault(require("../errors/StorageErrorFactory"));
const Models = tslib_1.__importStar(require("../generated/artifacts/models"));
const xml_1 = require("../generated/utils/xml");
const constants_1 = require("../utils/constants");
const utils_1 = require("../utils/utils");
const BaseHandler_1 = tslib_1.__importDefault(require("./BaseHandler"));
const BlobHandler_1 = tslib_1.__importDefault(require("./BlobHandler"));
/**
 * BlobHandler handles Azure Storage BlockBlob related requests.
 *
 * @export
 * @class BlockBlobHandler
 * @extends {BaseHandler}
 * @implements {IBlockBlobHandler}
 */
class BlockBlobHandler extends BaseHandler_1.default {
    async upload(body, contentLength, options, context) {
        // TODO: Check Lease status, and set to available if it's expired, see sample in BlobHandler.setMetadata()
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        const containerName = blobCtx.container;
        const blobName = blobCtx.blob;
        const date = context.startTime;
        const etag = utils_1.newEtag();
        options.blobHTTPHeaders = options.blobHTTPHeaders || {};
        const contentType = options.blobHTTPHeaders.blobContentType ||
            context.request.getHeader("content-type") ||
            "application/octet-stream";
        const contentMD5 = options.blobHTTPHeaders.blobContentMD5 ||
            context.request.getHeader("content-md5");
        const container = await this.dataStore.getContainer(accountName, containerName);
        if (!container) {
            throw StorageErrorFactory_1.default.getContainerNotFound(blobCtx.contextID);
        }
        const persistency = await this.dataStore.writePayload(body);
        // Calculate MD5 for validation
        const stream = await this.dataStore.readPayload(persistency);
        const calculatedContentMD5 = await utils_1.getMD5FromStream(stream);
        if (contentMD5 !== undefined) {
            if (typeof contentMD5 === "string") {
                const calculatedContentMD5String = Buffer.from(calculatedContentMD5).toString("base64");
                if (contentMD5 !== calculatedContentMD5String) {
                    throw StorageErrorFactory_1.default.getInvalidOperation(context.contextID, "Provided contentMD5 doesn't match.");
                }
            }
            else {
                if (contentMD5 !== undefined &&
                    !Buffer.from(contentMD5).equals(calculatedContentMD5)) {
                    throw StorageErrorFactory_1.default.getInvalidOperation(context.contextID, "Provided contentMD5 doesn't match.");
                }
            }
        }
        const blob = {
            deleted: false,
            metadata: options.metadata,
            accountName,
            containerName,
            name: blobName,
            properties: {
                creationTime: date,
                lastModified: date,
                etag,
                contentLength,
                contentType,
                contentEncoding: options.blobHTTPHeaders.blobContentEncoding,
                contentLanguage: options.blobHTTPHeaders.blobContentLanguage,
                contentMD5: calculatedContentMD5,
                contentDisposition: options.blobHTTPHeaders.blobContentDisposition,
                cacheControl: options.blobHTTPHeaders.blobCacheControl,
                blobType: Models.BlobType.BlockBlob,
                leaseStatus: Models.LeaseStatusType.Unlocked,
                leaseState: Models.LeaseStateType.Available,
                serverEncrypted: true,
                accessTier: Models.AccessTier.Hot,
                accessTierInferred: true
            },
            snapshot: "",
            isCommitted: true,
            persistency
        };
        // TODO: Need a lock for multi keys including containerName and blobName
        await this.dataStore.updateBlob(blob);
        const response = {
            statusCode: 201,
            eTag: etag,
            lastModified: date,
            contentMD5: blob.properties.contentMD5,
            requestId: blobCtx.contextID,
            version: constants_1.API_VERSION,
            date,
            isServerEncrypted: true
        };
        return response;
    }
    async stageBlock(blockId, contentLength, body, options, context) {
        // TODO: Check Lease status, and set to available if it's expired, see sample in BlobHandler.setMetadata()
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        const containerName = blobCtx.container;
        const blobName = blobCtx.blob;
        const date = blobCtx.startTime;
        const container = await this.dataStore.getContainer(accountName, containerName);
        if (!container) {
            throw StorageErrorFactory_1.default.getContainerNotFound(blobCtx.contextID);
        }
        const persistency = await this.dataStore.writePayload(body);
        const block = {
            accountName,
            containerName,
            blobName,
            isCommitted: false,
            name: blockId,
            size: contentLength,
            persistency
        };
        await this.dataStore.updateBlock(block);
        // Create an uncommitted block blob if doesn't exist
        // TODO: Lock
        let blob = await this.dataStore.getBlob(accountName, containerName, blobName);
        if (!blob) {
            const etag = utils_1.newEtag();
            blob = {
                deleted: false,
                accountName,
                containerName,
                name: blobName,
                properties: {
                    creationTime: date,
                    lastModified: date,
                    etag,
                    contentLength,
                    blobType: Models.BlobType.BlockBlob
                },
                snapshot: "",
                isCommitted: false
            };
            await this.dataStore.updateBlob(blob);
        }
        // TODO: Unlock
        const response = {
            statusCode: 201,
            contentMD5: undefined,
            requestId: blobCtx.contextID,
            version: constants_1.API_VERSION,
            date,
            isServerEncrypted: true
        };
        return response;
    }
    async stageBlockFromURL(blockId, contentLength, sourceUrl, options, context) {
        throw new NotImplementedError_1.default(context.contextID);
    }
    async commitBlockList(blocks, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        const containerName = blobCtx.container;
        const blobName = blobCtx.blob;
        const request = blobCtx.request;
        const container = await this.dataStore.getContainer(accountName, containerName);
        if (!container) {
            throw StorageErrorFactory_1.default.getContainerNotFound(blobCtx.contextID);
        }
        // TODO: Lock for container and blob
        let blob = await this.dataStore.getBlob(accountName, containerName, blobName);
        if (!blob) {
            // At least there should be a uncommitted blob
            // If not, there are some error happens
            // TODO: Which error should be thrown ere?
            // TODO: Unlock
            throw StorageErrorFactory_1.default.getBlobNotFound(blobCtx.contextID);
        }
        // Check Lease status
        BlobHandler_1.default.checkBlobLeaseOnWriteBlob(context, blob, options.leaseAccessConditions);
        options.blobHTTPHeaders = options.blobHTTPHeaders || {};
        const contentType = options.blobHTTPHeaders.blobContentType ||
            context.request.getHeader("content-type") ||
            "application/octet-stream";
        // Get all blocks in persistency layer
        const pUncommittedBlocks = await this.dataStore.listBlocks(accountName, containerName, blobName, false);
        const pCommittedBlocksMap = new Map(); // persistencyCommittedBlocksMap
        for (const pBlock of blob.committedBlocksInOrder || []) {
            pCommittedBlocksMap.set(pBlock.name, Object.assign({}, pBlock, { accountName,
                containerName,
                blobName, isCommitted: true }));
        }
        const pUncommittedBlocksMap = new Map(); // persistencyUncommittedBlocksMap
        for (const pBlock of pUncommittedBlocks) {
            if (!pBlock.isCommitted) {
                pUncommittedBlocksMap.set(pBlock.name, pBlock);
            }
        }
        // Here we leveraged generated code utils to parser xml
        // Re-parsing request body to get destination blocks
        // We don't leverage serialized blocks parameter because it doesn't include sequence
        const rawBody = request.getBody();
        const badRequestError = StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID);
        if (rawBody === undefined) {
            throw badRequestError;
        }
        const parsed = await xml_1.parseXML(rawBody, true);
        // Validate selected block list
        const selectedBlockList = [];
        // $$ is the built-in field of xml2js parsing results when enabling explicitChildrenWithOrder
        // TODO: Should make these fields explicit for parseXML method
        // TODO: What happens when committedBlocks and uncommittedBlocks contains same block ID?
        if (parsed !== undefined && parsed.$$ instanceof Array) {
            for (const block of parsed.$$) {
                const blockID = block._;
                const blockCommitType = block["#name"];
                if (blockID === undefined || blockCommitType === undefined) {
                    throw badRequestError;
                }
                switch (blockCommitType.toLowerCase()) {
                    case "uncommitted":
                        const pUncommittedBlock = pUncommittedBlocksMap.get(blockID);
                        if (pUncommittedBlock === undefined) {
                            throw badRequestError;
                        }
                        else {
                            pUncommittedBlock.isCommitted = true;
                            selectedBlockList.push(pUncommittedBlock);
                        }
                        break;
                    case "committed":
                        const pCommittedBlock = pCommittedBlocksMap.get(blockID);
                        if (pCommittedBlock === undefined) {
                            throw badRequestError;
                        }
                        else {
                            selectedBlockList.push(pCommittedBlock);
                        }
                        break;
                    case "latest":
                        const pLatestBlock = pUncommittedBlocksMap.get(blockID) ||
                            pCommittedBlocksMap.get(blockID);
                        if (pLatestBlock === undefined) {
                            throw badRequestError;
                        }
                        else {
                            pLatestBlock.isCommitted = true;
                            selectedBlockList.push(pLatestBlock);
                        }
                        break;
                    default:
                        throw badRequestError;
                }
            }
        }
        // Commit block list
        blob.committedBlocksInOrder = selectedBlockList;
        blob.isCommitted = true;
        blob.metadata = options.metadata;
        blob.properties.accessTier = Models.AccessTier.Hot;
        blob.properties.accessTierInferred = true;
        blob.properties.cacheControl = options.blobHTTPHeaders.blobCacheControl;
        blob.properties.contentType = contentType;
        blob.properties.contentMD5 = options.blobHTTPHeaders.blobContentMD5;
        blob.properties.contentEncoding =
            options.blobHTTPHeaders.blobContentEncoding;
        blob.properties.contentLanguage =
            options.blobHTTPHeaders.blobContentLanguage;
        blob.properties.contentDisposition =
            options.blobHTTPHeaders.blobContentDisposition;
        blob.properties.contentLength = selectedBlockList
            .map(block => block.size)
            .reduce((total, val) => {
            return total + val;
        });
        // set lease state to available if it's expired
        blob = BlobHandler_1.default.UpdateBlobLeaseStateOnWriteBlob(blob);
        await this.dataStore.updateBlob(blob);
        // TODO: recover deleted blocks when inserts failed
        await this.dataStore.deleteBlocks(accountName, containerName, blobName);
        // TODO: Unlock
        const contentMD5 = await utils_1.getMD5FromString(rawBody);
        const response = {
            statusCode: 201,
            eTag: utils_1.newEtag(),
            lastModified: blobCtx.startTime,
            contentMD5,
            requestId: blobCtx.contextID,
            version: constants_1.API_VERSION,
            date: blobCtx.startTime,
            isServerEncrypted: true
        };
        return response;
    }
    async getBlockList(options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        const containerName = blobCtx.container;
        const blobName = blobCtx.blob;
        const date = blobCtx.startTime;
        const container = await this.dataStore.getContainer(accountName, containerName);
        if (!container) {
            throw StorageErrorFactory_1.default.getContainerNotFound(blobCtx.contextID);
        }
        const blob = await this.dataStore.getBlob(accountName, containerName, blobName);
        if (!blob) {
            throw StorageErrorFactory_1.default.getBlobNotFound(blobCtx.contextID);
        }
        const blockList = await this.dataStore.listBlocks(accountName, containerName, blobName, false);
        const response = {
            statusCode: 200,
            lastModified: blob.properties.lastModified,
            eTag: blob.properties.etag,
            contentType: blob.properties.contentType,
            blobContentLength: blob.properties.contentLength,
            requestId: blobCtx.contextID,
            version: constants_1.API_VERSION,
            date,
            committedBlocks: [],
            uncommittedBlocks: []
        };
        response.uncommittedBlocks = blockList;
        response.committedBlocks = blob.committedBlocksInOrder;
        return response;
    }
}
exports.default = BlockBlobHandler;
//# sourceMappingURL=BlockBlobHandler.js.map