"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const v4_1 = tslib_1.__importDefault(require("uuid/v4"));
const Mutex_1 = tslib_1.__importDefault(require("../../common/Mutex"));
const BlobStorageContext_1 = tslib_1.__importDefault(require("../context/BlobStorageContext"));
const StorageErrorFactory_1 = tslib_1.__importDefault(require("../errors/StorageErrorFactory"));
const Models = tslib_1.__importStar(require("../generated/artifacts/models"));
const constants_1 = require("../utils/constants");
const utils_1 = require("../utils/utils");
const BaseHandler_1 = tslib_1.__importDefault(require("./BaseHandler"));
/**
 * ContainerHandler handles Azure Storage Blob container related requests.
 *
 * @export
 * @class ContainerHandler
 * @implements {IHandler}
 */
class ContainerHandler extends BaseHandler_1.default {
    constructor() {
        super(...arguments);
        /**
         * Default listing blobs max number.
         *
         * @private
         * @memberof ContainerHandler
         */
        this.LIST_BLOBS_MAX_RESULTS_DEFAULT = 5000;
    }
    /**
     * create container
     *
     * @param {Models.ContainerCreateOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerCreateResponse>}
     * @memberof ContainerHandler
     */
    async create(options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        const containerName = blobCtx.container;
        const lastModified = blobCtx.startTime;
        const etag = utils_1.newEtag();
        const container = await this.dataStore.getContainer(accountName, containerName);
        if (container) {
            throw StorageErrorFactory_1.default.getContainerAlreadyExists(blobCtx.contextID);
        }
        await this.dataStore.updateContainer({
            accountName,
            metadata: options.metadata,
            name: containerName,
            properties: {
                etag,
                lastModified,
                leaseStatus: Models.LeaseStatusType.Unlocked,
                leaseState: Models.LeaseStateType.Available,
                publicAccess: options.access,
                hasImmutabilityPolicy: false,
                hasLegalHold: false
            }
        });
        const response = {
            eTag: etag,
            lastModified,
            requestId: blobCtx.contextID,
            statusCode: 201,
            version: constants_1.API_VERSION
        };
        return response;
    }
    /**
     * get container properties
     *
     * @param {Models.ContainerGetPropertiesOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerGetPropertiesResponse>}
     * @memberof ContainerHandler
     */
    async getProperties(options, context) {
        const container = await this.getSimpleContainerFromStorage(context);
        const response = Object.assign({ eTag: container.properties.etag }, container.properties, { blobPublicAccess: container.properties.publicAccess, metadata: container.metadata, requestId: context.contextID, statusCode: 200, version: constants_1.API_VERSION });
        return response;
    }
    /**
     * get container properties with headers
     *
     * @param {Models.ContainerGetPropertiesOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerGetPropertiesResponse>}
     * @memberof ContainerHandler
     */
    async getPropertiesWithHead(options, context) {
        return this.getProperties(options, context);
    }
    /**
     * delete container
     *
     * @param {Models.ContainerDeleteMethodOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerDeleteResponse>}
     * @memberof ContainerHandler
     */
    async delete(options, context) {
        const container = await this.getSimpleContainerFromStorage(context);
        // Check Lease status
        if (container.properties.leaseStatus === Models.LeaseStatusType.Locked) {
            if (options.leaseAccessConditions === undefined ||
                options.leaseAccessConditions.leaseId === undefined ||
                options.leaseAccessConditions.leaseId === null) {
                const blobCtx = new BlobStorageContext_1.default(context);
                throw StorageErrorFactory_1.default.getContainerLeaseIdMissing(blobCtx.contextID);
            }
            else if (container.leaseId !== undefined &&
                options.leaseAccessConditions.leaseId.toLowerCase() !==
                    container.leaseId.toLowerCase()) {
                const blobCtx = new BlobStorageContext_1.default(context);
                throw StorageErrorFactory_1.default.getContainerLeaseIdMismatchWithContainerOperation(blobCtx.contextID);
            }
        }
        else if (options.leaseAccessConditions !== undefined &&
            options.leaseAccessConditions.leaseId !== undefined &&
            options.leaseAccessConditions.leaseId !== null &&
            options.leaseAccessConditions.leaseId !== "") {
            const blobCtx = new BlobStorageContext_1.default(context);
            throw StorageErrorFactory_1.default.getBlobLeaseLost(blobCtx.contextID);
        }
        // TODO: Mark container as being deleted status, then (mark) delete all blobs async
        // When above finishes, execute following delete container operation
        // Because following delete container operation will only delete DB metadata for container and
        // blobs under the container, but will not clean up blob data in disk
        await this.dataStore.deleteContainer(container.accountName, container.name);
        await this.dataStore.deleteBlobs(container.accountName, container.name);
        const response = {
            date: context.startTime,
            requestId: context.contextID,
            statusCode: 202,
            version: constants_1.API_VERSION
        };
        return response;
    }
    /**
     * set container metadata
     *
     * @param {Models.ContainerSetMetadataOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerSetMetadataResponse>}
     * @memberof ContainerHandler
     */
    async setMetadata(options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const containerName = blobCtx.container;
        let container;
        await Mutex_1.default.lock(containerName);
        try {
            container = await this.getSimpleContainerFromStorage(context);
        }
        catch (error) {
            await Mutex_1.default.unlock(containerName);
            throw error;
        }
        container.metadata = options.metadata;
        container.properties.lastModified = blobCtx.startTime;
        await this.dataStore.updateContainer(container);
        await Mutex_1.default.unlock(containerName);
        const response = {
            date: container.properties.lastModified,
            eTag: utils_1.newEtag(),
            lastModified: container.properties.lastModified,
            requestId: context.contextID,
            statusCode: 200
        };
        return response;
    }
    /**
     * Get container access policy
     *
     * @param {Models.ContainerGetAccessPolicyOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerGetAccessPolicyResponse>}
     * @memberof ContainerHandler
     */
    async getAccessPolicy(options, context) {
        // const blobCtx = new BlobStorageContext(context);
        // const containerName = blobCtx.container!;
        let container;
        // await Mutex.lock(containerName);
        try {
            container = await this.getSimpleContainerFromStorage(context);
        }
        catch (error) {
            // await Mutex.unlock(containerName);
            throw error;
        }
        this.checkLeaseOnReadContainer(context, container, options.leaseAccessConditions);
        const response = [];
        const responseArray = response;
        const responseObject = response;
        if (container.containerAcl !== undefined) {
            responseArray.push(...container.containerAcl);
        }
        responseObject.date = container.properties.lastModified;
        responseObject.blobPublicAccess = container.properties.publicAccess;
        responseObject.eTag = utils_1.newEtag();
        responseObject.lastModified = container.properties.lastModified;
        responseObject.requestId = context.contextID;
        responseObject.version = constants_1.API_VERSION;
        responseObject.statusCode = 200;
        // TODO: Need fix generator code since the output containerAcl can't be serialized correctly
        // tslint:disable-next-line:max-line-length
        // Correct responds： <?xml version="1.0" encoding="utf-8"?><SignedIdentifiers><SignedIdentifier><Id>123</Id><AccessPolicy><Start>2019-04-30T16:00:00.0000000Z</Start><Expiry>2019-12-31T16:00:00.0000000Z</Expiry><Permission>r</Permission></AccessPolicy></SignedIdentifier></SignedIdentifiers>
        // tslint:disable-next-line:max-line-length
        // Current responds: <?xml version="1.0" encoding="UTF-8" standalone="yes"?><parsedResponse><Id>123</Id><AccessPolicy><Start>2019-04-30T16:00:00Z</Start><Expiry>2019-12-31T16:00:00Z</Expiry><Permission>r</Permission></AccessPolicy></parsedResponse>"
        return response;
    }
    /**
     * set container access policy
     *
     * @param {Models.ContainerSetAccessPolicyOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerSetAccessPolicyResponse>}
     * @memberof ContainerHandler
     */
    async setAccessPolicy(options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const containerName = blobCtx.container;
        let container;
        await Mutex_1.default.lock(containerName);
        try {
            container = await this.getSimpleContainerFromStorage(context);
        }
        catch (error) {
            await Mutex_1.default.unlock(containerName);
            throw error;
        }
        // from rest spec, this is how lease works
        this.checkLeaseOnReadContainer(context, container, options.leaseAccessConditions);
        container.properties.publicAccess = options.access;
        container.containerAcl = options.containerAcl;
        container.properties.lastModified = blobCtx.startTime;
        await this.dataStore.updateContainer(container);
        await Mutex_1.default.unlock(containerName);
        const response = {
            date: container.properties.lastModified,
            eTag: utils_1.newEtag(),
            lastModified: container.properties.lastModified,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            statusCode: 200
        };
        return response;
    }
    /**
     * acquire container lease
     *
     * @param {Models.ContainerAcquireLeaseOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerAcquireLeaseResponse>}
     * @memberof ContainerHandler
     */
    async acquireLease(options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const containerName = blobCtx.container;
        let container;
        await Mutex_1.default.lock(containerName);
        try {
            container = await this.getSimpleContainerFromStorage(context);
        }
        catch (error) {
            await Mutex_1.default.unlock(containerName);
            throw error;
        }
        // check the lease action aligned with current lease state.
        if (container.properties.leaseState === Models.LeaseStateType.Breaking) {
            await Mutex_1.default.unlock(containerName);
            throw StorageErrorFactory_1.default.getLeaseAlreadyPresent(blobCtx.contextID);
        }
        if (container.properties.leaseState === Models.LeaseStateType.Leased &&
            options.proposedLeaseId !== container.leaseId) {
            await Mutex_1.default.unlock(containerName);
            throw StorageErrorFactory_1.default.getLeaseAlreadyPresent(context.contextID);
        }
        // update the lease information
        if (options.duration === -1 || options.duration === undefined) {
            container.properties.leaseDuration = Models.LeaseDurationType.Infinite;
        }
        else {
            // verify options.duration between 15 and 60
            if (options.duration > 60 || options.duration < 15) {
                await Mutex_1.default.unlock(containerName);
                throw StorageErrorFactory_1.default.getInvalidLeaseDuration(blobCtx.contextID);
            }
            container.properties.leaseDuration = Models.LeaseDurationType.Fixed;
            container.leaseExpireTime = blobCtx.startTime;
            container.leaseExpireTime.setSeconds(container.leaseExpireTime.getSeconds() + options.duration);
            container.leaseduration = options.duration;
        }
        container.properties.leaseState = Models.LeaseStateType.Leased;
        container.properties.leaseStatus = Models.LeaseStatusType.Locked;
        container.leaseId =
            options.proposedLeaseId !== "" && options.proposedLeaseId !== undefined
                ? options.proposedLeaseId
                : v4_1.default();
        container.leaseBreakExpireTime = undefined;
        await this.dataStore.updateContainer(container);
        await Mutex_1.default.unlock(containerName);
        const response = {
            date: blobCtx.startTime,
            eTag: container.properties.etag,
            lastModified: container.properties.lastModified,
            leaseId: container.leaseId,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            statusCode: 201
        };
        return response;
    }
    /**
     * release container lease
     *
     * @param {string} leaseId
     * @param {Models.ContainerReleaseLeaseOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerReleaseLeaseResponse>}
     * @memberof ContainerHandler
     */
    async releaseLease(leaseId, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const containerName = blobCtx.container;
        let container;
        await Mutex_1.default.lock(containerName);
        try {
            container = await this.getSimpleContainerFromStorage(context);
        }
        catch (error) {
            await Mutex_1.default.unlock(containerName);
            throw error;
        }
        // check the lease action aligned with current lease state.
        if (container.properties.leaseState === Models.LeaseStateType.Available) {
            await Mutex_1.default.unlock(containerName);
            throw StorageErrorFactory_1.default.getContainerLeaseIdMismatchWithLeaseOperation(blobCtx.contextID);
        }
        // Check lease ID
        if (container.leaseId !== leaseId) {
            await Mutex_1.default.unlock(containerName);
            throw StorageErrorFactory_1.default.getContainerLeaseIdMismatchWithLeaseOperation(blobCtx.contextID);
        }
        // update the lease information
        container.properties.leaseState = Models.LeaseStateType.Available;
        container.properties.leaseStatus = Models.LeaseStatusType.Unlocked;
        container.properties.leaseDuration = undefined;
        container.leaseduration = undefined;
        container.leaseId = undefined;
        container.leaseExpireTime = undefined;
        container.leaseBreakExpireTime = undefined;
        await this.dataStore.updateContainer(container);
        await Mutex_1.default.unlock(containerName);
        const response = {
            date: blobCtx.startTime,
            eTag: container.properties.etag,
            lastModified: container.properties.lastModified,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            statusCode: 200
        };
        return response;
    }
    /**
     * renew container lease
     *
     * @param {string} leaseId
     * @param {Models.ContainerRenewLeaseOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerRenewLeaseResponse>}
     * @memberof ContainerHandler
     */
    async renewLease(leaseId, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const containerName = blobCtx.container;
        let container;
        await Mutex_1.default.lock(containerName);
        try {
            container = await this.getSimpleContainerFromStorage(context);
        }
        catch (error) {
            await Mutex_1.default.unlock(containerName);
            throw error;
        }
        // check the lease action aligned with current lease state.
        if (container.properties.leaseState === Models.LeaseStateType.Available) {
            await Mutex_1.default.unlock(containerName);
            throw StorageErrorFactory_1.default.getContainerLeaseIdMismatchWithLeaseOperation(blobCtx.contextID);
        }
        if (container.properties.leaseState === Models.LeaseStateType.Breaking ||
            container.properties.leaseState === Models.LeaseStateType.Broken) {
            await Mutex_1.default.unlock(containerName);
            throw StorageErrorFactory_1.default.getLeaseIsBrokenAndCannotBeRenewed(blobCtx.contextID);
        }
        // Check lease ID
        if (container.leaseId !== leaseId) {
            await Mutex_1.default.unlock(containerName);
            throw StorageErrorFactory_1.default.getContainerLeaseIdMismatchWithLeaseOperation(blobCtx.contextID);
        }
        // update the lease information
        container.properties.leaseState = Models.LeaseStateType.Leased;
        container.properties.leaseStatus = Models.LeaseStatusType.Locked;
        // when container.leaseduration has value (not -1), means fixed duration
        if (container.leaseduration !== undefined &&
            container.leaseduration !== -1) {
            container.leaseExpireTime = blobCtx.startTime;
            container.leaseExpireTime.setSeconds(container.leaseExpireTime.getSeconds() + container.leaseduration);
            container.properties.leaseDuration = Models.LeaseDurationType.Fixed;
        }
        else {
            container.properties.leaseDuration = Models.LeaseDurationType.Infinite;
        }
        await this.dataStore.updateContainer(container);
        await Mutex_1.default.unlock(containerName);
        const response = {
            date: blobCtx.startTime,
            eTag: container.properties.etag,
            lastModified: container.properties.lastModified,
            leaseId: container.leaseId,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            statusCode: 200
        };
        return response;
    }
    /**
     * break container lease
     *
     * @param {Models.ContainerBreakLeaseOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerBreakLeaseResponse>}
     * @memberof ContainerHandler
     */
    async breakLease(options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const containerName = blobCtx.container;
        let container;
        let leaseTimeinSecond;
        leaseTimeinSecond = 0;
        await Mutex_1.default.lock(containerName);
        try {
            container = await this.getSimpleContainerFromStorage(context);
        }
        catch (error) {
            await Mutex_1.default.unlock(containerName);
            throw error;
        }
        // check the lease action aligned with current lease state.
        if (container.properties.leaseState === Models.LeaseStateType.Available) {
            await Mutex_1.default.unlock(containerName);
            throw StorageErrorFactory_1.default.getContainerLeaseNotPresentWithLeaseOperation(blobCtx.contextID);
        }
        // update the lease information
        // verify options.breakPeriod between 0 and 60
        if (options.breakPeriod !== undefined &&
            (options.breakPeriod > 60 || options.breakPeriod < 0)) {
            await Mutex_1.default.unlock(containerName);
            throw StorageErrorFactory_1.default.getInvalidLeaseBreakPeriod(blobCtx.contextID);
        }
        if (container.properties.leaseState === Models.LeaseStateType.Expired ||
            container.properties.leaseState === Models.LeaseStateType.Broken ||
            options.breakPeriod === 0 ||
            options.breakPeriod === undefined) {
            container.properties.leaseState = Models.LeaseStateType.Broken;
            container.properties.leaseStatus = Models.LeaseStatusType.Unlocked;
            container.properties.leaseDuration = undefined;
            container.leaseduration = undefined;
            container.leaseExpireTime = undefined;
            container.leaseBreakExpireTime = undefined;
            leaseTimeinSecond = 0;
        }
        else {
            container.properties.leaseState = Models.LeaseStateType.Breaking;
            container.properties.leaseStatus = Models.LeaseStatusType.Locked;
            container.leaseduration = undefined;
            if (container.properties.leaseDuration === Models.LeaseDurationType.Infinite) {
                container.properties.leaseDuration = undefined;
                container.leaseExpireTime = undefined;
                container.leaseBreakExpireTime = new Date(blobCtx.startTime);
                container.leaseBreakExpireTime.setSeconds(container.leaseBreakExpireTime.getSeconds() + options.breakPeriod);
                leaseTimeinSecond = options.breakPeriod;
            }
            else {
                let newleaseBreakExpireTime = new Date(blobCtx.startTime);
                newleaseBreakExpireTime.setSeconds(newleaseBreakExpireTime.getSeconds() + options.breakPeriod);
                if (container.leaseExpireTime !== undefined &&
                    newleaseBreakExpireTime > container.leaseExpireTime) {
                    newleaseBreakExpireTime = container.leaseExpireTime;
                }
                if (container.leaseBreakExpireTime === undefined ||
                    container.leaseBreakExpireTime > newleaseBreakExpireTime) {
                    container.leaseBreakExpireTime = newleaseBreakExpireTime;
                }
                leaseTimeinSecond = Math.round(Math.abs(container.leaseBreakExpireTime.getTime() -
                    blobCtx.startTime.getTime()) / 1000);
                container.leaseExpireTime = undefined;
                container.properties.leaseDuration = undefined;
            }
        }
        await this.dataStore.updateContainer(container);
        await Mutex_1.default.unlock(containerName);
        const response = {
            date: blobCtx.startTime,
            eTag: container.properties.etag,
            lastModified: container.properties.lastModified,
            leaseTime: leaseTimeinSecond,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            statusCode: 202
        };
        return response;
    }
    /**
     * change container lease
     *
     * @param {string} leaseId
     * @param {string} proposedLeaseId
     * @param {Models.ContainerChangeLeaseOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerChangeLeaseResponse>}
     * @memberof ContainerHandler
     */
    async changeLease(leaseId, proposedLeaseId, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const containerName = blobCtx.container;
        let container;
        await Mutex_1.default.lock(containerName);
        try {
            container = await this.getSimpleContainerFromStorage(context);
        }
        catch (error) {
            await Mutex_1.default.unlock(containerName);
            throw error;
        }
        // check the lease action aligned with current lease state.
        if (container.properties.leaseState === Models.LeaseStateType.Available ||
            container.properties.leaseState === Models.LeaseStateType.Expired ||
            container.properties.leaseState === Models.LeaseStateType.Broken) {
            await Mutex_1.default.unlock(containerName);
            throw StorageErrorFactory_1.default.getContainerLeaseNotPresentWithLeaseOperation(blobCtx.contextID);
        }
        if (container.properties.leaseState === Models.LeaseStateType.Breaking) {
            await Mutex_1.default.unlock(containerName);
            throw StorageErrorFactory_1.default.getLeaseIsBreakingAndCannotBeChanged(blobCtx.contextID);
        }
        // Check lease ID
        if (container.leaseId !== leaseId &&
            container.leaseId !== proposedLeaseId) {
            await Mutex_1.default.unlock(containerName);
            throw StorageErrorFactory_1.default.getContainerLeaseIdMismatchWithLeaseOperation(blobCtx.contextID);
        }
        // update the lease information, only need update lease ID
        container.leaseId = proposedLeaseId;
        await this.dataStore.updateContainer(container);
        await Mutex_1.default.unlock(containerName);
        const response = {
            date: blobCtx.startTime,
            eTag: container.properties.etag,
            lastModified: container.properties.lastModified,
            leaseId: container.leaseId,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            statusCode: 200
        };
        return response;
    }
    /**
     * list blobs flat segments
     *
     * @param {Models.ContainerListBlobFlatSegmentOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerListBlobFlatSegmentResponse>}
     * @memberof ContainerHandler
     */
    async listBlobFlatSegment(options, context) {
        const container = await this.getSimpleContainerFromStorage(context);
        const request = context.request;
        const accountName = container.accountName;
        const containerName = container.name;
        const marker = parseInt(options.marker || "0", 10);
        const delimiter = "";
        options.marker = options.marker || "";
        let includeSnapshots = false;
        if (options.include !== undefined) {
            if (options.include.includes(Models.ListBlobsIncludeItem.Snapshots)) {
                includeSnapshots = true;
            }
        }
        const [blobs, nextMarker] = await this.dataStore.listBlobs(accountName, containerName, undefined, options.prefix, options.maxresults, marker, includeSnapshots);
        const blobItems = [];
        for (const blob of blobs) {
            blob.deleted = blob.deleted !== true ? undefined : true;
            blobItems.push(blob);
        }
        // only need an empty array for the prefixes
        const blobPrefixes = [];
        const serviceEndpoint = `${request.getEndpoint()}/${accountName}`;
        const response = {
            statusCode: 200,
            contentType: "application/xml",
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            date: context.startTime,
            serviceEndpoint,
            containerName,
            prefix: options.prefix || "",
            marker: options.marker,
            maxResults: options.maxresults || this.LIST_BLOBS_MAX_RESULTS_DEFAULT,
            delimiter,
            segment: {
                blobItems,
                blobPrefixes
            },
            nextMarker: `${nextMarker || ""}`
        };
        return response;
    }
    /**
     * List blobs hierarchy.
     *
     * @param {string} delimiter
     * @param {Models.ContainerListBlobHierarchySegmentOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ContainerListBlobHierarchySegmentResponse>}
     * @memberof ContainerHandler
     */
    async listBlobHierarchySegment(delimiter, options, context) {
        // TODO: Need update list out blobs lease properties with BlobHandler.updateLeaseAttributes()
        const container = await this.getSimpleContainerFromStorage(context);
        const request = context.request;
        const accountName = container.accountName;
        const containerName = container.name;
        const marker = parseInt(options.marker || "0", 10);
        delimiter = delimiter === "" ? "/" : delimiter;
        options.prefix = options.prefix || "";
        options.marker = options.marker || "";
        let includeSnapshots = false;
        if (options.include !== undefined) {
            if (options.include.includes(Models.ListBlobsIncludeItem.Snapshots)) {
                includeSnapshots = true;
            }
        }
        const [blobs, nextMarker] = await this.dataStore.listBlobs(accountName, containerName, undefined, options.prefix, options.maxresults, marker, includeSnapshots);
        const blobItems = [];
        const blobPrefixes = [];
        const blobPrefixesSet = new Set();
        const prefixLength = options.prefix.length;
        for (const blob of blobs) {
            const delimiterPosAfterPrefix = blob.name.indexOf(delimiter, prefixLength);
            // This is a blob
            if (delimiterPosAfterPrefix < 0) {
                blob.deleted = blob.deleted !== true ? undefined : true;
                blobItems.push(blob);
            }
            else {
                // This is a prefix
                const prefix = blob.name.substr(0, delimiterPosAfterPrefix + 1);
                blobPrefixesSet.add(prefix);
            }
        }
        const iter = blobPrefixesSet.values();
        let val;
        while (!(val = iter.next()).done) {
            blobPrefixes.push({ name: val.value });
        }
        const serviceEndpoint = `${request.getEndpoint()}/${accountName}`;
        const response = {
            statusCode: 200,
            contentType: "application/xml",
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            date: context.startTime,
            serviceEndpoint,
            containerName,
            prefix: options.prefix,
            marker: options.marker,
            maxResults: options.maxresults || this.LIST_BLOBS_MAX_RESULTS_DEFAULT,
            delimiter,
            segment: {
                blobItems,
                blobPrefixes
            },
            nextMarker: `${nextMarker || ""}`
        };
        return response;
    }
    /**
     * get account info
     *
     * @param {Context} context
     * @returns {Promise<Models.ContainerGetAccountInfoResponse>}
     * @memberof ContainerHandler
     */
    async getAccountInfo(context) {
        return utils_1.getContainerGetAccountInfoResponse(context);
    }
    /**
     * get account info with headers
     *
     * @param {Context} context
     * @returns {Promise<Models.ContainerGetAccountInfoResponse>}
     * @memberof ContainerHandler
     */
    async getAccountInfoWithHead(context) {
        return utils_1.getContainerGetAccountInfoResponse(context);
    }
    /**
     * Get container object from persistency layer according to request context.
     *
     * @private
     * @param {Context} context
     * @returns {Promise<BlobModel>}
     * @memberof ContainerHandler
     */
    async getSimpleContainerFromStorage(context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        const containerName = blobCtx.container;
        let container = await this.dataStore.getContainer(accountName, containerName);
        if (!container) {
            throw StorageErrorFactory_1.default.getContainerNotFound(blobCtx.contextID);
        }
        container = this.updateLeaseAttributes(container, blobCtx.startTime);
        return container;
    }
    /**
     * Update container lease Attributes according to the current time.
     * The Attribute not set back
     *
     * @private
     * @param {ContainerModel} container
     * @param {Date} currentTime
     * @returns {ContainerModel}
     * @memberof ContainerHandler
     */
    updateLeaseAttributes(container, currentTime) {
        // check Leased -> Expired
        if (container.properties.leaseState === Models.LeaseStateType.Leased &&
            container.properties.leaseDuration === Models.LeaseDurationType.Fixed) {
            if (container.leaseExpireTime !== undefined &&
                currentTime > container.leaseExpireTime) {
                container.properties.leaseState = Models.LeaseStateType.Expired;
                container.properties.leaseStatus = Models.LeaseStatusType.Unlocked;
                container.properties.leaseDuration = undefined;
                container.leaseExpireTime = undefined;
                container.leaseBreakExpireTime = undefined;
            }
        }
        // check Breaking -> Broken
        if (container.properties.leaseState === Models.LeaseStateType.Breaking) {
            if (container.leaseBreakExpireTime !== undefined &&
                currentTime > container.leaseBreakExpireTime) {
                container.properties.leaseState = Models.LeaseStateType.Broken;
                container.properties.leaseStatus = Models.LeaseStatusType.Unlocked;
                container.properties.leaseDuration = undefined;
                container.leaseExpireTime = undefined;
                container.leaseBreakExpireTime = undefined;
            }
        }
        return container;
    }
    /**
     * Check Container lease status on Read Container.
     *
     * @private
     * @param {Context} context
     * @param {ContainerModel} container
     * @param {LeaseAccessConditions} leaseAccessConditions
     * @returns {void}
     * @memberof ContainerHandler
     */
    checkLeaseOnReadContainer(context, container, leaseAccessConditions) {
        // check only when input Leased Id is not empty
        if (leaseAccessConditions !== undefined &&
            leaseAccessConditions.leaseId !== undefined &&
            leaseAccessConditions.leaseId !== "") {
            // return error when lease is unlocked
            if (container.properties.leaseStatus === Models.LeaseStatusType.Unlocked) {
                const blobCtx = new BlobStorageContext_1.default(context);
                throw StorageErrorFactory_1.default.getContainerLeaseLost(blobCtx.contextID);
            }
            else if (container.leaseId !== undefined &&
                leaseAccessConditions.leaseId.toLowerCase() !==
                    container.leaseId.toLowerCase()) {
                // return error when lease is locked but lease ID not match
                const blobCtx = new BlobStorageContext_1.default(context);
                throw StorageErrorFactory_1.default.getContainerLeaseIdMismatchWithContainerOperation(blobCtx.contextID);
            }
        }
    }
}
exports.default = ContainerHandler;
//# sourceMappingURL=ContainerHandler.js.map