"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const BlobStorageContext_1 = tslib_1.__importDefault(require("../context/BlobStorageContext"));
const NotImplementedError_1 = tslib_1.__importDefault(require("../errors/NotImplementedError"));
const StorageErrorFactory_1 = tslib_1.__importDefault(require("../errors/StorageErrorFactory"));
const Models = tslib_1.__importStar(require("../generated/artifacts/models"));
const constants_1 = require("../utils/constants");
const utils_1 = require("../utils/utils");
const BaseHandler_1 = tslib_1.__importDefault(require("./BaseHandler"));
const BlobHandler_1 = tslib_1.__importDefault(require("./BlobHandler"));
/**
 * PageBlobHandler handles Azure Storage PageBlob related requests.
 *
 * @export
 * @class PageBlobHandler
 * @extends {BaseHandler}
 * @implements {IPageBlobHandler}
 */
class PageBlobHandler extends BaseHandler_1.default {
    constructor(dataStore, logger, rangesManager) {
        super(dataStore, logger);
        this.rangesManager = rangesManager;
    }
    async create(contentLength, blobContentLength, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        const containerName = blobCtx.container;
        const blobName = blobCtx.blob;
        const date = blobCtx.startTime;
        if (contentLength !== 0) {
            throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "Content-Length must be 0 for Create Page Blob request.");
        }
        if (blobContentLength % 512 !== 0) {
            throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "x-ms-content-length must be aligned to a 512-byte boundary.");
        }
        options.blobHTTPHeaders = options.blobHTTPHeaders || {};
        const contentType = options.blobHTTPHeaders.blobContentType ||
            context.request.getHeader("content-type") ||
            "application/octet-stream";
        const container = await this.dataStore.getContainer(accountName, containerName);
        if (!container) {
            throw StorageErrorFactory_1.default.getContainerNotFound(blobCtx.contextID);
        }
        // const accessTierInferred = options.pageBlobAccessTier === undefined;
        // Check Blob size match tier
        // if (
        //   !accessTierInferred &&
        //   blobContentLength > PageBlobAccessTierThreshold.get(tier)!
        // ) {
        //   throw StorageErrorFactory.getBlobBlobTierInadequateForContentLength(
        //     blobCtx.contextID!
        //   );
        // }
        const etag = utils_1.newEtag();
        const blob = {
            deleted: false,
            metadata: options.metadata,
            accountName,
            containerName,
            name: blobName,
            properties: {
                creationTime: date,
                lastModified: date,
                etag,
                contentLength: blobContentLength,
                contentType,
                contentEncoding: options.blobHTTPHeaders.blobContentEncoding,
                contentLanguage: options.blobHTTPHeaders.blobContentLanguage,
                contentMD5: options.blobHTTPHeaders.blobContentMD5,
                contentDisposition: options.blobHTTPHeaders.blobContentDisposition,
                cacheControl: options.blobHTTPHeaders.blobCacheControl,
                blobSequenceNumber: options.blobSequenceNumber,
                blobType: Models.BlobType.PageBlob,
                leaseStatus: Models.LeaseStatusType.Unlocked,
                leaseState: Models.LeaseStateType.Available,
                serverEncrypted: true
                // accessTier: accessTierInferred
                //   ? ((options.pageBlobAccessTier as any) as Models.AccessTier)
                //   : Models.AccessTier.P4, // TODO: Infer tier from size
                // accessTierInferred
            },
            snapshot: "",
            isCommitted: true,
            pageRangesInOrder: []
        };
        // TODO: What's happens when create page blob right before commit block list? Or should we lock
        // Should we check if there is an uncommitted blob?
        this.dataStore.updateBlob(blob);
        const response = {
            statusCode: 201,
            eTag: etag,
            lastModified: blob.properties.lastModified,
            contentMD5: blob.properties.contentMD5,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            date,
            isServerEncrypted: true
        };
        return response;
    }
    async uploadPages(body, contentLength, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        const containerName = blobCtx.container;
        const blobName = blobCtx.blob;
        const date = blobCtx.startTime;
        if (contentLength % 512 !== 0) {
            throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "content-length or x-ms-content-length must be aligned to a 512-byte boundary.");
        }
        const container = await this.dataStore.getContainer(accountName, containerName);
        if (!container) {
            throw StorageErrorFactory_1.default.getContainerNotFound(blobCtx.contextID);
        }
        let blob = await this.dataStore.getBlob(accountName, containerName, blobName);
        if (!blob) {
            throw StorageErrorFactory_1.default.getBlobNotFound(blobCtx.contextID);
        }
        if (blob.properties.blobType !== Models.BlobType.PageBlob) {
            throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "Get Page Ranges could only be against a page blob.");
        }
        // Check Lease status
        BlobHandler_1.default.checkBlobLeaseOnWriteBlob(context, blob, options.leaseAccessConditions);
        const ranges = utils_1.deserializePageBlobRangeHeader(blobCtx.request.getHeader("range"), blobCtx.request.getHeader("x-ms-range"));
        if (!ranges) {
            throw StorageErrorFactory_1.default.getInvalidPageRange(blobCtx.contextID);
        }
        const start = ranges[0];
        const end = ranges[1]; // Inclusive
        if (end - start + 1 !== contentLength) {
            throw StorageErrorFactory_1.default.getInvalidPageRange(blobCtx.contextID);
        }
        // Persisted request payload
        const persistency = await this.dataStore.writePayload(body);
        this.rangesManager.mergeRange(blob.pageRangesInOrder || [], {
            start,
            end,
            persistency
        });
        // set lease state to available if it's expired
        blob = BlobHandler_1.default.UpdateBlobLeaseStateOnWriteBlob(blob);
        await this.dataStore.updateBlob(blob);
        const response = {
            statusCode: 201,
            eTag: utils_1.newEtag(),
            lastModified: date,
            contentMD5: undefined,
            blobSequenceNumber: blob.properties.blobSequenceNumber,
            requestId: blobCtx.contextID,
            version: constants_1.API_VERSION,
            date,
            isServerEncrypted: true
        };
        return response;
    }
    async clearPages(contentLength, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        const containerName = blobCtx.container;
        const blobName = blobCtx.blob;
        const date = blobCtx.startTime;
        if (contentLength !== 0) {
            throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "content-length or x-ms-content-length must be 0 for clear pages operation.");
        }
        const container = await this.dataStore.getContainer(accountName, containerName);
        if (!container) {
            throw StorageErrorFactory_1.default.getContainerNotFound(blobCtx.contextID);
        }
        // TODO: Lock
        const blob = await this.dataStore.getBlob(accountName, containerName, blobName);
        if (!blob) {
            throw StorageErrorFactory_1.default.getBlobNotFound(blobCtx.contextID);
        }
        if (blob.properties.blobType !== Models.BlobType.PageBlob) {
            throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "Get Page Ranges could only be against a page blob.");
        }
        const ranges = utils_1.deserializePageBlobRangeHeader(blobCtx.request.getHeader("range"), blobCtx.request.getHeader("x-ms-range"));
        if (!ranges) {
            throw StorageErrorFactory_1.default.getInvalidPageRange(blobCtx.contextID);
        }
        const start = ranges[0];
        const end = ranges[1];
        this.rangesManager.clearRange(blob.pageRangesInOrder || [], {
            start,
            end
        });
        await this.dataStore.updateBlob(blob);
        const response = {
            statusCode: 201,
            eTag: utils_1.newEtag(),
            lastModified: date,
            contentMD5: undefined,
            blobSequenceNumber: blob.properties.blobSequenceNumber,
            requestId: blobCtx.contextID,
            version: constants_1.API_VERSION,
            date
        };
        return response;
    }
    async getPageRanges(options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        const containerName = blobCtx.container;
        const blobName = blobCtx.blob;
        const date = blobCtx.startTime;
        const container = await this.dataStore.getContainer(accountName, containerName);
        if (!container) {
            throw StorageErrorFactory_1.default.getContainerNotFound(blobCtx.contextID);
        }
        const blob = await this.dataStore.getBlob(accountName, containerName, blobName, options.snapshot);
        if (!blob) {
            throw StorageErrorFactory_1.default.getBlobNotFound(blobCtx.contextID);
        }
        if (blob.properties.blobType !== Models.BlobType.PageBlob) {
            throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "Get Page Ranges could only be against a page blob.");
        }
        let ranges = utils_1.deserializePageBlobRangeHeader(blobCtx.request.getHeader("range"), blobCtx.request.getHeader("x-ms-range"));
        if (!ranges) {
            ranges = [0, blob.properties.contentLength - 1];
        }
        blob.pageRangesInOrder = blob.pageRangesInOrder || [];
        const impactedRanges = this.rangesManager.cutRanges(blob.pageRangesInOrder, {
            start: ranges[0],
            end: ranges[1]
        });
        const response = {
            statusCode: 200,
            pageRange: impactedRanges,
            eTag: blob.properties.etag,
            blobContentLength: blob.properties.contentLength,
            lastModified: date,
            requestId: blobCtx.contextID,
            version: constants_1.API_VERSION,
            date
        };
        return response;
    }
    getPageRangesDiff(options, context) {
        throw new NotImplementedError_1.default(context.contextID);
    }
    async resize(blobContentLength, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        const containerName = blobCtx.container;
        const blobName = blobCtx.blob;
        const date = blobCtx.startTime;
        if (blobContentLength % 512 !== 0) {
            throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "x-ms-blob-content-length must be aligned to a 512-byte boundary for Page Blob Resize request.");
        }
        const container = await this.dataStore.getContainer(accountName, containerName);
        if (!container) {
            throw StorageErrorFactory_1.default.getContainerNotFound(blobCtx.contextID);
        }
        const blob = await this.dataStore.getBlob(accountName, containerName, blobName);
        if (!blob) {
            throw StorageErrorFactory_1.default.getBlobNotFound(blobCtx.contextID);
        }
        if (blob.properties.blobType !== Models.BlobType.PageBlob) {
            throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "Resize could only be against a page blob.");
        }
        blob.pageRangesInOrder = blob.pageRangesInOrder || [];
        if (blob.properties.contentLength > blobContentLength) {
            const start = blobContentLength;
            const end = blob.properties.contentLength - 1;
            this.rangesManager.clearRange(blob.pageRangesInOrder || [], {
                start,
                end
            });
        }
        blob.properties.contentLength = blobContentLength;
        blob.properties.lastModified = blobCtx.startTime;
        this.dataStore.updateBlob(blob);
        const response = {
            statusCode: 200,
            eTag: utils_1.newEtag(),
            lastModified: blob.properties.lastModified,
            blobSequenceNumber: blob.properties.blobSequenceNumber,
            requestId: blobCtx.contextID,
            version: constants_1.API_VERSION,
            date
        };
        return response;
    }
    async updateSequenceNumber(sequenceNumberAction, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        const containerName = blobCtx.container;
        const blobName = blobCtx.blob;
        const date = blobCtx.startTime;
        const container = await this.dataStore.getContainer(accountName, containerName);
        if (!container) {
            throw StorageErrorFactory_1.default.getContainerNotFound(blobCtx.contextID);
        }
        // TODO: Lock
        const blob = await this.dataStore.getBlob(accountName, containerName, blobName);
        if (!blob) {
            throw StorageErrorFactory_1.default.getBlobNotFound(blobCtx.contextID);
        }
        if (blob.properties.blobType !== Models.BlobType.PageBlob) {
            throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "Get Page Ranges could only be against a page blob.");
        }
        if (blob.properties.blobSequenceNumber === undefined) {
            blob.properties.blobSequenceNumber = 0;
        }
        switch (sequenceNumberAction) {
            case Models.SequenceNumberActionType.Max:
                if (options.blobSequenceNumber === undefined) {
                    throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "x-ms-blob-sequence-number is required when x-ms-sequence-number-action is set to max.");
                }
                blob.properties.blobSequenceNumber = Math.max(blob.properties.blobSequenceNumber, options.blobSequenceNumber);
                break;
            case Models.SequenceNumberActionType.Increment:
                if (options.blobSequenceNumber !== undefined) {
                    throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "x-ms-blob-sequence-number cannot be provided when x-ms-sequence-number-action is set to increment.");
                }
                blob.properties.blobSequenceNumber++;
                break;
            case Models.SequenceNumberActionType.Update:
                if (options.blobSequenceNumber === undefined) {
                    throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "x-ms-blob-sequence-number is required when x-ms-sequence-number-action is set to update.");
                }
                blob.properties.blobSequenceNumber = options.blobSequenceNumber;
                break;
            default:
                throw StorageErrorFactory_1.default.getInvalidOperation(blobCtx.contextID, "Unsupported x-ms-sequence-number-action value.");
        }
        this.dataStore.updateBlob(blob);
        const response = {
            statusCode: 200,
            eTag: utils_1.newEtag(),
            lastModified: blob.properties.lastModified,
            blobSequenceNumber: blob.properties.blobSequenceNumber,
            requestId: blobCtx.contextID,
            version: constants_1.API_VERSION,
            date
        };
        return response;
    }
    copyIncremental(copySource, options, context) {
        throw new NotImplementedError_1.default(context.contextID);
    }
}
exports.default = PageBlobHandler;
//# sourceMappingURL=PageBlobHandler.js.map