"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const BlobStorageContext_1 = tslib_1.__importDefault(require("../context/BlobStorageContext"));
const Models = tslib_1.__importStar(require("../generated/artifacts/models"));
const xml_1 = require("../generated/utils/xml");
const constants_1 = require("../utils/constants");
const utils_1 = require("../utils/utils");
const BaseHandler_1 = tslib_1.__importDefault(require("./BaseHandler"));
/**
 * ServiceHandler handles Azure Storage Blob service related requests.
 *
 * @export
 * @class ServiceHandler
 * @implements {IHandler}
 */
class ServiceHandler extends BaseHandler_1.default {
    constructor() {
        super(...arguments);
        /**
         * Default listing containers max number.
         *
         * @private
         * @memberof ServiceHandler
         */
        this.LIST_CONTAINERS_MAX_RESULTS_DEFAULT = 2000;
        /**
         * Default service properties.
         *
         * @private
         * @memberof ServiceHandler
         */
        this.defaultServiceProperties = {
            cors: [],
            defaultServiceVersion: constants_1.API_VERSION,
            hourMetrics: {
                enabled: false,
                retentionPolicy: {
                    enabled: false
                },
                version: "1.0"
            },
            logging: {
                deleteProperty: true,
                read: true,
                retentionPolicy: {
                    enabled: false
                },
                version: "1.0",
                write: true
            },
            minuteMetrics: {
                enabled: false,
                retentionPolicy: {
                    enabled: false
                },
                version: "1.0"
            },
            staticWebsite: {
                enabled: false
            }
        };
    }
    /**
     * Set blob service properties.
     *
     * @param {Models.StorageServiceProperties} storageServiceProperties
     * @param {Models.ServiceSetPropertiesOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ServiceSetPropertiesResponse>}
     * @memberof ServiceHandler
     */
    async setProperties(storageServiceProperties, options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        // TODO: deserializor has a bug that when cors is undefined,
        // it will serialize it to empty array instead of undefined
        const body = blobCtx.request.getBody();
        const parsedBody = await xml_1.parseXML(body || "");
        if (!parsedBody.hasOwnProperty("cors") &&
            !parsedBody.hasOwnProperty("Cors")) {
            storageServiceProperties.cors = undefined;
        }
        let properties = await this.dataStore.getServiceProperties(accountName);
        if (properties === undefined) {
            properties = Object.assign({}, this.defaultServiceProperties, { accountName });
        }
        properties.cors =
            storageServiceProperties.cors === undefined
                ? properties.cors
                : storageServiceProperties.cors;
        properties.defaultServiceVersion =
            storageServiceProperties.defaultServiceVersion ||
                properties.defaultServiceVersion;
        properties.deleteRetentionPolicy =
            storageServiceProperties.deleteRetentionPolicy ||
                properties.deleteRetentionPolicy;
        properties.hourMetrics =
            storageServiceProperties.hourMetrics || properties.hourMetrics;
        properties.logging = storageServiceProperties.logging || properties.logging;
        properties.minuteMetrics =
            storageServiceProperties.minuteMetrics || properties.minuteMetrics;
        properties.staticWebsite =
            storageServiceProperties.staticWebsite || properties.staticWebsite;
        await this.dataStore.updateServiceProperties(Object.assign({}, properties, { accountName }));
        const response = {
            requestId: context.contextID,
            statusCode: 202,
            version: constants_1.API_VERSION
        };
        return response;
    }
    /**
     * Get blob service properties.
     *
     * @param {Models.ServiceGetPropertiesOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ServiceGetPropertiesResponse>}
     * @memberof ServiceHandler
     */
    async getProperties(options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const accountName = blobCtx.account;
        let properties = await this.dataStore.getServiceProperties(accountName);
        if (!properties) {
            properties = Object.assign({}, this.defaultServiceProperties, { accountName });
        }
        if (properties.cors === undefined) {
            properties.cors = [];
        }
        const response = Object.assign({}, properties, { requestId: context.contextID, statusCode: 200, version: constants_1.API_VERSION });
        return response;
    }
    async getStatistics(options, context) {
        const response = {
            statusCode: 200,
            requestId: context.contextID,
            version: constants_1.API_VERSION,
            date: context.startTime,
            geoReplication: {
                status: Models.GeoReplicationStatusType.Live,
                lastSyncTime: context.startTime
            }
        };
        return response;
    }
    /**
     * List containers.
     *
     * @param {Models.ServiceListContainersSegmentOptionalParams} options
     * @param {Context} context
     * @returns {Promise<Models.ServiceListContainersSegmentResponse>}
     * @memberof ServiceHandler
     */
    async listContainersSegment(options, context) {
        const blobCtx = new BlobStorageContext_1.default(context);
        const request = blobCtx.request;
        const accountName = blobCtx.account;
        options.maxresults =
            options.maxresults || this.LIST_CONTAINERS_MAX_RESULTS_DEFAULT;
        options.prefix = options.prefix || "";
        const marker = parseInt(options.marker || "0", 10);
        const containers = await this.dataStore.listContainers(accountName, options.prefix, options.maxresults, marker);
        // TODO: Need update list out container lease properties with ContainerHandler.updateLeaseAttributes()
        const serviceEndpoint = `${request.getEndpoint()}/${accountName}`;
        const res = {
            containerItems: containers[0],
            maxResults: options.maxresults,
            nextMarker: `${containers[1] || ""}`,
            prefix: options.prefix,
            serviceEndpoint,
            statusCode: 200,
            requestId: context.contextID,
            version: constants_1.API_VERSION
        };
        return res;
    }
    async getAccountInfo(context) {
        return utils_1.getContainerGetAccountInfoResponse(context);
    }
    async getAccountInfoWithHead(context) {
        return utils_1.getContainerGetAccountInfoResponse(context);
    }
}
exports.default = ServiceHandler;
//# sourceMappingURL=ServiceHandler.js.map