#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs_1 = require("fs");
const path_1 = require("path");
const util_1 = require("util");
const Environment_1 = tslib_1.__importDefault(require("../common/Environment"));
const Logger = tslib_1.__importStar(require("../common/Logger"));
const BlobConfiguration_1 = tslib_1.__importDefault(require("./BlobConfiguration"));
const BlobServer_1 = tslib_1.__importDefault(require("./BlobServer"));
const constants_1 = require("./utils/constants");
// tslint:disable:no-console
const accessAsync = util_1.promisify(fs_1.access);
/**
 * Entry for Azurite blob service.
 */
async function main() {
    // Initialize and validate environment values from command line parameters
    const env = new Environment_1.default();
    await accessAsync(env.location);
    if (env.debug !== undefined) {
        await accessAsync(path_1.dirname(env.debug));
    }
    // Initialize server configuration
    const config = new BlobConfiguration_1.default(env.blobHost, env.blobPort, path_1.join(env.location, constants_1.DEFAULT_LOKI_DB_PATH), path_1.join(env.location, constants_1.DEFAULT_BLOB_PERSISTENCE_PATH), !env.silent, undefined, env.debug !== undefined, env.debug);
    // We use logger singleton as global debugger logger to track detailed outputs cross layers
    // Note that, debug log is different from access log which is only available in request handler layer to
    // track every request. Access log is not singleton, and initialized in specific RequestHandlerFactory implementations
    // Enable debug log by default before first release for debugging purpose
    Logger.configLogger(config.enableDebugLog, config.debugLogFilePath);
    // Create server instance
    const server = new BlobServer_1.default(config);
    // Start server
    console.log(`Azurite Blob service is starting on ${config.host}:${config.port}`);
    await server.start();
    console.log(`Azurite Blob service successfully listens on ${server.getHttpServerAddress()}`);
    // Handle close event
    const beforeCloseMessage = `Azurite Blob service is closing...`;
    const afterCloseMessage = `Azurite Blob service successfully closed`;
    process
        .once("message", msg => {
        if (msg === "shutdown") {
            console.log(beforeCloseMessage);
            server.close().then(() => {
                console.log(afterCloseMessage);
            });
        }
    })
        .once("SIGINT", () => {
        console.log(beforeCloseMessage);
        server.close().then(() => {
            console.log(afterCloseMessage);
        });
    });
}
main().catch(err => {
    console.error(`Exit due to unhandled error: ${err.message}`);
    process.exit(1);
});
//# sourceMappingURL=main.js.map