"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Logger_1 = require("../../common/Logger");
const NoLoggerStrategy_1 = tslib_1.__importDefault(require("../../common/NoLoggerStrategy"));
var State;
(function (State) {
    State[State["LISTING_EXTENTS_IN_BLOBS"] = 0] = "LISTING_EXTENTS_IN_BLOBS";
    State[State["LISTING_EXTENTS_IN_BLOCKS"] = 1] = "LISTING_EXTENTS_IN_BLOCKS";
    State[State["DONE"] = 2] = "DONE";
})(State || (State = {}));
/**
 * An async iterator which enumerates all extents being used.
 *
 * @export
 * @class LokiReferredExtentsAsyncIterator
 * @implements {AsyncIterator<IPersistencyChunk[]>}
 */
class LokiReferredExtentsAsyncIterator {
    constructor(blobDataStore, logger = new Logger_1.Logger(new NoLoggerStrategy_1.default())) {
        this.blobDataStore = blobDataStore;
        this.logger = logger;
        this.state = State.LISTING_EXTENTS_IN_BLOBS;
    }
    async next() {
        if (this.state === State.LISTING_EXTENTS_IN_BLOBS) {
            const [blobs, marker] = await this.blobDataStore.listBlobs(undefined, undefined, undefined, undefined, undefined, this.blobListingMarker, true);
            this.blobListingMarker = marker;
            if (marker === undefined) {
                this.state = State.LISTING_EXTENTS_IN_BLOCKS;
            }
            const extents = [];
            for (const blob of blobs) {
                this.logger.debug(`LokiReferredExtentsAsyncIterator:next() Handle blob ${blob.accountName} ${blob.containerName} ${blob.name} ${blob.snapshot} Blocks: ${(blob.committedBlocksInOrder || []).length} PageRanges: ${(blob.pageRangesInOrder || []).length} Persistency: ${blob.persistency ? blob.persistency.id : ""}`);
                for (const block of blob.committedBlocksInOrder || []) {
                    extents.push(block.persistency);
                }
                for (const range of blob.pageRangesInOrder || []) {
                    extents.push(range.persistency);
                }
                if (blob.persistency) {
                    extents.push(blob.persistency);
                }
            }
            return {
                done: false,
                value: extents
            };
        }
        else if (this.state === State.LISTING_EXTENTS_IN_BLOCKS) {
            // TODO: Make listBlocks operation segment
            const blocks = await this.blobDataStore.listBlocks();
            this.logger.debug(`LokiReferredExtentsAsyncIterator:next() Handle blocks ${blocks.length}`);
            this.state = State.DONE;
            return {
                done: false,
                value: blocks.map(value => value.persistency)
            };
        }
        else {
            return {
                done: true,
                value: []
            };
        }
    }
}
exports.default = LokiReferredExtentsAsyncIterator;
//# sourceMappingURL=LokiReferredExtentsAsyncIterator.js.map