"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Abstract Server class for Azurite HTTP or HTTPS Servers.
 *
 * @export
 * @abstract
 * @class Server
 */
class ServerBase {
    /**
     * Creates an instance of HTTP or HTTPS server.
     *
     * @param {string} host Server host,for example, "127.0.0.1"
     * @param {number} port Server port, for example, 10000
     * @param {http.Server | https.Server} httpServer A HTTP or HTTPS server instance without request listener bound
     * @param {IRequestListenerFactory} requestListenerFactory A request listener factory
     * @memberof ServerBase
     */
    constructor(host, port, httpServer, requestListenerFactory) {
        this.host = host;
        this.port = port;
        this.httpServer = httpServer;
        // Remove predefined request listeners to avoid double request handling
        this.httpServer.removeAllListeners("request");
        this.httpServer.on("request", requestListenerFactory.createRequestListener());
    }
    /**
     * Get HTTP server listening address and port string.
     * Note this may be different from host and port parameters values, because
     * when port is 0, system will select a rand port number for listening.
     * This method will return the port and address being used.
     *
     * @returns {string}
     * @memberof ServerBase
     */
    getHttpServerAddress() {
        const address = this.httpServer.address();
        if (typeof address === "string" || address === null) {
            return address || "";
        }
        else {
            return `${address.address}:${address.port}`;
        }
    }
    /**
     * Initialize and start the server to server incoming HTTP requests.
     * beforeStart() and afterStart() will be executed before and after start().
     *
     * @abstract
     * @returns {Promise<void>}
     * @memberof Server
     */
    async start() {
        await this.beforeStart();
        await new Promise((resolve, reject) => {
            this.httpServer.listen(this.port, this.host, resolve).on("error", reject);
        });
        await this.afterStart();
    }
    /**
     * Dispose HTTP server and clean up other resources.
     *
     * beforeClose() and afterClose() will be executed before and after close().
     *
     * We name this method as close instead of dispose, because in practices, usually we cannot re-open the resources
     * disposed, but can re-open the resources closed.
     *
     * @abstract
     * @returns {Promise<void>}
     * @memberof Server
     */
    async close() {
        await this.beforeClose();
        // Remove request listener to reject incoming requests
        this.httpServer.removeAllListeners("request");
        // Close HTTP server first to deny incoming connections
        // You will find this will not close server immediately because there maybe existing keep-alive connections
        // Calling httpServer.close will only stop accepting incoming requests
        // and wait for existing keep-alive connections timeout
        // Default keep-alive timeout is 5 seconds defined by httpServer.keepAliveTimeout
        // TODO: Add a middleware to reject incoming request over existing keep-alive connections
        // https://github.com/nodejs/node/issues/2642
        await new Promise(resolve => {
            this.httpServer.close(resolve);
        });
        await this.afterClose();
    }
    /**
     * Async task before server starts.
     *
     * @protected
     * @abstract
     * @returns {Promise<void>}
     * @memberof Server
     */
    async beforeStart() {
        /** NOOP */
    }
    /**
     * Async task after server starts.
     *
     * @protected
     * @abstract
     * @returns {Promise<void>}
     * @memberof Server
     */
    async afterStart() {
        /** NOOP */
    }
    /**
     * Async task before server closes.
     *
     * @protected
     * @abstract
     * @returns {Promise<void>}
     * @memberof Server
     */
    async beforeClose() {
        /** NOOP */
    }
    /**
     * Async task after server closes.
     *
     * @protected
     * @abstract
     * @returns {Promise<void>}
     * @memberof Server
     */
    async afterClose() {
        /** NOOP */
    }
}
exports.default = ServerBase;
//# sourceMappingURL=ServerBase.js.map