/**
 * An interface representing StorageError.
 */
export interface StorageError {
    message?: string;
}
/**
 * An Access policy
 */
export interface AccessPolicy {
    /**
     * the date-time the policy is active
     * **NOTE: This entity will be treated as a string instead of a Date because the API can
     * potentially deal with a higher precision value than what is supported by JavaScript.**
     */
    start?: string;
    /**
     * the date-time the policy expires
     * **NOTE: This entity will be treated as a string instead of a Date because the API can
     * potentially deal with a higher precision value than what is supported by JavaScript.**
     */
    expiry?: string;
    /**
     * the permissions for the acl policy
     */
    permission: string;
}
/**
 * Properties of a blob
 */
export interface BlobProperties {
    creationTime?: Date;
    lastModified: Date;
    etag: string;
    /**
     * Size in bytes
     */
    contentLength?: number;
    contentType?: string;
    contentEncoding?: string;
    contentLanguage?: string;
    contentMD5?: Uint8Array;
    contentDisposition?: string;
    cacheControl?: string;
    blobSequenceNumber?: number;
    /**
     * Possible values include: 'BlockBlob', 'PageBlob', 'AppendBlob'
     */
    blobType?: BlobType;
    /**
     * Possible values include: 'locked', 'unlocked'
     */
    leaseStatus?: LeaseStatusType;
    /**
     * Possible values include: 'available', 'leased', 'expired', 'breaking', 'broken'
     */
    leaseState?: LeaseStateType;
    /**
     * Possible values include: 'infinite', 'fixed'
     */
    leaseDuration?: LeaseDurationType;
    copyId?: string;
    /**
     * Possible values include: 'pending', 'success', 'aborted', 'failed'
     */
    copyStatus?: CopyStatusType;
    copySource?: string;
    copyProgress?: string;
    copyCompletionTime?: Date;
    copyStatusDescription?: string;
    serverEncrypted?: boolean;
    incrementalCopy?: boolean;
    destinationSnapshot?: string;
    deletedTime?: Date;
    remainingRetentionDays?: number;
    /**
     * Possible values include: 'P4', 'P6', 'P10', 'P15', 'P20', 'P30', 'P40', 'P50', 'P60', 'P70',
     * 'P80', 'Hot', 'Cool', 'Archive'
     */
    accessTier?: AccessTier;
    accessTierInferred?: boolean;
    /**
     * Possible values include: 'rehydrate-pending-to-hot', 'rehydrate-pending-to-cool'
     */
    archiveStatus?: ArchiveStatus;
    accessTierChangeTime?: Date;
}
/**
 * An Azure Storage blob
 */
export interface BlobItem {
    name: string;
    deleted?: boolean;
    snapshot: string;
    properties: BlobProperties;
    metadata?: {
        [propertyName: string]: string;
    };
}
/**
 * An interface representing BlobFlatListSegment.
 */
export interface BlobFlatListSegment {
    blobItems: BlobItem[];
}
/**
 * An enumeration of blobs
 */
export interface ListBlobsFlatSegmentResponse {
    serviceEndpoint: string;
    containerName: string;
    prefix: string;
    marker: string;
    maxResults: number;
    delimiter: string;
    segment: BlobFlatListSegment;
    nextMarker: string;
}
/**
 * An interface representing BlobPrefix.
 */
export interface BlobPrefix {
    name: string;
}
/**
 * An interface representing BlobHierarchyListSegment.
 */
export interface BlobHierarchyListSegment {
    blobPrefixes?: BlobPrefix[];
    blobItems: BlobItem[];
}
/**
 * An enumeration of blobs
 */
export interface ListBlobsHierarchySegmentResponse {
    serviceEndpoint: string;
    containerName: string;
    prefix: string;
    marker: string;
    maxResults: number;
    delimiter: string;
    segment: BlobHierarchyListSegment;
    nextMarker: string;
}
/**
 * Represents a single block in a block blob.  It describes the block's ID and size.
 */
export interface Block {
    /**
     * The base64 encoded block ID.
     */
    name: string;
    /**
     * The block size in bytes.
     */
    size: number;
}
/**
 * An interface representing BlockList.
 */
export interface BlockList {
    committedBlocks?: Block[];
    uncommittedBlocks?: Block[];
}
/**
 * An interface representing BlockLookupList.
 */
export interface BlockLookupList {
    committed?: string[];
    uncommitted?: string[];
    latest?: string[];
}
/**
 * Properties of a container
 */
export interface ContainerProperties {
    lastModified: Date;
    etag: string;
    /**
     * Possible values include: 'locked', 'unlocked'
     */
    leaseStatus?: LeaseStatusType;
    /**
     * Possible values include: 'available', 'leased', 'expired', 'breaking', 'broken'
     */
    leaseState?: LeaseStateType;
    /**
     * Possible values include: 'infinite', 'fixed'
     */
    leaseDuration?: LeaseDurationType;
    /**
     * Possible values include: 'container', 'blob'
     */
    publicAccess?: PublicAccessType;
    hasImmutabilityPolicy?: boolean;
    hasLegalHold?: boolean;
}
/**
 * An Azure Storage container
 */
export interface ContainerItem {
    name: string;
    properties: ContainerProperties;
    metadata?: {
        [propertyName: string]: string;
    };
}
/**
 * An enumeration of containers
 */
export interface ListContainersSegmentResponse {
    serviceEndpoint: string;
    prefix: string;
    marker?: string;
    maxResults: number;
    containerItems: ContainerItem[];
    nextMarker: string;
}
/**
 * CORS is an HTTP feature that enables a web application running under one domain to access
 * resources in another domain. Web browsers implement a security restriction known as same-origin
 * policy that prevents a web page from calling APIs in a different domain; CORS provides a secure
 * way to allow one domain (the origin domain) to call APIs in another domain
 */
export interface CorsRule {
    /**
     * The origin domains that are permitted to make a request against the storage service via CORS.
     * The origin domain is the domain from which the request originates. Note that the origin must
     * be an exact case-sensitive match with the origin that the user age sends to the service. You
     * can also use the wildcard character '*' to allow all origin domains to make requests via CORS.
     */
    allowedOrigins: string;
    /**
     * The methods (HTTP request verbs) that the origin domain may use for a CORS request. (comma
     * separated)
     */
    allowedMethods: string;
    /**
     * the request headers that the origin domain may specify on the CORS request.
     */
    allowedHeaders: string;
    /**
     * The response headers that may be sent in the response to the CORS request and exposed by the
     * browser to the request issuer
     */
    exposedHeaders: string;
    /**
     * The maximum amount time that a browser should cache the preflight OPTIONS request.
     */
    maxAgeInSeconds: number;
}
/**
 * Geo-Replication information for the Secondary Storage Service
 */
export interface GeoReplication {
    /**
     * The status of the secondary location. Possible values include: 'live', 'bootstrap',
     * 'unavailable'
     */
    status: GeoReplicationStatusType;
    /**
     * A GMT date/time value, to the second. All primary writes preceding this value are guaranteed
     * to be available for read operations at the secondary. Primary writes after this point in time
     * may or may not be available for reads.
     */
    lastSyncTime: Date;
}
/**
 * the retention policy which determines how long the associated data should persist
 */
export interface RetentionPolicy {
    /**
     * Indicates whether a retention policy is enabled for the storage service
     */
    enabled: boolean;
    /**
     * Indicates the number of days that metrics or logging or soft-deleted data should be retained.
     * All data older than this value will be deleted
     */
    days?: number;
}
/**
 * Azure Analytics Logging settings.
 */
export interface Logging {
    /**
     * The version of Storage Analytics to configure.
     */
    version: string;
    /**
     * Indicates whether all delete requests should be logged.
     */
    deleteProperty: boolean;
    /**
     * Indicates whether all read requests should be logged.
     */
    read: boolean;
    /**
     * Indicates whether all write requests should be logged.
     */
    write: boolean;
    retentionPolicy: RetentionPolicy;
}
/**
 * a summary of request statistics grouped by API in hour or minute aggregates for blobs
 */
export interface Metrics {
    /**
     * The version of Storage Analytics to configure.
     */
    version?: string;
    /**
     * Indicates whether metrics are enabled for the Blob service.
     */
    enabled: boolean;
    /**
     * Indicates whether metrics should generate summary statistics for called API operations.
     */
    includeAPIs?: boolean;
    retentionPolicy?: RetentionPolicy;
}
/**
 * An interface representing PageRange.
 */
export interface PageRange {
    start: number;
    end: number;
}
/**
 * An interface representing ClearRange.
 */
export interface ClearRange {
    start: number;
    end: number;
}
/**
 * the list of pages
 */
export interface PageList {
    pageRange?: PageRange[];
    clearRange?: ClearRange[];
}
/**
 * signed identifier
 */
export interface SignedIdentifier {
    /**
     * a unique id
     */
    id: string;
    accessPolicy: AccessPolicy;
}
/**
 * The properties that enable an account to host a static website
 */
export interface StaticWebsite {
    /**
     * Indicates whether this account is hosting a static website
     */
    enabled: boolean;
    /**
     * The default name of the index page under each directory
     */
    indexDocument?: string;
    /**
     * The absolute path of the custom 404 page
     */
    errorDocument404Path?: string;
}
/**
 * Storage Service Properties.
 */
export interface StorageServiceProperties {
    logging?: Logging;
    hourMetrics?: Metrics;
    minuteMetrics?: Metrics;
    /**
     * The set of CORS rules.
     */
    cors?: CorsRule[];
    /**
     * The default version to use for requests to the Blob service if an incoming request's version
     * is not specified. Possible values include version 2008-10-27 and all more recent versions
     */
    defaultServiceVersion?: string;
    deleteRetentionPolicy?: RetentionPolicy;
    staticWebsite?: StaticWebsite;
}
/**
 * Stats for the storage service.
 */
export interface StorageServiceStats {
    geoReplication?: GeoReplication;
}
/**
 * Additional parameters for a set of operations.
 */
export interface LeaseAccessConditions {
    /**
     * If specified, the operation only succeeds if the container's lease is active and matches this
     * ID.
     */
    leaseId?: string;
}
/**
 * Additional parameters for a set of operations.
 */
export interface ModifiedAccessConditions {
    /**
     * Specify this header value to operate only on a blob if it has been modified since the
     * specified date/time.
     */
    ifModifiedSince?: Date;
    /**
     * Specify this header value to operate only on a blob if it has not been modified since the
     * specified date/time.
     */
    ifUnmodifiedSince?: Date;
    /**
     * Specify an ETag value to operate only on blobs with a matching value.
     */
    ifMatch?: string;
    /**
     * Specify an ETag value to operate only on blobs without a matching value.
     */
    ifNoneMatch?: string;
}
/**
 * Additional parameters for a set of operations.
 */
export interface BlobHTTPHeaders {
    /**
     * Optional. Sets the blob's cache control. If specified, this property is stored with the blob
     * and returned with a read request.
     */
    blobCacheControl?: string;
    /**
     * Optional. Sets the blob's content type. If specified, this property is stored with the blob
     * and returned with a read request.
     */
    blobContentType?: string;
    /**
     * Optional. An MD5 hash of the blob content. Note that this hash is not validated, as the hashes
     * for the individual blocks were validated when each was uploaded.
     */
    blobContentMD5?: Uint8Array;
    /**
     * Optional. Sets the blob's content encoding. If specified, this property is stored with the
     * blob and returned with a read request.
     */
    blobContentEncoding?: string;
    /**
     * Optional. Set the blob's content language. If specified, this property is stored with the blob
     * and returned with a read request.
     */
    blobContentLanguage?: string;
    /**
     * Optional. Sets the blob's Content-Disposition header.
     */
    blobContentDisposition?: string;
}
/**
 * Additional parameters for startCopyFromURL operation.
 */
export interface SourceModifiedAccessConditions {
    /**
     * Specify this header value to operate only on a blob if it has been modified since the
     * specified date/time.
     */
    sourceIfModifiedSince?: Date;
    /**
     * Specify this header value to operate only on a blob if it has not been modified since the
     * specified date/time.
     */
    sourceIfUnmodifiedSince?: Date;
    /**
     * Specify an ETag value to operate only on blobs with a matching value.
     */
    sourceIfMatches?: string;
    /**
     * Specify an ETag value to operate only on blobs without a matching value.
     */
    sourceIfNoneMatch?: string;
}
/**
 * Additional parameters for a set of operations, such as: PageBlob_uploadPages,
 * PageBlob_clearPages.
 */
export interface SequenceNumberAccessConditions {
    /**
     * Specify this header value to operate only on a blob if it has a sequence number less than or
     * equal to the specified.
     */
    ifSequenceNumberLessThanOrEqualTo?: number;
    /**
     * Specify this header value to operate only on a blob if it has a sequence number less than the
     * specified.
     */
    ifSequenceNumberLessThan?: number;
    /**
     * Specify this header value to operate only on a blob if it has the specified sequence number.
     */
    ifSequenceNumberEqualTo?: number;
}
/**
 * Additional parameters for appendBlock operation.
 */
export interface AppendPositionAccessConditions {
    /**
     * Optional conditional header. The max length in bytes permitted for the append blob. If the
     * Append Block operation would cause the blob to exceed that limit or if the blob size is
     * already greater than the value specified in this header, the request will fail with
     * MaxBlobSizeConditionNotMet error (HTTP status code 412 - Precondition Failed).
     */
    maxSize?: number;
    /**
     * Optional conditional header, used only for the Append Block operation. A number indicating the
     * byte offset to compare. Append Block will succeed only if the append position is equal to this
     * number. If it is not, the request will fail with the AppendPositionConditionNotMet error (HTTP
     * status code 412 - Precondition Failed).
     */
    appendPosition?: number;
}
/**
 * An interface representing AzuriteServerBlobOptions.
 */
export interface AzuriteServerBlobOptions {
    /**
     * Specifies the version of the operation to use for this request.
     */
    version?: string;
}
/**
 * Optional Parameters.
 */
export interface ServiceSetPropertiesOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
}
/**
 * Optional Parameters.
 */
export interface ServiceGetPropertiesOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
}
/**
 * Optional Parameters.
 */
export interface ServiceGetStatisticsOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
}
/**
 * Optional Parameters.
 */
export interface ServiceListContainersSegmentOptionalParams {
    /**
     * Filters the results to return only containers whose name begins with the specified prefix.
     */
    prefix?: string;
    /**
     * A string value that identifies the portion of the list of containers to be returned with the
     * next listing operation. The operation returns the NextMarker value within the response body if
     * the listing operation did not return all containers remaining to be listed with the current
     * page. The NextMarker value can be used as the value for the marker parameter in a subsequent
     * call to request the next page of list items. The marker value is opaque to the client.
     */
    marker?: string;
    /**
     * Specifies the maximum number of containers to return. If the request does not specify
     * maxresults, or specifies a value greater than 5000, the server will return up to 5000 items.
     * Note that if the listing operation crosses a partition boundary, then the service will return
     * a continuation token for retrieving the remainder of the results. For this reason, it is
     * possible that the service will return fewer results than specified by maxresults, or than the
     * default of 5000.
     */
    maxresults?: number;
    /**
     * Include this parameter to specify that the container's metadata be returned as part of the
     * response body. Possible values include: 'metadata'
     */
    include?: ListContainersIncludeType;
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
}
/**
 * Optional Parameters.
 */
export interface ContainerCreateOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Optional. Specifies a user-defined name-value pair associated with the blob. If no name-value
     * pairs are specified, the operation will copy the metadata from the source blob or file to the
     * destination blob. If one or more name-value pairs are specified, the destination blob is
     * created with the specified metadata, and metadata is not copied from the source blob or file.
     * Note that beginning with version 2009-09-19, metadata names must adhere to the naming rules
     * for C# identifiers. See Naming and Referencing Containers, Blobs, and Metadata for more
     * information.
     */
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * Specifies whether data in the container may be accessed publicly and the level of access.
     * Possible values include: 'container', 'blob'
     */
    access?: PublicAccessType;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
}
/**
 * Optional Parameters.
 */
export interface ContainerGetPropertiesOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface ContainerGetPropertiesWithHeadOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface ContainerDeleteMethodOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface ContainerSetMetadataOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Optional. Specifies a user-defined name-value pair associated with the blob. If no name-value
     * pairs are specified, the operation will copy the metadata from the source blob or file to the
     * destination blob. If one or more name-value pairs are specified, the destination blob is
     * created with the specified metadata, and metadata is not copied from the source blob or file.
     * Note that beginning with version 2009-09-19, metadata names must adhere to the naming rules
     * for C# identifiers. See Naming and Referencing Containers, Blobs, and Metadata for more
     * information.
     */
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface ContainerGetAccessPolicyOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface ContainerSetAccessPolicyOptionalParams {
    /**
     * the acls for the container
     */
    containerAcl?: SignedIdentifier[];
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Specifies whether data in the container may be accessed publicly and the level of access.
     * Possible values include: 'container', 'blob'
     */
    access?: PublicAccessType;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface ContainerAcquireLeaseOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Specifies the duration of the lease, in seconds, or negative one (-1) for a lease that never
     * expires. A non-infinite lease can be between 15 and 60 seconds. A lease duration cannot be
     * changed using renew or change.
     */
    duration?: number;
    /**
     * Proposed lease ID, in a GUID string format. The Blob service returns 400 (Invalid request) if
     * the proposed lease ID is not in the correct format. See Guid Constructor (String) for a list
     * of valid GUID string formats.
     */
    proposedLeaseId?: string;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface ContainerReleaseLeaseOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface ContainerRenewLeaseOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface ContainerBreakLeaseOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * For a break operation, proposed duration the lease should continue before it is broken, in
     * seconds, between 0 and 60. This break period is only used if it is shorter than the time
     * remaining on the lease. If longer, the time remaining on the lease is used. A new lease will
     * not be available before the break period has expired, but the lease may be held for longer
     * than the break period. If this header does not appear with a break operation, a fixed-duration
     * lease breaks after the remaining lease period elapses, and an infinite lease breaks
     * immediately.
     */
    breakPeriod?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface ContainerChangeLeaseOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface ContainerListBlobFlatSegmentOptionalParams {
    /**
     * Filters the results to return only containers whose name begins with the specified prefix.
     */
    prefix?: string;
    /**
     * A string value that identifies the portion of the list of containers to be returned with the
     * next listing operation. The operation returns the NextMarker value within the response body if
     * the listing operation did not return all containers remaining to be listed with the current
     * page. The NextMarker value can be used as the value for the marker parameter in a subsequent
     * call to request the next page of list items. The marker value is opaque to the client.
     */
    marker?: string;
    /**
     * Specifies the maximum number of containers to return. If the request does not specify
     * maxresults, or specifies a value greater than 5000, the server will return up to 5000 items.
     * Note that if the listing operation crosses a partition boundary, then the service will return
     * a continuation token for retrieving the remainder of the results. For this reason, it is
     * possible that the service will return fewer results than specified by maxresults, or than the
     * default of 5000.
     */
    maxresults?: number;
    /**
     * Include this parameter to specify one or more datasets to include in the response.
     */
    include?: ListBlobsIncludeItem[];
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
}
/**
 * Optional Parameters.
 */
export interface ContainerListBlobHierarchySegmentOptionalParams {
    /**
     * Filters the results to return only containers whose name begins with the specified prefix.
     */
    prefix?: string;
    /**
     * A string value that identifies the portion of the list of containers to be returned with the
     * next listing operation. The operation returns the NextMarker value within the response body if
     * the listing operation did not return all containers remaining to be listed with the current
     * page. The NextMarker value can be used as the value for the marker parameter in a subsequent
     * call to request the next page of list items. The marker value is opaque to the client.
     */
    marker?: string;
    /**
     * Specifies the maximum number of containers to return. If the request does not specify
     * maxresults, or specifies a value greater than 5000, the server will return up to 5000 items.
     * Note that if the listing operation crosses a partition boundary, then the service will return
     * a continuation token for retrieving the remainder of the results. For this reason, it is
     * possible that the service will return fewer results than specified by maxresults, or than the
     * default of 5000.
     */
    maxresults?: number;
    /**
     * Include this parameter to specify one or more datasets to include in the response.
     */
    include?: ListBlobsIncludeItem[];
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
}
/**
 * Optional Parameters.
 */
export interface BlobDownloadOptionalParams {
    /**
     * The snapshot parameter is an opaque DateTime value that, when present, specifies the blob
     * snapshot to retrieve. For more information on working with blob snapshots, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/creating-a-snapshot-of-a-blob">Creating
     * a Snapshot of a Blob.</a>
     */
    snapshot?: string;
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Return only the bytes of the blob in the specified range.
     */
    range?: string;
    /**
     * When set to true and specified together with the Range, the service returns the MD5 hash for
     * the range, as long as the range is less than or equal to 4 MB in size.
     */
    rangeGetContentMD5?: boolean;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlobGetPropertiesOptionalParams {
    /**
     * The snapshot parameter is an opaque DateTime value that, when present, specifies the blob
     * snapshot to retrieve. For more information on working with blob snapshots, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/creating-a-snapshot-of-a-blob">Creating
     * a Snapshot of a Blob.</a>
     */
    snapshot?: string;
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlobDeleteMethodOptionalParams {
    /**
     * The snapshot parameter is an opaque DateTime value that, when present, specifies the blob
     * snapshot to retrieve. For more information on working with blob snapshots, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/creating-a-snapshot-of-a-blob">Creating
     * a Snapshot of a Blob.</a>
     */
    snapshot?: string;
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Required if the blob has associated snapshots. Specify one of the following two options:
     * include: Delete the base blob and all of its snapshots. only: Delete only the blob's snapshots
     * and not the blob itself. Possible values include: 'include', 'only'
     */
    deleteSnapshots?: DeleteSnapshotsOptionType;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlobUndeleteOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
}
/**
 * Optional Parameters.
 */
export interface BlobSetHTTPHeadersOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    blobHTTPHeaders?: BlobHTTPHeaders;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlobSetMetadataOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Optional. Specifies a user-defined name-value pair associated with the blob. If no name-value
     * pairs are specified, the operation will copy the metadata from the source blob or file to the
     * destination blob. If one or more name-value pairs are specified, the destination blob is
     * created with the specified metadata, and metadata is not copied from the source blob or file.
     * Note that beginning with version 2009-09-19, metadata names must adhere to the naming rules
     * for C# identifiers. See Naming and Referencing Containers, Blobs, and Metadata for more
     * information.
     */
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlobAcquireLeaseOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Specifies the duration of the lease, in seconds, or negative one (-1) for a lease that never
     * expires. A non-infinite lease can be between 15 and 60 seconds. A lease duration cannot be
     * changed using renew or change.
     */
    duration?: number;
    /**
     * Proposed lease ID, in a GUID string format. The Blob service returns 400 (Invalid request) if
     * the proposed lease ID is not in the correct format. See Guid Constructor (String) for a list
     * of valid GUID string formats.
     */
    proposedLeaseId?: string;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlobReleaseLeaseOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlobRenewLeaseOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlobChangeLeaseOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlobBreakLeaseOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * For a break operation, proposed duration the lease should continue before it is broken, in
     * seconds, between 0 and 60. This break period is only used if it is shorter than the time
     * remaining on the lease. If longer, the time remaining on the lease is used. A new lease will
     * not be available before the break period has expired, but the lease may be held for longer
     * than the break period. If this header does not appear with a break operation, a fixed-duration
     * lease breaks after the remaining lease period elapses, and an infinite lease breaks
     * immediately.
     */
    breakPeriod?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlobCreateSnapshotOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Optional. Specifies a user-defined name-value pair associated with the blob. If no name-value
     * pairs are specified, the operation will copy the metadata from the source blob or file to the
     * destination blob. If one or more name-value pairs are specified, the destination blob is
     * created with the specified metadata, and metadata is not copied from the source blob or file.
     * Note that beginning with version 2009-09-19, metadata names must adhere to the naming rules
     * for C# identifiers. See Naming and Referencing Containers, Blobs, and Metadata for more
     * information.
     */
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlobStartCopyFromURLOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Optional. Specifies a user-defined name-value pair associated with the blob. If no name-value
     * pairs are specified, the operation will copy the metadata from the source blob or file to the
     * destination blob. If one or more name-value pairs are specified, the destination blob is
     * created with the specified metadata, and metadata is not copied from the source blob or file.
     * Note that beginning with version 2009-09-19, metadata names must adhere to the naming rules
     * for C# identifiers. See Naming and Referencing Containers, Blobs, and Metadata for more
     * information.
     */
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    sourceModifiedAccessConditions?: SourceModifiedAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlobAbortCopyFromURLOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlobSetTierOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
}
/**
 * Optional Parameters.
 */
export interface PageBlobCreateOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Optional. Specifies a user-defined name-value pair associated with the blob. If no name-value
     * pairs are specified, the operation will copy the metadata from the source blob or file to the
     * destination blob. If one or more name-value pairs are specified, the destination blob is
     * created with the specified metadata, and metadata is not copied from the source blob or file.
     * Note that beginning with version 2009-09-19, metadata names must adhere to the naming rules
     * for C# identifiers. See Naming and Referencing Containers, Blobs, and Metadata for more
     * information.
     */
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * Set for page blobs only. The sequence number is a user-controlled value that you can use to
     * track requests. The value of the sequence number must be between 0 and 2^63 - 1. Default
     * value: 0.
     */
    blobSequenceNumber?: number;
    /**
     * Set for page blobs only. For page blobs on a premium storage account only. Specifies the tier
     * to be set on the blob. Possible values include: 'P4', 'P6', 'P10', 'P15', 'P20', 'P30', 'P40',
     * 'P50', 'P60', 'P70', 'P80'
     */
    pageBlobAccessTier?: PageBlobAccessTier;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    blobHTTPHeaders?: BlobHTTPHeaders;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface PageBlobUploadPagesOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Return only the bytes of the blob in the specified range.
     */
    range?: string;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    sequenceNumberAccessConditions?: SequenceNumberAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface PageBlobClearPagesOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Return only the bytes of the blob in the specified range.
     */
    range?: string;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    sequenceNumberAccessConditions?: SequenceNumberAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface PageBlobGetPageRangesOptionalParams {
    /**
     * The snapshot parameter is an opaque DateTime value that, when present, specifies the blob
     * snapshot to retrieve. For more information on working with blob snapshots, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/creating-a-snapshot-of-a-blob">Creating
     * a Snapshot of a Blob.</a>
     */
    snapshot?: string;
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Return only the bytes of the blob in the specified range.
     */
    range?: string;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface PageBlobGetPageRangesDiffOptionalParams {
    /**
     * The snapshot parameter is an opaque DateTime value that, when present, specifies the blob
     * snapshot to retrieve. For more information on working with blob snapshots, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/creating-a-snapshot-of-a-blob">Creating
     * a Snapshot of a Blob.</a>
     */
    snapshot?: string;
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Optional in version 2015-07-08 and newer. The prevsnapshot parameter is a DateTime value that
     * specifies that the response will contain only pages that were changed between target blob and
     * previous snapshot. Changed pages include both updated and cleared pages. The target blob may
     * be a snapshot, as long as the snapshot specified by prevsnapshot is the older of the two. Note
     * that incremental snapshots are currently supported only for blobs created on or after January
     * 1, 2016.
     */
    prevsnapshot?: string;
    /**
     * Return only the bytes of the blob in the specified range.
     */
    range?: string;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface PageBlobResizeOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface PageBlobUpdateSequenceNumberOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Set for page blobs only. The sequence number is a user-controlled value that you can use to
     * track requests. The value of the sequence number must be between 0 and 2^63 - 1. Default
     * value: 0.
     */
    blobSequenceNumber?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface PageBlobCopyIncrementalOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Optional. Specifies a user-defined name-value pair associated with the blob. If no name-value
     * pairs are specified, the operation will copy the metadata from the source blob or file to the
     * destination blob. If one or more name-value pairs are specified, the destination blob is
     * created with the specified metadata, and metadata is not copied from the source blob or file.
     * Note that beginning with version 2009-09-19, metadata names must adhere to the naming rules
     * for C# identifiers. See Naming and Referencing Containers, Blobs, and Metadata for more
     * information.
     */
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface AppendBlobCreateOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Optional. Specifies a user-defined name-value pair associated with the blob. If no name-value
     * pairs are specified, the operation will copy the metadata from the source blob or file to the
     * destination blob. If one or more name-value pairs are specified, the destination blob is
     * created with the specified metadata, and metadata is not copied from the source blob or file.
     * Note that beginning with version 2009-09-19, metadata names must adhere to the naming rules
     * for C# identifiers. See Naming and Referencing Containers, Blobs, and Metadata for more
     * information.
     */
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    blobHTTPHeaders?: BlobHTTPHeaders;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface AppendBlobAppendBlockOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    appendPositionAccessConditions?: AppendPositionAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlockBlobUploadOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Optional. Specifies a user-defined name-value pair associated with the blob. If no name-value
     * pairs are specified, the operation will copy the metadata from the source blob or file to the
     * destination blob. If one or more name-value pairs are specified, the destination blob is
     * created with the specified metadata, and metadata is not copied from the source blob or file.
     * Note that beginning with version 2009-09-19, metadata names must adhere to the naming rules
     * for C# identifiers. See Naming and Referencing Containers, Blobs, and Metadata for more
     * information.
     */
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    blobHTTPHeaders?: BlobHTTPHeaders;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlockBlobStageBlockOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlockBlobStageBlockFromURLOptionalParams {
    /**
     * Bytes of source data in the specified range.
     */
    sourceRange?: string;
    /**
     * Specify the md5 calculated for the range of bytes that must be read from the copy source.
     */
    sourceContentMD5?: Uint8Array;
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlockBlobCommitBlockListOptionalParams {
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Optional. Specifies a user-defined name-value pair associated with the blob. If no name-value
     * pairs are specified, the operation will copy the metadata from the source blob or file to the
     * destination blob. If one or more name-value pairs are specified, the destination blob is
     * created with the specified metadata, and metadata is not copied from the source blob or file.
     * Note that beginning with version 2009-09-19, metadata names must adhere to the naming rules
     * for C# identifiers. See Naming and Referencing Containers, Blobs, and Metadata for more
     * information.
     */
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    blobHTTPHeaders?: BlobHTTPHeaders;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
    /**
     * Additional parameters for the operation
     */
    modifiedAccessConditions?: ModifiedAccessConditions;
}
/**
 * Optional Parameters.
 */
export interface BlockBlobGetBlockListOptionalParams {
    /**
     * The snapshot parameter is an opaque DateTime value that, when present, specifies the blob
     * snapshot to retrieve. For more information on working with blob snapshots, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/creating-a-snapshot-of-a-blob">Creating
     * a Snapshot of a Blob.</a>
     */
    snapshot?: string;
    /**
     * Specifies whether to return the list of committed blocks, the list of uncommitted blocks, or
     * both lists together. Possible values include: 'committed', 'uncommitted', 'all'. Default
     * value: 'committed'.
     */
    listType?: BlockListType;
    /**
     * The timeout parameter is expressed in seconds. For more information, see <a
     * href="https://docs.microsoft.com/en-us/rest/api/storageservices/fileservices/setting-timeouts-for-blob-service-operations">Setting
     * Timeouts for Blob Service Operations.</a>
     */
    timeout?: number;
    /**
     * Provides a client-generated, opaque value with a 1 KB character limit that is recorded in the
     * analytics logs when storage analytics logging is enabled.
     */
    requestId?: string;
    /**
     * Additional parameters for the operation
     */
    leaseAccessConditions?: LeaseAccessConditions;
}
/**
 * Defines headers for SetProperties operation.
 */
export interface ServiceSetPropertiesHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    errorCode?: string;
}
/**
 * Defines headers for GetProperties operation.
 */
export interface ServiceGetPropertiesHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    errorCode?: string;
}
/**
 * Defines headers for GetStatistics operation.
 */
export interface ServiceGetStatisticsHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for ListContainersSegment operation.
 */
export interface ServiceListContainersSegmentHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    errorCode?: string;
}
/**
 * Defines headers for GetAccountInfo operation.
 */
export interface ServiceGetAccountInfoHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * Identifies the sku name of the account. Possible values include: 'Standard_LRS',
     * 'Standard_GRS', 'Standard_RAGRS', 'Standard_ZRS', 'Premium_LRS'
     */
    skuName?: SkuName;
    /**
     * Identifies the account kind. Possible values include: 'Storage', 'BlobStorage', 'StorageV2'
     */
    accountKind?: AccountKind;
    errorCode?: string;
}
/**
 * Defines headers for GetAccountInfoWithHead operation.
 */
export interface ServiceGetAccountInfoWithHeadHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * Identifies the sku name of the account. Possible values include: 'Standard_LRS',
     * 'Standard_GRS', 'Standard_RAGRS', 'Standard_ZRS', 'Premium_LRS'
     */
    skuName?: SkuName;
    /**
     * Identifies the account kind. Possible values include: 'Storage', 'BlobStorage', 'StorageV2'
     */
    accountKind?: AccountKind;
    errorCode?: string;
}
/**
 * Defines headers for Create operation.
 */
export interface ContainerCreateHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for GetProperties operation.
 */
export interface ContainerGetPropertiesHeaders {
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * When a blob is leased, specifies whether the lease is of infinite or fixed duration. Possible
     * values include: 'infinite', 'fixed'
     */
    leaseDuration?: LeaseDurationType;
    /**
     * Lease state of the blob. Possible values include: 'available', 'leased', 'expired',
     * 'breaking', 'broken'
     */
    leaseState?: LeaseStateType;
    /**
     * The current lease status of the blob. Possible values include: 'locked', 'unlocked'
     */
    leaseStatus?: LeaseStatusType;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * Indicated whether data in the container may be accessed publicly and the level of access.
     * Possible values include: 'container', 'blob'
     */
    blobPublicAccess?: PublicAccessType;
    /**
     * Indicates whether the container has an immutability policy set on it.
     */
    hasImmutabilityPolicy?: boolean;
    /**
     * Indicates whether the container has a legal hold.
     */
    hasLegalHold?: boolean;
    errorCode?: string;
}
/**
 * Defines headers for GetPropertiesWithHead operation.
 */
export interface ContainerGetPropertiesWithHeadHeaders {
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * When a blob is leased, specifies whether the lease is of infinite or fixed duration. Possible
     * values include: 'infinite', 'fixed'
     */
    leaseDuration?: LeaseDurationType;
    /**
     * Lease state of the blob. Possible values include: 'available', 'leased', 'expired',
     * 'breaking', 'broken'
     */
    leaseState?: LeaseStateType;
    /**
     * The current lease status of the blob. Possible values include: 'locked', 'unlocked'
     */
    leaseStatus?: LeaseStatusType;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * Indicated whether data in the container may be accessed publicly and the level of access.
     * Possible values include: 'container', 'blob'
     */
    blobPublicAccess?: PublicAccessType;
    /**
     * Indicates whether the container has an immutability policy set on it.
     */
    hasImmutabilityPolicy?: boolean;
    /**
     * Indicates whether the container has a legal hold.
     */
    hasLegalHold?: boolean;
    errorCode?: string;
}
/**
 * Defines headers for Delete operation.
 */
export interface ContainerDeleteHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for SetMetadata operation.
 */
export interface ContainerSetMetadataHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for GetAccessPolicy operation.
 */
export interface ContainerGetAccessPolicyHeaders {
    /**
     * Indicated whether data in the container may be accessed publicly and the level of access.
     * Possible values include: 'container', 'blob'
     */
    blobPublicAccess?: PublicAccessType;
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for SetAccessPolicy operation.
 */
export interface ContainerSetAccessPolicyHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for AcquireLease operation.
 */
export interface ContainerAcquireLeaseHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * Uniquely identifies a container's lease
     */
    leaseId?: string;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for ReleaseLease operation.
 */
export interface ContainerReleaseLeaseHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for RenewLease operation.
 */
export interface ContainerRenewLeaseHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * Uniquely identifies a container's lease
     */
    leaseId?: string;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for BreakLease operation.
 */
export interface ContainerBreakLeaseHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * Approximate time remaining in the lease period, in seconds.
     */
    leaseTime?: number;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for ChangeLease operation.
 */
export interface ContainerChangeLeaseHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * Uniquely identifies a container's lease
     */
    leaseId?: string;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for ListBlobFlatSegment operation.
 */
export interface ContainerListBlobFlatSegmentHeaders {
    /**
     * The content type specified for the blob. The default content type is
     * 'application/octet-stream'
     */
    contentType?: string;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for ListBlobHierarchySegment operation.
 */
export interface ContainerListBlobHierarchySegmentHeaders {
    /**
     * The content type specified for the blob. The default content type is
     * 'application/octet-stream'
     */
    contentType?: string;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for GetAccountInfo operation.
 */
export interface ContainerGetAccountInfoHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * Identifies the sku name of the account. Possible values include: 'Standard_LRS',
     * 'Standard_GRS', 'Standard_RAGRS', 'Standard_ZRS', 'Premium_LRS'
     */
    skuName?: SkuName;
    /**
     * Identifies the account kind. Possible values include: 'Storage', 'BlobStorage', 'StorageV2'
     */
    accountKind?: AccountKind;
    errorCode?: string;
}
/**
 * Defines headers for GetAccountInfoWithHead operation.
 */
export interface ContainerGetAccountInfoWithHeadHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * Identifies the sku name of the account. Possible values include: 'Standard_LRS',
     * 'Standard_GRS', 'Standard_RAGRS', 'Standard_ZRS', 'Premium_LRS'
     */
    skuName?: SkuName;
    /**
     * Identifies the account kind. Possible values include: 'Storage', 'BlobStorage', 'StorageV2'
     */
    accountKind?: AccountKind;
    errorCode?: string;
}
/**
 * Defines headers for Download operation.
 */
export interface BlobDownloadHeaders {
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * The number of bytes present in the response body.
     */
    contentLength?: number;
    /**
     * The content type specified for the blob. The default content type is
     * 'application/octet-stream'
     */
    contentType?: string;
    /**
     * Indicates the range of bytes returned in the event that the client requested a subset of the
     * blob by setting the 'Range' request header.
     */
    contentRange?: string;
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * If the blob has an MD5 hash and this operation is to read the full blob, this response header
     * is returned so that the client can check for message content integrity.
     */
    contentMD5?: Uint8Array;
    /**
     * This header returns the value that was specified for the Content-Encoding request header
     */
    contentEncoding?: string;
    /**
     * This header is returned if it was previously specified for the blob.
     */
    cacheControl?: string;
    /**
     * This header returns the value that was specified for the 'x-ms-blob-content-disposition'
     * header. The Content-Disposition response header field conveys additional information about how
     * to process the response payload, and also can be used to attach additional metadata. For
     * example, if set to attachment, it indicates that the user-agent should not display the
     * response, but instead show a Save As dialog with a filename other than the blob name
     * specified.
     */
    contentDisposition?: string;
    /**
     * This header returns the value that was specified for the Content-Language request header.
     */
    contentLanguage?: string;
    /**
     * The current sequence number for a page blob. This header is not returned for block blobs or
     * append blobs
     */
    blobSequenceNumber?: number;
    /**
     * The blob's type. Possible values include: 'BlockBlob', 'PageBlob', 'AppendBlob'
     */
    blobType?: BlobType;
    /**
     * Conclusion time of the last attempted Copy Blob operation where this blob was the destination
     * blob. This value can specify the time of a completed, aborted, or failed copy attempt. This
     * header does not appear if a copy is pending, if this blob has never been the destination in a
     * Copy Blob operation, or if this blob has been modified after a concluded Copy Blob operation
     * using Set Blob Properties, Put Blob, or Put Block List.
     */
    copyCompletionTime?: Date;
    /**
     * Only appears when x-ms-copy-status is failed or pending. Describes the cause of the last fatal
     * or non-fatal copy operation failure. This header does not appear if this blob has never been
     * the destination in a Copy Blob operation, or if this blob has been modified after a concluded
     * Copy Blob operation using Set Blob Properties, Put Blob, or Put Block List
     */
    copyStatusDescription?: string;
    /**
     * String identifier for this copy operation. Use with Get Blob Properties to check the status of
     * this copy operation, or pass to Abort Copy Blob to abort a pending copy.
     */
    copyId?: string;
    /**
     * Contains the number of bytes copied and the total bytes in the source in the last attempted
     * Copy Blob operation where this blob was the destination blob. Can show between 0 and
     * Content-Length bytes copied. This header does not appear if this blob has never been the
     * destination in a Copy Blob operation, or if this blob has been modified after a concluded Copy
     * Blob operation using Set Blob Properties, Put Blob, or Put Block List
     */
    copyProgress?: string;
    /**
     * URL up to 2 KB in length that specifies the source blob or file used in the last attempted
     * Copy Blob operation where this blob was the destination blob. This header does not appear if
     * this blob has never been the destination in a Copy Blob operation, or if this blob has been
     * modified after a concluded Copy Blob operation using Set Blob Properties, Put Blob, or Put
     * Block List.
     */
    copySource?: string;
    /**
     * State of the copy operation identified by x-ms-copy-id. Possible values include: 'pending',
     * 'success', 'aborted', 'failed'
     */
    copyStatus?: CopyStatusType;
    /**
     * When a blob is leased, specifies whether the lease is of infinite or fixed duration. Possible
     * values include: 'infinite', 'fixed'
     */
    leaseDuration?: LeaseDurationType;
    /**
     * Lease state of the blob. Possible values include: 'available', 'leased', 'expired',
     * 'breaking', 'broken'
     */
    leaseState?: LeaseStateType;
    /**
     * The current lease status of the blob. Possible values include: 'locked', 'unlocked'
     */
    leaseStatus?: LeaseStatusType;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * Indicates that the service supports requests for partial blob content.
     */
    acceptRanges?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * The number of committed blocks present in the blob. This header is returned only for append
     * blobs.
     */
    blobCommittedBlockCount?: number;
    /**
     * The value of this header is set to true if the blob data and application metadata are
     * completely encrypted using the specified algorithm. Otherwise, the value is set to false (when
     * the blob is unencrypted, or if only parts of the blob/application metadata are encrypted).
     */
    isServerEncrypted?: boolean;
    /**
     * If the blob has a MD5 hash, and if request contains range header (Range or x-ms-range), this
     * response header is returned with the value of the whole blob's MD5 value. This value may or
     * may not be equal to the value returned in Content-MD5 header, with the latter calculated from
     * the requested range
     */
    blobContentMD5?: Uint8Array;
    errorCode?: string;
}
/**
 * Defines headers for GetProperties operation.
 */
export interface BlobGetPropertiesHeaders {
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * Returns the date and time the blob was created.
     */
    creationTime?: Date;
    metadata?: {
        [propertyName: string]: string;
    };
    /**
     * The blob's type. Possible values include: 'BlockBlob', 'PageBlob', 'AppendBlob'
     */
    blobType?: BlobType;
    /**
     * Conclusion time of the last attempted Copy Blob operation where this blob was the destination
     * blob. This value can specify the time of a completed, aborted, or failed copy attempt. This
     * header does not appear if a copy is pending, if this blob has never been the destination in a
     * Copy Blob operation, or if this blob has been modified after a concluded Copy Blob operation
     * using Set Blob Properties, Put Blob, or Put Block List.
     */
    copyCompletionTime?: Date;
    /**
     * Only appears when x-ms-copy-status is failed or pending. Describes the cause of the last fatal
     * or non-fatal copy operation failure. This header does not appear if this blob has never been
     * the destination in a Copy Blob operation, or if this blob has been modified after a concluded
     * Copy Blob operation using Set Blob Properties, Put Blob, or Put Block List
     */
    copyStatusDescription?: string;
    /**
     * String identifier for this copy operation. Use with Get Blob Properties to check the status of
     * this copy operation, or pass to Abort Copy Blob to abort a pending copy.
     */
    copyId?: string;
    /**
     * Contains the number of bytes copied and the total bytes in the source in the last attempted
     * Copy Blob operation where this blob was the destination blob. Can show between 0 and
     * Content-Length bytes copied. This header does not appear if this blob has never been the
     * destination in a Copy Blob operation, or if this blob has been modified after a concluded Copy
     * Blob operation using Set Blob Properties, Put Blob, or Put Block List
     */
    copyProgress?: string;
    /**
     * URL up to 2 KB in length that specifies the source blob or file used in the last attempted
     * Copy Blob operation where this blob was the destination blob. This header does not appear if
     * this blob has never been the destination in a Copy Blob operation, or if this blob has been
     * modified after a concluded Copy Blob operation using Set Blob Properties, Put Blob, or Put
     * Block List.
     */
    copySource?: string;
    /**
     * State of the copy operation identified by x-ms-copy-id. Possible values include: 'pending',
     * 'success', 'aborted', 'failed'
     */
    copyStatus?: CopyStatusType;
    /**
     * Included if the blob is incremental copy blob.
     */
    isIncrementalCopy?: boolean;
    /**
     * Included if the blob is incremental copy blob or incremental copy snapshot, if
     * x-ms-copy-status is success. Snapshot time of the last successful incremental copy snapshot
     * for this blob.
     */
    destinationSnapshot?: string;
    /**
     * When a blob is leased, specifies whether the lease is of infinite or fixed duration. Possible
     * values include: 'infinite', 'fixed'
     */
    leaseDuration?: LeaseDurationType;
    /**
     * Lease state of the blob. Possible values include: 'available', 'leased', 'expired',
     * 'breaking', 'broken'
     */
    leaseState?: LeaseStateType;
    /**
     * The current lease status of the blob. Possible values include: 'locked', 'unlocked'
     */
    leaseStatus?: LeaseStatusType;
    /**
     * The number of bytes present in the response body.
     */
    contentLength?: number;
    /**
     * The content type specified for the blob. The default content type is
     * 'application/octet-stream'
     */
    contentType?: string;
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * If the blob has an MD5 hash and this operation is to read the full blob, this response header
     * is returned so that the client can check for message content integrity.
     */
    contentMD5?: Uint8Array;
    /**
     * This header returns the value that was specified for the Content-Encoding request header
     */
    contentEncoding?: string;
    /**
     * This header returns the value that was specified for the 'x-ms-blob-content-disposition'
     * header. The Content-Disposition response header field conveys additional information about how
     * to process the response payload, and also can be used to attach additional metadata. For
     * example, if set to attachment, it indicates that the user-agent should not display the
     * response, but instead show a Save As dialog with a filename other than the blob name
     * specified.
     */
    contentDisposition?: string;
    /**
     * This header returns the value that was specified for the Content-Language request header.
     */
    contentLanguage?: string;
    /**
     * This header is returned if it was previously specified for the blob.
     */
    cacheControl?: string;
    /**
     * The current sequence number for a page blob. This header is not returned for block blobs or
     * append blobs
     */
    blobSequenceNumber?: number;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * Indicates that the service supports requests for partial blob content.
     */
    acceptRanges?: string;
    /**
     * The number of committed blocks present in the blob. This header is returned only for append
     * blobs.
     */
    blobCommittedBlockCount?: number;
    /**
     * The value of this header is set to true if the blob data and application metadata are
     * completely encrypted using the specified algorithm. Otherwise, the value is set to false (when
     * the blob is unencrypted, or if only parts of the blob/application metadata are encrypted).
     */
    isServerEncrypted?: boolean;
    /**
     * The tier of page blob on a premium storage account or tier of block blob on blob storage LRS
     * accounts. For a list of allowed premium page blob tiers, see
     * https://docs.microsoft.com/en-us/azure/virtual-machines/windows/premium-storage#features. For
     * blob storage LRS accounts, valid values are Hot/Cool/Archive.
     */
    accessTier?: string;
    /**
     * For page blobs on a premium storage account only. If the access tier is not explicitly set on
     * the blob, the tier is inferred based on its content length and this header will be returned
     * with true value.
     */
    accessTierInferred?: boolean;
    /**
     * For blob storage LRS accounts, valid values are
     * rehydrate-pending-to-hot/rehydrate-pending-to-cool. If the blob is being rehydrated and is not
     * complete then this header is returned indicating that rehydrate is pending and also tells the
     * destination tier.
     */
    archiveStatus?: string;
    /**
     * The time the tier was changed on the object. This is only returned if the tier on the block
     * blob was ever set.
     */
    accessTierChangeTime?: Date;
    errorCode?: string;
}
/**
 * Defines headers for Delete operation.
 */
export interface BlobDeleteHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for Create operation.
 */
export interface PageBlobCreateHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * If the blob has an MD5 hash and this operation is to read the full blob, this response header
     * is returned so that the client can check for message content integrity.
     */
    contentMD5?: Uint8Array;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * The value of this header is set to true if the contents of the request are successfully
     * encrypted using the specified algorithm, and false otherwise.
     */
    isServerEncrypted?: boolean;
    errorCode?: string;
}
/**
 * Defines headers for Create operation.
 */
export interface AppendBlobCreateHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * If the blob has an MD5 hash and this operation is to read the full blob, this response header
     * is returned so that the client can check for message content integrity.
     */
    contentMD5?: Uint8Array;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * The value of this header is set to true if the contents of the request are successfully
     * encrypted using the specified algorithm, and false otherwise.
     */
    isServerEncrypted?: boolean;
    errorCode?: string;
}
/**
 * Defines headers for Upload operation.
 */
export interface BlockBlobUploadHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * If the blob has an MD5 hash and this operation is to read the full blob, this response header
     * is returned so that the client can check for message content integrity.
     */
    contentMD5?: Uint8Array;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * The value of this header is set to true if the contents of the request are successfully
     * encrypted using the specified algorithm, and false otherwise.
     */
    isServerEncrypted?: boolean;
    errorCode?: string;
}
/**
 * Defines headers for Undelete operation.
 */
export interface BlobUndeleteHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated.
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for SetHTTPHeaders operation.
 */
export interface BlobSetHTTPHeadersHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * The current sequence number for a page blob. This header is not returned for block blobs or
     * append blobs
     */
    blobSequenceNumber?: number;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for SetMetadata operation.
 */
export interface BlobSetMetadataHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * The value of this header is set to true if the contents of the request are successfully
     * encrypted using the specified algorithm, and false otherwise.
     */
    isServerEncrypted?: boolean;
    errorCode?: string;
}
/**
 * Defines headers for AcquireLease operation.
 */
export interface BlobAcquireLeaseHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the blob was last modified. Any operation that modifies the blob,
     * including an update of the blob's metadata or properties, changes the last-modified time of
     * the blob.
     */
    lastModified?: Date;
    /**
     * Uniquely identifies a blobs's lease
     */
    leaseId?: string;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for ReleaseLease operation.
 */
export interface BlobReleaseLeaseHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the blob was last modified. Any operation that modifies the blob,
     * including an update of the blob's metadata or properties, changes the last-modified time of
     * the blob.
     */
    lastModified?: Date;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for RenewLease operation.
 */
export interface BlobRenewLeaseHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the blob was last modified. Any operation that modifies the blob,
     * including an update of the blob's metadata or properties, changes the last-modified time of
     * the blob.
     */
    lastModified?: Date;
    /**
     * Uniquely identifies a blobs's lease
     */
    leaseId?: string;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for ChangeLease operation.
 */
export interface BlobChangeLeaseHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the blob was last modified. Any operation that modifies the blob,
     * including an update of the blob's metadata or properties, changes the last-modified time of
     * the blob.
     */
    lastModified?: Date;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Uniquely identifies a blobs's lease
     */
    leaseId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for BreakLease operation.
 */
export interface BlobBreakLeaseHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the blob was last modified. Any operation that modifies the blob,
     * including an update of the blob's metadata or properties, changes the last-modified time of
     * the blob.
     */
    lastModified?: Date;
    /**
     * Approximate time remaining in the lease period, in seconds.
     */
    leaseTime?: number;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for CreateSnapshot operation.
 */
export interface BlobCreateSnapshotHeaders {
    /**
     * Uniquely identifies the snapshot and indicates the snapshot version. It may be used in
     * subsequent requests to access the snapshot
     */
    snapshot?: string;
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for StartCopyFromURL operation.
 */
export interface BlobStartCopyFromURLHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * String identifier for this copy operation. Use with Get Blob Properties to check the status of
     * this copy operation, or pass to Abort Copy Blob to abort a pending copy.
     */
    copyId?: string;
    /**
     * State of the copy operation identified by x-ms-copy-id. Possible values include: 'pending',
     * 'success', 'aborted', 'failed'
     */
    copyStatus?: CopyStatusType;
    errorCode?: string;
}
/**
 * Defines headers for AbortCopyFromURL operation.
 */
export interface BlobAbortCopyFromURLHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for SetTier operation.
 */
export interface BlobSetTierHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and newer.
     */
    version?: string;
    errorCode?: string;
}
/**
 * Defines headers for GetAccountInfo operation.
 */
export interface BlobGetAccountInfoHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * Identifies the sku name of the account. Possible values include: 'Standard_LRS',
     * 'Standard_GRS', 'Standard_RAGRS', 'Standard_ZRS', 'Premium_LRS'
     */
    skuName?: SkuName;
    /**
     * Identifies the account kind. Possible values include: 'Storage', 'BlobStorage', 'StorageV2'
     */
    accountKind?: AccountKind;
    errorCode?: string;
}
/**
 * Defines headers for GetAccountInfoWithHead operation.
 */
export interface BlobGetAccountInfoWithHeadHeaders {
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * Identifies the sku name of the account. Possible values include: 'Standard_LRS',
     * 'Standard_GRS', 'Standard_RAGRS', 'Standard_ZRS', 'Premium_LRS'
     */
    skuName?: SkuName;
    /**
     * Identifies the account kind. Possible values include: 'Storage', 'BlobStorage', 'StorageV2'
     */
    accountKind?: AccountKind;
    errorCode?: string;
}
/**
 * Defines headers for StageBlock operation.
 */
export interface BlockBlobStageBlockHeaders {
    /**
     * If the blob has an MD5 hash and this operation is to read the full blob, this response header
     * is returned so that the client can check for message content integrity.
     */
    contentMD5?: Uint8Array;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * The value of this header is set to true if the contents of the request are successfully
     * encrypted using the specified algorithm, and false otherwise.
     */
    isServerEncrypted?: boolean;
    errorCode?: string;
}
/**
 * Defines headers for StageBlockFromURL operation.
 */
export interface BlockBlobStageBlockFromURLHeaders {
    /**
     * If the blob has an MD5 hash and this operation is to read the full blob, this response header
     * is returned so that the client can check for message content integrity.
     */
    contentMD5?: Uint8Array;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * The value of this header is set to true if the contents of the request are successfully
     * encrypted using the specified algorithm, and false otherwise.
     */
    isServerEncrypted?: boolean;
    errorCode?: string;
}
/**
 * Defines headers for CommitBlockList operation.
 */
export interface BlockBlobCommitBlockListHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * If the blob has an MD5 hash and this operation is to read the full blob, this response header
     * is returned so that the client can check for message content integrity.
     */
    contentMD5?: Uint8Array;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * The value of this header is set to true if the contents of the request are successfully
     * encrypted using the specified algorithm, and false otherwise.
     */
    isServerEncrypted?: boolean;
    errorCode?: string;
}
/**
 * Defines headers for GetBlockList operation.
 */
export interface BlockBlobGetBlockListHeaders {
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * The content type specified for the blob. The default content type is
     * 'application/octet-stream'
     */
    contentType?: string;
    /**
     * The size of the blob in bytes.
     */
    blobContentLength?: number;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for UploadPages operation.
 */
export interface PageBlobUploadPagesHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * If the blob has an MD5 hash and this operation is to read the full blob, this response header
     * is returned so that the client can check for message content integrity.
     */
    contentMD5?: Uint8Array;
    /**
     * The current sequence number for the page blob.
     */
    blobSequenceNumber?: number;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * The value of this header is set to true if the contents of the request are successfully
     * encrypted using the specified algorithm, and false otherwise.
     */
    isServerEncrypted?: boolean;
    errorCode?: string;
}
/**
 * Defines headers for ClearPages operation.
 */
export interface PageBlobClearPagesHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * If the blob has an MD5 hash and this operation is to read the full blob, this response header
     * is returned so that the client can check for message content integrity.
     */
    contentMD5?: Uint8Array;
    /**
     * The current sequence number for the page blob.
     */
    blobSequenceNumber?: number;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for GetPageRanges operation.
 */
export interface PageBlobGetPageRangesHeaders {
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * The size of the blob in bytes.
     */
    blobContentLength?: number;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for GetPageRangesDiff operation.
 */
export interface PageBlobGetPageRangesDiffHeaders {
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * The size of the blob in bytes.
     */
    blobContentLength?: number;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for Resize operation.
 */
export interface PageBlobResizeHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * The current sequence number for a page blob. This header is not returned for block blobs or
     * append blobs
     */
    blobSequenceNumber?: number;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for UpdateSequenceNumber operation.
 */
export interface PageBlobUpdateSequenceNumberHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * The current sequence number for a page blob. This header is not returned for block blobs or
     * append blobs
     */
    blobSequenceNumber?: number;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    errorCode?: string;
}
/**
 * Defines headers for CopyIncremental operation.
 */
export interface PageBlobCopyIncrementalHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * String identifier for this copy operation. Use with Get Blob Properties to check the status of
     * this copy operation, or pass to Abort Copy Blob to abort a pending copy.
     */
    copyId?: string;
    /**
     * State of the copy operation identified by x-ms-copy-id. Possible values include: 'pending',
     * 'success', 'aborted', 'failed'
     */
    copyStatus?: CopyStatusType;
    errorCode?: string;
}
/**
 * Defines headers for AppendBlock operation.
 */
export interface AppendBlobAppendBlockHeaders {
    /**
     * The ETag contains a value that you can use to perform operations conditionally. If the request
     * version is 2011-08-18 or newer, the ETag value will be in quotes.
     */
    eTag?: string;
    /**
     * Returns the date and time the container was last modified. Any operation that modifies the
     * blob, including an update of the blob's metadata or properties, changes the last-modified time
     * of the blob.
     */
    lastModified?: Date;
    /**
     * If the blob has an MD5 hash and this operation is to read the full blob, this response header
     * is returned so that the client can check for message content integrity.
     */
    contentMD5?: Uint8Array;
    /**
     * This header uniquely identifies the request that was made and can be used for troubleshooting
     * the request.
     */
    requestId?: string;
    /**
     * Indicates the version of the Blob service used to execute the request. This header is returned
     * for requests made against version 2009-09-19 and above.
     */
    version?: string;
    /**
     * UTC date/time value generated by the service that indicates the time at which the response was
     * initiated
     */
    date?: Date;
    /**
     * This response header is returned only for append operations. It returns the offset at which
     * the block was committed, in bytes.
     */
    blobAppendOffset?: string;
    /**
     * The number of committed blocks present in the blob. This header is returned only for append
     * blobs.
     */
    blobCommittedBlockCount?: number;
    errorCode?: string;
}
/**
 * Defines values for PublicAccessType.
 * Possible values include: 'container', 'blob'
 * @readonly
 * @enum {string}
 */
export declare enum PublicAccessType {
    Container = "container",
    Blob = "blob"
}
/**
 * Defines values for CopyStatusType.
 * Possible values include: 'pending', 'success', 'aborted', 'failed'
 * @readonly
 * @enum {string}
 */
export declare enum CopyStatusType {
    Pending = "pending",
    Success = "success",
    Aborted = "aborted",
    Failed = "failed"
}
/**
 * Defines values for LeaseDurationType.
 * Possible values include: 'infinite', 'fixed'
 * @readonly
 * @enum {string}
 */
export declare enum LeaseDurationType {
    Infinite = "infinite",
    Fixed = "fixed"
}
/**
 * Defines values for LeaseStateType.
 * Possible values include: 'available', 'leased', 'expired', 'breaking', 'broken'
 * @readonly
 * @enum {string}
 */
export declare enum LeaseStateType {
    Available = "available",
    Leased = "leased",
    Expired = "expired",
    Breaking = "breaking",
    Broken = "broken"
}
/**
 * Defines values for LeaseStatusType.
 * Possible values include: 'locked', 'unlocked'
 * @readonly
 * @enum {string}
 */
export declare enum LeaseStatusType {
    Locked = "locked",
    Unlocked = "unlocked"
}
/**
 * Defines values for AccessTier.
 * Possible values include: 'P4', 'P6', 'P10', 'P15', 'P20', 'P30', 'P40', 'P50', 'P60', 'P70',
 * 'P80', 'Hot', 'Cool', 'Archive'
 * @readonly
 * @enum {string}
 */
export declare enum AccessTier {
    P4 = "P4",
    P6 = "P6",
    P10 = "P10",
    P15 = "P15",
    P20 = "P20",
    P30 = "P30",
    P40 = "P40",
    P50 = "P50",
    P60 = "P60",
    P70 = "P70",
    P80 = "P80",
    Hot = "Hot",
    Cool = "Cool",
    Archive = "Archive"
}
/**
 * Defines values for ArchiveStatus.
 * Possible values include: 'rehydrate-pending-to-hot', 'rehydrate-pending-to-cool'
 * @readonly
 * @enum {string}
 */
export declare enum ArchiveStatus {
    RehydratePendingToHot = "rehydrate-pending-to-hot",
    RehydratePendingToCool = "rehydrate-pending-to-cool"
}
/**
 * Defines values for BlobType.
 * Possible values include: 'BlockBlob', 'PageBlob', 'AppendBlob'
 * @readonly
 * @enum {string}
 */
export declare enum BlobType {
    BlockBlob = "BlockBlob",
    PageBlob = "PageBlob",
    AppendBlob = "AppendBlob"
}
/**
 * Defines values for StorageErrorCode.
 * Possible values include: 'AccountAlreadyExists', 'AccountBeingCreated', 'AccountIsDisabled',
 * 'AuthenticationFailed', 'ConditionHeadersNotSupported', 'ConditionNotMet', 'EmptyMetadataKey',
 * 'InsufficientAccountPermissions', 'InternalError', 'InvalidAuthenticationInfo',
 * 'InvalidHeaderValue', 'InvalidHttpVerb', 'InvalidInput', 'InvalidMd5', 'InvalidMetadata',
 * 'InvalidQueryParameterValue', 'InvalidRange', 'InvalidResourceName', 'InvalidUri',
 * 'InvalidXmlDocument', 'InvalidXmlNodeValue', 'Md5Mismatch', 'MetadataTooLarge',
 * 'MissingContentLengthHeader', 'MissingRequiredQueryParameter', 'MissingRequiredHeader',
 * 'MissingRequiredXmlNode', 'MultipleConditionHeadersNotSupported', 'OperationTimedOut',
 * 'OutOfRangeInput', 'OutOfRangeQueryParameterValue', 'RequestBodyTooLarge',
 * 'ResourceTypeMismatch', 'RequestUrlFailedToParse', 'ResourceAlreadyExists', 'ResourceNotFound',
 * 'ServerBusy', 'UnsupportedHeader', 'UnsupportedXmlNode', 'UnsupportedQueryParameter',
 * 'UnsupportedHttpVerb', 'AppendPositionConditionNotMet', 'BlobAlreadyExists', 'BlobNotFound',
 * 'BlobOverwritten', 'BlobTierInadequateForContentLength', 'BlockCountExceedsLimit',
 * 'BlockListTooLong', 'CannotChangeToLowerTier', 'CannotVerifyCopySource',
 * 'ContainerAlreadyExists', 'ContainerBeingDeleted', 'ContainerDisabled', 'ContainerNotFound',
 * 'ContentLengthLargerThanTierLimit', 'CopyAcrossAccountsNotSupported', 'CopyIdMismatch',
 * 'FeatureVersionMismatch', 'IncrementalCopyBlobMismatch',
 * 'IncrementalCopyOfEralierVersionSnapshotNotAllowed', 'IncrementalCopySourceMustBeSnapshot',
 * 'InfiniteLeaseDurationRequired', 'InvalidBlobOrBlock', 'InvalidBlobTier', 'InvalidBlobType',
 * 'InvalidBlockId', 'InvalidBlockList', 'InvalidOperation', 'InvalidPageRange',
 * 'InvalidSourceBlobType', 'InvalidSourceBlobUrl', 'InvalidVersionForPageBlobOperation',
 * 'LeaseAlreadyPresent', 'LeaseAlreadyBroken', 'LeaseIdMismatchWithBlobOperation',
 * 'LeaseIdMismatchWithContainerOperation', 'LeaseIdMismatchWithLeaseOperation', 'LeaseIdMissing',
 * 'LeaseIsBreakingAndCannotBeAcquired', 'LeaseIsBreakingAndCannotBeChanged',
 * 'LeaseIsBrokenAndCannotBeRenewed', 'LeaseLost', 'LeaseNotPresentWithBlobOperation',
 * 'LeaseNotPresentWithContainerOperation', 'LeaseNotPresentWithLeaseOperation',
 * 'MaxBlobSizeConditionNotMet', 'NoPendingCopyOperation',
 * 'OperationNotAllowedOnIncrementalCopyBlob', 'PendingCopyOperation',
 * 'PreviousSnapshotCannotBeNewer', 'PreviousSnapshotNotFound',
 * 'PreviousSnapshotOperationNotSupported', 'SequenceNumberConditionNotMet',
 * 'SequenceNumberIncrementTooLarge', 'SnapshotCountExceeded', 'SnaphotOperationRateExceeded',
 * 'SnapshotsPresent', 'SourceConditionNotMet', 'SystemInUse', 'TargetConditionNotMet',
 * 'UnauthorizedBlobOverwrite', 'BlobBeingRehydrated', 'BlobArchived', 'BlobNotArchived'
 * @readonly
 * @enum {string}
 */
export declare enum StorageErrorCode {
    AccountAlreadyExists = "AccountAlreadyExists",
    AccountBeingCreated = "AccountBeingCreated",
    AccountIsDisabled = "AccountIsDisabled",
    AuthenticationFailed = "AuthenticationFailed",
    ConditionHeadersNotSupported = "ConditionHeadersNotSupported",
    ConditionNotMet = "ConditionNotMet",
    EmptyMetadataKey = "EmptyMetadataKey",
    InsufficientAccountPermissions = "InsufficientAccountPermissions",
    InternalError = "InternalError",
    InvalidAuthenticationInfo = "InvalidAuthenticationInfo",
    InvalidHeaderValue = "InvalidHeaderValue",
    InvalidHttpVerb = "InvalidHttpVerb",
    InvalidInput = "InvalidInput",
    InvalidMd5 = "InvalidMd5",
    InvalidMetadata = "InvalidMetadata",
    InvalidQueryParameterValue = "InvalidQueryParameterValue",
    InvalidRange = "InvalidRange",
    InvalidResourceName = "InvalidResourceName",
    InvalidUri = "InvalidUri",
    InvalidXmlDocument = "InvalidXmlDocument",
    InvalidXmlNodeValue = "InvalidXmlNodeValue",
    Md5Mismatch = "Md5Mismatch",
    MetadataTooLarge = "MetadataTooLarge",
    MissingContentLengthHeader = "MissingContentLengthHeader",
    MissingRequiredQueryParameter = "MissingRequiredQueryParameter",
    MissingRequiredHeader = "MissingRequiredHeader",
    MissingRequiredXmlNode = "MissingRequiredXmlNode",
    MultipleConditionHeadersNotSupported = "MultipleConditionHeadersNotSupported",
    OperationTimedOut = "OperationTimedOut",
    OutOfRangeInput = "OutOfRangeInput",
    OutOfRangeQueryParameterValue = "OutOfRangeQueryParameterValue",
    RequestBodyTooLarge = "RequestBodyTooLarge",
    ResourceTypeMismatch = "ResourceTypeMismatch",
    RequestUrlFailedToParse = "RequestUrlFailedToParse",
    ResourceAlreadyExists = "ResourceAlreadyExists",
    ResourceNotFound = "ResourceNotFound",
    ServerBusy = "ServerBusy",
    UnsupportedHeader = "UnsupportedHeader",
    UnsupportedXmlNode = "UnsupportedXmlNode",
    UnsupportedQueryParameter = "UnsupportedQueryParameter",
    UnsupportedHttpVerb = "UnsupportedHttpVerb",
    AppendPositionConditionNotMet = "AppendPositionConditionNotMet",
    BlobAlreadyExists = "BlobAlreadyExists",
    BlobNotFound = "BlobNotFound",
    BlobOverwritten = "BlobOverwritten",
    BlobTierInadequateForContentLength = "BlobTierInadequateForContentLength",
    BlockCountExceedsLimit = "BlockCountExceedsLimit",
    BlockListTooLong = "BlockListTooLong",
    CannotChangeToLowerTier = "CannotChangeToLowerTier",
    CannotVerifyCopySource = "CannotVerifyCopySource",
    ContainerAlreadyExists = "ContainerAlreadyExists",
    ContainerBeingDeleted = "ContainerBeingDeleted",
    ContainerDisabled = "ContainerDisabled",
    ContainerNotFound = "ContainerNotFound",
    ContentLengthLargerThanTierLimit = "ContentLengthLargerThanTierLimit",
    CopyAcrossAccountsNotSupported = "CopyAcrossAccountsNotSupported",
    CopyIdMismatch = "CopyIdMismatch",
    FeatureVersionMismatch = "FeatureVersionMismatch",
    IncrementalCopyBlobMismatch = "IncrementalCopyBlobMismatch",
    IncrementalCopyOfEralierVersionSnapshotNotAllowed = "IncrementalCopyOfEralierVersionSnapshotNotAllowed",
    IncrementalCopySourceMustBeSnapshot = "IncrementalCopySourceMustBeSnapshot",
    InfiniteLeaseDurationRequired = "InfiniteLeaseDurationRequired",
    InvalidBlobOrBlock = "InvalidBlobOrBlock",
    InvalidBlobTier = "InvalidBlobTier",
    InvalidBlobType = "InvalidBlobType",
    InvalidBlockId = "InvalidBlockId",
    InvalidBlockList = "InvalidBlockList",
    InvalidOperation = "InvalidOperation",
    InvalidPageRange = "InvalidPageRange",
    InvalidSourceBlobType = "InvalidSourceBlobType",
    InvalidSourceBlobUrl = "InvalidSourceBlobUrl",
    InvalidVersionForPageBlobOperation = "InvalidVersionForPageBlobOperation",
    LeaseAlreadyPresent = "LeaseAlreadyPresent",
    LeaseAlreadyBroken = "LeaseAlreadyBroken",
    LeaseIdMismatchWithBlobOperation = "LeaseIdMismatchWithBlobOperation",
    LeaseIdMismatchWithContainerOperation = "LeaseIdMismatchWithContainerOperation",
    LeaseIdMismatchWithLeaseOperation = "LeaseIdMismatchWithLeaseOperation",
    LeaseIdMissing = "LeaseIdMissing",
    LeaseIsBreakingAndCannotBeAcquired = "LeaseIsBreakingAndCannotBeAcquired",
    LeaseIsBreakingAndCannotBeChanged = "LeaseIsBreakingAndCannotBeChanged",
    LeaseIsBrokenAndCannotBeRenewed = "LeaseIsBrokenAndCannotBeRenewed",
    LeaseLost = "LeaseLost",
    LeaseNotPresentWithBlobOperation = "LeaseNotPresentWithBlobOperation",
    LeaseNotPresentWithContainerOperation = "LeaseNotPresentWithContainerOperation",
    LeaseNotPresentWithLeaseOperation = "LeaseNotPresentWithLeaseOperation",
    MaxBlobSizeConditionNotMet = "MaxBlobSizeConditionNotMet",
    NoPendingCopyOperation = "NoPendingCopyOperation",
    OperationNotAllowedOnIncrementalCopyBlob = "OperationNotAllowedOnIncrementalCopyBlob",
    PendingCopyOperation = "PendingCopyOperation",
    PreviousSnapshotCannotBeNewer = "PreviousSnapshotCannotBeNewer",
    PreviousSnapshotNotFound = "PreviousSnapshotNotFound",
    PreviousSnapshotOperationNotSupported = "PreviousSnapshotOperationNotSupported",
    SequenceNumberConditionNotMet = "SequenceNumberConditionNotMet",
    SequenceNumberIncrementTooLarge = "SequenceNumberIncrementTooLarge",
    SnapshotCountExceeded = "SnapshotCountExceeded",
    SnaphotOperationRateExceeded = "SnaphotOperationRateExceeded",
    SnapshotsPresent = "SnapshotsPresent",
    SourceConditionNotMet = "SourceConditionNotMet",
    SystemInUse = "SystemInUse",
    TargetConditionNotMet = "TargetConditionNotMet",
    UnauthorizedBlobOverwrite = "UnauthorizedBlobOverwrite",
    BlobBeingRehydrated = "BlobBeingRehydrated",
    BlobArchived = "BlobArchived",
    BlobNotArchived = "BlobNotArchived"
}
/**
 * Defines values for GeoReplicationStatusType.
 * Possible values include: 'live', 'bootstrap', 'unavailable'
 * @readonly
 * @enum {string}
 */
export declare enum GeoReplicationStatusType {
    Live = "live",
    Bootstrap = "bootstrap",
    Unavailable = "unavailable"
}
/**
 * Defines values for PageBlobAccessTier.
 * Possible values include: 'P4', 'P6', 'P10', 'P15', 'P20', 'P30', 'P40', 'P50', 'P60', 'P70',
 * 'P80'
 * @readonly
 * @enum {string}
 */
export declare enum PageBlobAccessTier {
    P4 = "P4",
    P6 = "P6",
    P10 = "P10",
    P15 = "P15",
    P20 = "P20",
    P30 = "P30",
    P40 = "P40",
    P50 = "P50",
    P60 = "P60",
    P70 = "P70",
    P80 = "P80"
}
/**
 * Defines values for BlockListType.
 * Possible values include: 'committed', 'uncommitted', 'all'
 * @readonly
 * @enum {string}
 */
export declare enum BlockListType {
    Committed = "committed",
    Uncommitted = "uncommitted",
    All = "all"
}
/**
 * Defines values for DeleteSnapshotsOptionType.
 * Possible values include: 'include', 'only'
 * @readonly
 * @enum {string}
 */
export declare enum DeleteSnapshotsOptionType {
    Include = "include",
    Only = "only"
}
/**
 * Defines values for ListBlobsIncludeItem.
 * Possible values include: 'snapshots', 'metadata', 'uncommittedblobs', 'copy', 'deleted'
 * @readonly
 * @enum {string}
 */
export declare enum ListBlobsIncludeItem {
    Snapshots = "snapshots",
    Metadata = "metadata",
    Uncommittedblobs = "uncommittedblobs",
    Copy = "copy",
    Deleted = "deleted"
}
/**
 * Defines values for ListContainersIncludeType.
 * Possible values include: 'metadata'
 * @readonly
 * @enum {string}
 */
export declare enum ListContainersIncludeType {
    Metadata = "metadata"
}
/**
 * Defines values for SequenceNumberActionType.
 * Possible values include: 'max', 'update', 'increment'
 * @readonly
 * @enum {string}
 */
export declare enum SequenceNumberActionType {
    Max = "max",
    Update = "update",
    Increment = "increment"
}
/**
 * Defines values for SkuName.
 * Possible values include: 'Standard_LRS', 'Standard_GRS', 'Standard_RAGRS', 'Standard_ZRS',
 * 'Premium_LRS'
 * @readonly
 * @enum {string}
 */
export declare enum SkuName {
    StandardLRS = "Standard_LRS",
    StandardGRS = "Standard_GRS",
    StandardRAGRS = "Standard_RAGRS",
    StandardZRS = "Standard_ZRS",
    PremiumLRS = "Premium_LRS"
}
/**
 * Defines values for AccountKind.
 * Possible values include: 'Storage', 'BlobStorage', 'StorageV2'
 * @readonly
 * @enum {string}
 */
export declare enum AccountKind {
    Storage = "Storage",
    BlobStorage = "BlobStorage",
    StorageV2 = "StorageV2"
}
/**
 * Contains response data for the setProperties operation.
 */
export declare type ServiceSetPropertiesResponse = ServiceSetPropertiesHeaders & {
    /**
     * The response status code.
     */
    statusCode: 202;
};
/**
 * Contains response data for the getProperties operation.
 */
export declare type ServiceGetPropertiesResponse = StorageServiceProperties & ServiceGetPropertiesHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the getStatistics operation.
 */
export declare type ServiceGetStatisticsResponse = StorageServiceStats & ServiceGetStatisticsHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the listContainersSegment operation.
 */
export declare type ServiceListContainersSegmentResponse = ListContainersSegmentResponse & ServiceListContainersSegmentHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the getAccountInfo operation.
 */
export declare type ServiceGetAccountInfoResponse = ServiceGetAccountInfoHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the getAccountInfoWithHead operation.
 */
export declare type ServiceGetAccountInfoWithHeadResponse = ServiceGetAccountInfoWithHeadHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the create operation.
 */
export declare type ContainerCreateResponse = ContainerCreateHeaders & {
    /**
     * The response status code.
     */
    statusCode: 201;
};
/**
 * Contains response data for the getProperties operation.
 */
export declare type ContainerGetPropertiesResponse = ContainerGetPropertiesHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the getPropertiesWithHead operation.
 */
export declare type ContainerGetPropertiesWithHeadResponse = ContainerGetPropertiesWithHeadHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the deleteMethod operation.
 */
export declare type ContainerDeleteResponse = ContainerDeleteHeaders & {
    /**
     * The response status code.
     */
    statusCode: 202;
};
/**
 * Contains response data for the setMetadata operation.
 */
export declare type ContainerSetMetadataResponse = ContainerSetMetadataHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the getAccessPolicy operation.
 */
export declare type ContainerGetAccessPolicyResponse = Array<SignedIdentifier> & ContainerGetAccessPolicyHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the setAccessPolicy operation.
 */
export declare type ContainerSetAccessPolicyResponse = ContainerSetAccessPolicyHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the acquireLease operation.
 */
export declare type ContainerAcquireLeaseResponse = ContainerAcquireLeaseHeaders & {
    /**
     * The response status code.
     */
    statusCode: 201;
};
/**
 * Contains response data for the releaseLease operation.
 */
export declare type ContainerReleaseLeaseResponse = ContainerReleaseLeaseHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the renewLease operation.
 */
export declare type ContainerRenewLeaseResponse = ContainerRenewLeaseHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the breakLease operation.
 */
export declare type ContainerBreakLeaseResponse = ContainerBreakLeaseHeaders & {
    /**
     * The response status code.
     */
    statusCode: 202;
};
/**
 * Contains response data for the changeLease operation.
 */
export declare type ContainerChangeLeaseResponse = ContainerChangeLeaseHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the listBlobFlatSegment operation.
 */
export declare type ContainerListBlobFlatSegmentResponse = ListBlobsFlatSegmentResponse & ContainerListBlobFlatSegmentHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the listBlobHierarchySegment operation.
 */
export declare type ContainerListBlobHierarchySegmentResponse = ListBlobsHierarchySegmentResponse & ContainerListBlobHierarchySegmentHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the getAccountInfo operation.
 */
export declare type ContainerGetAccountInfoResponse = ContainerGetAccountInfoHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the getAccountInfoWithHead operation.
 */
export declare type ContainerGetAccountInfoWithHeadResponse = ContainerGetAccountInfoWithHeadHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the download operation.
 */
export declare type BlobDownloadResponse = BlobDownloadHeaders & {
    /**
     * The response body as a node.js Readable stream.
     */
    body?: NodeJS.ReadableStream;
} & {
    /**
     * The response status code.
     */
    statusCode: 200 | 206;
};
/**
 * Contains response data for the getProperties operation.
 */
export declare type BlobGetPropertiesResponse = BlobGetPropertiesHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the deleteMethod operation.
 */
export declare type BlobDeleteResponse = BlobDeleteHeaders & {
    /**
     * The response status code.
     */
    statusCode: 202;
};
/**
 * Contains response data for the undelete operation.
 */
export declare type BlobUndeleteResponse = BlobUndeleteHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the setHTTPHeaders operation.
 */
export declare type BlobSetHTTPHeadersResponse = BlobSetHTTPHeadersHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the setMetadata operation.
 */
export declare type BlobSetMetadataResponse = BlobSetMetadataHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the acquireLease operation.
 */
export declare type BlobAcquireLeaseResponse = BlobAcquireLeaseHeaders & {
    /**
     * The response status code.
     */
    statusCode: 201;
};
/**
 * Contains response data for the releaseLease operation.
 */
export declare type BlobReleaseLeaseResponse = BlobReleaseLeaseHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the renewLease operation.
 */
export declare type BlobRenewLeaseResponse = BlobRenewLeaseHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the changeLease operation.
 */
export declare type BlobChangeLeaseResponse = BlobChangeLeaseHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the breakLease operation.
 */
export declare type BlobBreakLeaseResponse = BlobBreakLeaseHeaders & {
    /**
     * The response status code.
     */
    statusCode: 202;
};
/**
 * Contains response data for the createSnapshot operation.
 */
export declare type BlobCreateSnapshotResponse = BlobCreateSnapshotHeaders & {
    /**
     * The response status code.
     */
    statusCode: 201;
};
/**
 * Contains response data for the startCopyFromURL operation.
 */
export declare type BlobStartCopyFromURLResponse = BlobStartCopyFromURLHeaders & {
    /**
     * The response status code.
     */
    statusCode: 202;
};
/**
 * Contains response data for the abortCopyFromURL operation.
 */
export declare type BlobAbortCopyFromURLResponse = BlobAbortCopyFromURLHeaders & {
    /**
     * The response status code.
     */
    statusCode: 204;
};
/**
 * Contains response data for the setTier operation.
 */
export declare type BlobSetTierResponse = BlobSetTierHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200 | 202;
};
/**
 * Contains response data for the getAccountInfo operation.
 */
export declare type BlobGetAccountInfoResponse = BlobGetAccountInfoHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the getAccountInfoWithHead operation.
 */
export declare type BlobGetAccountInfoWithHeadResponse = BlobGetAccountInfoWithHeadHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the create operation.
 */
export declare type PageBlobCreateResponse = PageBlobCreateHeaders & {
    /**
     * The response status code.
     */
    statusCode: 201;
};
/**
 * Contains response data for the uploadPages operation.
 */
export declare type PageBlobUploadPagesResponse = PageBlobUploadPagesHeaders & {
    /**
     * The response status code.
     */
    statusCode: 201;
};
/**
 * Contains response data for the clearPages operation.
 */
export declare type PageBlobClearPagesResponse = PageBlobClearPagesHeaders & {
    /**
     * The response status code.
     */
    statusCode: 201;
};
/**
 * Contains response data for the getPageRanges operation.
 */
export declare type PageBlobGetPageRangesResponse = PageList & PageBlobGetPageRangesHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the getPageRangesDiff operation.
 */
export declare type PageBlobGetPageRangesDiffResponse = PageList & PageBlobGetPageRangesDiffHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the resize operation.
 */
export declare type PageBlobResizeResponse = PageBlobResizeHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the updateSequenceNumber operation.
 */
export declare type PageBlobUpdateSequenceNumberResponse = PageBlobUpdateSequenceNumberHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
/**
 * Contains response data for the copyIncremental operation.
 */
export declare type PageBlobCopyIncrementalResponse = PageBlobCopyIncrementalHeaders & {
    /**
     * The response status code.
     */
    statusCode: 202;
};
/**
 * Contains response data for the create operation.
 */
export declare type AppendBlobCreateResponse = AppendBlobCreateHeaders & {
    /**
     * The response status code.
     */
    statusCode: 201;
};
/**
 * Contains response data for the appendBlock operation.
 */
export declare type AppendBlobAppendBlockResponse = AppendBlobAppendBlockHeaders & {
    /**
     * The response status code.
     */
    statusCode: 201;
};
/**
 * Contains response data for the upload operation.
 */
export declare type BlockBlobUploadResponse = BlockBlobUploadHeaders & {
    /**
     * The response status code.
     */
    statusCode: 201;
};
/**
 * Contains response data for the stageBlock operation.
 */
export declare type BlockBlobStageBlockResponse = BlockBlobStageBlockHeaders & {
    /**
     * The response status code.
     */
    statusCode: 201;
};
/**
 * Contains response data for the stageBlockFromURL operation.
 */
export declare type BlockBlobStageBlockFromURLResponse = BlockBlobStageBlockFromURLHeaders & {
    /**
     * The response status code.
     */
    statusCode: 201;
};
/**
 * Contains response data for the commitBlockList operation.
 */
export declare type BlockBlobCommitBlockListResponse = BlockBlobCommitBlockListHeaders & {
    /**
     * The response status code.
     */
    statusCode: 201;
};
/**
 * Contains response data for the getBlockList operation.
 */
export declare type BlockBlobGetBlockListResponse = BlockList & BlockBlobGetBlockListHeaders & {
    /**
     * The response status code.
     */
    statusCode: 200;
};
//# sourceMappingURL=models.d.ts.map