import { BlobModel, BlockModel, ContainerModel, IBlobDataStore, IPersistencyChunk, ServicePropertiesModel } from "./IBlobDataStore";
/**
 * Maintains mapping relationship between extent ID and relative local file path/name.
 *
 * @interface IExtentModel
 */
interface IExtentModel {
    /**
     * Extent ID.
     *
     * @type {string}
     * @memberof IExtentModel
     */
    id: string;
    /**
     * Relative local file path/name.
     *
     * @type {string}
     * @memberof IExtentModel
     */
    path: string;
}
/**
 * This is a persistency layer data source implementation based on loki DB.
 *
 * Notice that, following design is for emulator purpose only, and doesn't design for best performance.
 * We may want to optimize the persistency layer performance in the future. Such as by distributing metadata
 * into different collections, or make binary payload write as an append-only pattern.
 *
 * Loki DB includes following collections and documents:
 *
 * -- SERVICE_PROPERTIES_COLLECTION // Collection contains service properties
 *                                  // Default collection name is $SERVICES_COLLECTION$
 *                                  // Each document maps to 1 account blob service
 *                                  // Unique document properties: accountName
 * -- CONTAINERS_COLLECTION  // Collection contains all containers
 *                           // Default collection name is $CONTAINERS_COLLECTION$
 *                           // Each document maps to 1 container
 *                           // Unique document properties: accountName, (container)name
 * -- BLOBS_COLLECTION       // Collection contains all blobs
 *                           // Default collection name is $BLOBS_COLLECTION$
 *                           // Each document maps to a blob
 *                           // Unique document properties: accountName, containerName, (blob)name, snapshot
 * -- BLOCKS_COLLECTION      // Block blob blocks collection includes all UNCOMMITTED blocks
 *                           // Unique document properties: accountName, containerName, blobName, name, isCommitted
 * -- EXTENTS_COLLECTION     // Collections maintain extents information, including extentID, mapped local file path
 *                           // Unique document properties: id, path
 *
 * @export
 * @class LokiBlobDataStore
 */
export default class LokiBlobDataStore implements IBlobDataStore {
    readonly lokiDBPath: string;
    readonly persistencePath: string;
    private readonly db;
    private initialized;
    private closed;
    private readonly SERVICES_COLLECTION;
    private readonly CONTAINERS_COLLECTION;
    private readonly BLOBS_COLLECTION;
    private readonly BLOCKS_COLLECTION;
    private readonly EXTENTS_COLLECTION;
    private readonly FDCache;
    constructor(lokiDBPath: string, persistencePath: string);
    isInitialized(): boolean;
    isClosed(): boolean;
    init(): Promise<void>;
    /**
     * Close loki DB.
     *
     * @returns {Promise<void>}
     * @memberof LokiBlobDataStore
     */
    close(): Promise<void>;
    /**
     * Update blob service properties. Create service properties document if not exists in DB.
     * Assume service properties collection has been created.
     *
     * @template T
     * @param {T} serviceProperties
     * @returns {Promise<T>}
     * @memberof LokiBlobDataStore
     */
    updateServiceProperties<T extends ServicePropertiesModel>(serviceProperties: T): Promise<T>;
    /**
     * Get service properties.
     *
     * @param {string} account
     * @template T
     * @returns {Promise<T | undefined>}
     * @memberof LokiBlobDataStore
     */
    getServiceProperties<T extends ServicePropertiesModel>(account: string): Promise<T | undefined>;
    /**
     * Get a container item from persistency layer by account and container name.
     *
     * @template T
     * @param {string} account
     * @param {string} container
     * @returns {(Promise<T | undefined>)}
     * @memberof LokiBlobDataStore
     */
    getContainer<T extends ContainerModel>(account: string, container: string): Promise<T | undefined>;
    /**
     * Delete container item if exists from persistency layer.
     * Note that this method will only remove container related document from persistency layer.
     * Make sure blobs under the container has been properly handled before calling this method.
     *
     * @param {string} account
     * @param {string} container
     * @returns {Promise<void>}
     * @memberof LokiBlobDataStore
     */
    deleteContainer(account: string, container: string): Promise<void>;
    /**
     * Update a container item in persistency layer. If the container doesn't exist, it will be created.
     *
     * @template T
     * @param {T} container
     * @returns {Promise<T>}
     * @memberof LokiBlobDataStore
     */
    updateContainer<T extends ContainerModel>(container: T): Promise<T>;
    /**
     * List containers with query conditions specified.
     *
     * @template T
     * @param {string} account
     * @param {string} [prefix]
     * @param {number} [maxResults=2000]
     * @param {number} [marker]
     * @returns {(Promise<[T[], number | undefined]>)} A tuple including containers and next marker
     * @memberof LokiBlobDataStore
     */
    listContainers<T extends ContainerModel>(account: string, prefix?: string, maxResults?: number, marker?: number): Promise<[T[], number | undefined]>;
    /**
     * Update blob item in persistency layer. Will create if blob doesn't exist.
     *
     * @template T
     * @param {T} blob
     * @returns {Promise<T>}
     * @memberof LokiBlobDataStore
     */
    updateBlob<T extends BlobModel>(blob: T): Promise<T>;
    /**
     * Gets a blob item from persistency layer by container name and blob name.
     *
     * @template T
     * @param {string} account
     * @param {string} container
     * @param {string} blob
     * @param {string} [snapshot=""]
     * @returns {(Promise<T | undefined>)}
     * @memberof LokiBlobDataStore
     */
    getBlob<T extends BlobModel>(account: string, container: string, blob: string, snapshot?: string): Promise<T | undefined>;
    /**
     * List blobs with query conditions specified.
     *
     * @template T
     * @param {string} [account]
     * @param {string} [container]
     * @param {string} [blob] If blob name provided, prefix will be ignored.
     * @param {string} [prefix]
     * @param {number} [maxResults=5000]
     * @param {number} [marker]
     * @returns {(Promise<[T[], number | undefined]>)} A tuple including list blobs and next marker.
     * @memberof LokiBlobDataStore
     */
    listBlobs<T extends BlobModel>(account?: string, container?: string, blob?: string, prefix?: string | undefined, maxResults?: number | undefined, marker?: number | undefined, includeSnapshots?: boolean | undefined): Promise<[T[], number | undefined]>;
    insertExtent(id: string, path: string): Promise<void>;
    /**
     * Get extent row record from collection. ExtentID or extent mapped relative file path
     * must be provided at least one.
     *
     * @param {string} [id]
     * @param {string} [path]
     * @returns {(Promise<IExtentModel | undefined>)}
     * @memberof LokiBlobDataStore
     */
    getExtent(id?: string, path?: string): Promise<IExtentModel | undefined>;
    /**
     * List extents segmented.
     *
     * @param {string} [id]
     * @param {string} [path]
     * @param {(number | undefined)} [maxResults=5000]
     * @param {(number | undefined)} [marker]
     * @returns {(Promise<[IExtentModel[], number | undefined]>)}
     * @memberof LokiBlobDataStore
     */
    listExtents(id?: string, path?: string, maxResults?: number | undefined, marker?: number | undefined): Promise<[IExtentModel[], number | undefined]>;
    /**
     * Delete an extent row in collection.
     *
     * @param {string} id
     * @returns {Promise<void>}
     * @memberof LokiBlobDataStore
     */
    deleteExtent(id: string): Promise<void>;
    /**
     * Delete blob item from persistency layer.
     *
     * @param {string} account
     * @param {string} container
     * @param {string} blob
     * @param {string} [snapshot=""]
     * @returns {Promise<void>}
     * @memberof LokiBlobDataStore
     */
    deleteBlob(account: string, container: string, blob: string, snapshot?: string): Promise<void>;
    deleteBlobs(account: string, container: string): Promise<void>;
    /**
     * Update block in DB. Will create if block doesn't exist.
     *
     * @template T
     * @param {T} block
     * @returns {Promise<T>}
     * @memberof LokiBlobDataStore
     */
    updateBlock<T extends BlockModel>(block: T): Promise<T>;
    /**
     * Delete all blocks for a blob in persistency layer.
     *
     * @param {string} account
     * @param {string} container
     * @param {string} blob
     * @returns {Promise<void>}
     * @memberof LokiBlobDataStore
     */
    deleteBlocks(account: string, container: string, blob: string): Promise<void>;
    /**
     * Insert blocks for a blob in persistency layer. Existing blocks with same name will be replaced.
     *
     * @template T
     * @param {T[]} blocks
     * @returns {Promise<T[]>}
     * @memberof LokiBlobDataStore
     */
    insertBlocks<T extends BlockModel>(blocks: T[]): Promise<T[]>;
    /**
     * Gets block for a blob from persistency layer by account, container, blob and block names.
     *
     * @template T
     * @param {string} account
     * @param {string} container
     * @param {string} blob
     * @param {string} block
     * @param {boolean} isCommitted
     * @returns {Promise<T | undefined>}
     * @memberof LokiBlobDataStore
     */
    getBlock<T extends BlockModel>(account: string, container: string, blob: string, block: string, isCommitted: boolean): Promise<T | undefined>;
    /**
     * Gets blocks list for a blob from persistency layer by account, container and blob names.
     *
     * @template T
     * @param {string} [account]
     * @param {string} [container]
     * @param {string} [blob]
     * @param {boolean} [isCommitted]
     * @returns {(Promise<T[]>)}
     * @memberof LokiBlobDataStore
     */
    listBlocks<T extends BlockModel>(account?: string, container?: string, blob?: string, isCommitted?: boolean): Promise<T[]>;
    /**
     * Persist payload and return a persistency chunk for tracking.
     *
     * @param {NodeJS.ReadableStream | Buffer} payload
     * @returns {Promise<IPersistencyChunk>} Returns the unique persistency chunk
     * @memberof LokiBlobDataStore
     */
    writePayload(payload: NodeJS.ReadableStream | Buffer): Promise<IPersistencyChunk>;
    /**
     * Reads a subset of persistency layer chunk for a given persistency chunk model.
     *
     * @param {IPersistencyChunk} [persistency] Optional. A chunk model pointing to a persistency chunk.
     *                                          Pass undefined will return an empty readable stream without any data.
     *                                          Pass ZERO_PERSISTENCY_CHUNK_ID as persistency.id will return a
     *                                          stream with zero bytes
     * @param {number} [offset] Optional. Payload reads chunk (not extent) offset. Default is 0
     * @param {number} [count] Optional. Payload reads count. Default is Infinity
     * @returns {Promise<NodeJS.ReadableStream>}
     * @memberof LokiBlobDataStore
     */
    readPayload(persistency?: IPersistencyChunk, offset?: number, count?: number): Promise<NodeJS.ReadableStream>;
    /**
     * Merge persistency payloads into a single payload and return a ReadableStream
     * from the merged stream according to the offset and count.
     *
     * @param {(IPersistencyChunk)[]} persistencyArray Persistency chunk model list
     * @param {number} [offset] Optional. Reads offset from the merged persistency (sub)chunks. Default is 0
     * @param {number} [count] Optional. Reads count from the merged persistency (sub)chunks. Default is Infinity
     * @returns {Promise<NodeJS.ReadableStream>}
     * @memberof LokiBlobDataStore
     */
    readPayloads(persistencyArray: (IPersistencyChunk)[], offset?: number, count?: number): Promise<NodeJS.ReadableStream>;
    /**
     * Remove payloads from persistency layer.
     *
     * @param {Iterable<string | IPersistencyChunk>} persistency
     * @returns {Promise<void>}
     * @memberof LokiBlobDataStore
     */
    deletePayloads(persistency: Iterable<string | IPersistencyChunk>): Promise<void>;
    /**
     * Create an async iterator to enumerate all extent IDs.
     *
     * @returns {AsyncIterator<string[]>}
     * @memberof IBlobDataStore
     */
    iteratorAllExtents(): AsyncIterator<string[]>;
    /**
     * Create an async iterator to enumerate all extent records referred or being used.
     *
     * @returns {AsyncIterator<IPersistencyChunk[]>}
     * @memberof IBlobDataStore
     */
    iteratorReferredExtents(): AsyncIterator<IPersistencyChunk[]>;
    /**
     * Allocate a new extent ID.
     *
     * @private
     * @returns {string}
     * @memberof LokiBlobDataStore
     */
    private generateExtentID;
    /**
     * Generate a file name/path relative to persistencePath for a given extent ID.
     *
     * @private
     * @param {string} id Extent ID
     * @returns {string}
     * @memberof LokiBlobDataStore
     */
    private generateRelativeFilePath;
    /**
     * Get a full path for a give relative file path.
     * The full path could be used to open, read and write files.
     *
     * @private
     * @param {string} relativePath
     * @returns {string}
     * @memberof LokiBlobDataStore
     */
    private getPersistencyPath;
    /**
     * Get persistency file path for an created extent.
     * The full path could be used to open, read and write files.
     *
     * @private
     * @param {(string | IPersistencyChunk)} persistency Extent ID or chunk
     * @returns {Promise<string>}
     * @memberof LokiBlobDataStore
     */
    private getExtentPersistencyPath;
    /**
     * Get size for a given persistency (sub)chunk.
     *
     * @private
     * @param {IPersistencyChunk } persistency
     * @returns {Promise<number>}
     * @memberof LokiBlobDataStore
     */
    private getPersistencySize;
    /**
     * LokiJS will persist Uint8Array into Object.
     * This method will restore object to Uint8Array.
     *
     * @private
     * @param {*} obj
     * @returns {(Uint8Array | undefined)}
     * @memberof LokiBlobDataStore
     */
    private restoreUint8Array;
    /**
     * Escape a string to be used as a regex.
     *
     * @private
     * @param {string} regex
     * @returns {string}
     * @memberof LokiBlobDataStore
     */
    private escapeRegex;
}
export {};
//# sourceMappingURL=LokiBlobDataStore.d.ts.map